@@ -1,12 +1,16 @@
+import os.path
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import textwrap
 import os
 import glob
-from calibre.customize import FileTypePlugin, MetadataReaderPlugin, MetadataWriterPlugin
+from calibre.customize import FileTypePlugin, MetadataReaderPlugin, \
+    MetadataWriterPlugin, PreferencesPlugin, InterfaceActionBase
 from calibre.constants import numeric_version
+from calibre.ebooks.metadata.archive import ArchiveExtract, get_cbz_metadata
 
+# To archive plugins {{{
 class HTML2ZIP(FileTypePlugin):
     name = 'HTML to ZIP'
     author = 'Kovid Goyal'
@@ -28,6 +32,7 @@ every time you add an HTML file to the library.\
 
         with TemporaryDirectory('_plugin_html2zip') as tdir:
             recs =[('debug_pipeline', tdir, OptionRecommendation.HIGH)]
+            recs.append(['keep_ligatures', True, OptionRecommendation.HIGH])
             if self.site_customization and self.site_customization.strip():
                 recs.append(['input_encoding', self.site_customization.strip(),
                     OptionRecommendation.HIGH])
@@ -49,6 +54,39 @@ every time you add an HTML file to the library.\
         'include: cp1252, latin1, iso-8859-1 and utf-8.')
 
 
+class PML2PMLZ(FileTypePlugin):
+    name = 'PML to PMLZ'
+    author = 'John Schember'
+    description = _('Create a PMLZ archive containing the PML file '
+        'and all images in the directory pmlname_img or images. '
+        'This plugin is run every time you add '
+        'a PML file to the library.')
+    version = numeric_version
+    file_types = set(['pml'])
+    supported_platforms = ['windows', 'osx', 'linux']
+    on_import = True
+
+    def run(self, pmlfile):
+        import zipfile
+
+        of = self.temporary_file('_plugin_pml2pmlz.pmlz')
+        pmlz = zipfile.ZipFile(of.name, 'w')
+        pmlz.write(pmlfile, os.path.basename(pmlfile), zipfile.ZIP_DEFLATED)
+
+        pml_img = os.path.splitext(pmlfile)[0] + '_img'
+        i_img = os.path.join(os.path.dirname(pmlfile),'images')
+        img_dir = pml_img if os.path.isdir(pml_img) else i_img if \
+            os.path.isdir(i_img) else ''
+        if img_dir:
+            for image in glob.glob(os.path.join(img_dir, '*.png')):
+                pmlz.write(image, os.path.join('images', (os.path.basename(image))))
+        pmlz.close()
+
+        return of.name
+
+# }}}
+
+# Metadata reader plugins {{{
 class ComicMetadataReader(MetadataReaderPlugin):
 
     name = 'Read comic metadata'
@@ -64,12 +102,29 @@ class ComicMetadataReader(MetadataReaderPlugin):
         from calibre.ebooks.metadata import MetaInformation
         ret = extract_first(stream)
         mi = MetaInformation(None, None)
+        stream.seek(0)
+        if ftype == 'cbz':
+            try:
+                mi.smart_update(get_cbz_metadata(stream))
+            except:
+                pass
         if ret is not None:
             path, data = ret
             ext = os.path.splitext(path)[1][1:]
             mi.cover_data = (ext.lower(), data)
         return mi
 
+class CHMMetadataReader(MetadataReaderPlugin):
+
+    name        = 'Read CHM metadata'
+    file_types  = set(['chm'])
+    description = _('Read metadata from %s files') % 'CHM'
+
+    def get_metadata(self, stream, ftype):
+        from calibre.ebooks.chm.metadata import get_metadata
+        return get_metadata(stream)
+
+
 class EPUBMetadataReader(MetadataReaderPlugin):
 
     name        = 'Read EPUB metadata'
@@ -197,6 +252,17 @@ class PDFMetadataReader(MetadataReaderPlugin):
             return get_quick_metadata(stream)
         return get_metadata(stream)
 
+class PMLMetadataReader(MetadataReaderPlugin):
+
+    name        = 'Read PML metadata'
+    file_types  = set(['pml', 'pmlz'])
+    description = _('Read metadata from %s files') % 'PML'
+    author      = 'John Schember'
+
+    def get_metadata(self, stream, ftype):
+        from calibre.ebooks.metadata.pml import get_metadata
+        return get_metadata(stream)
+
 class RARMetadataReader(MetadataReaderPlugin):
 
     name = 'Read RAR metadata'
@@ -258,7 +324,9 @@ class ZipMetadataReader(MetadataReaderPlugin):
     def get_metadata(self, stream, ftype):
         from calibre.ebooks.metadata.zip import get_metadata
         return get_metadata(stream)
+# }}}
 
+# Metadata writer plugins {{{
 
 class EPUBMetadataWriter(MetadataWriterPlugin):
 
@@ -268,7 +336,7 @@ class EPUBMetadataWriter(MetadataWriterPlugin):
 
     def set_metadata(self, stream, mi, type):
         from calibre.ebooks.metadata.epub import set_metadata
-        set_metadata(stream, mi)
+        set_metadata(stream, mi, apply_null=self.apply_null)
 
 class LRFMetadataWriter(MetadataWriterPlugin):
 
@@ -323,6 +391,18 @@ class RTFMetadataWriter(MetadataWriterPlugin):
         from calibre.ebooks.metadata.rtf import set_metadata
         set_metadata(stream, mi)
 
+class TOPAZMetadataWriter(MetadataWriterPlugin):
+
+    name        = 'Set TOPAZ metadata'
+    file_types  = set(['tpz', 'azw1'])
+    description = _('Set metadata in %s files')%'TOPAZ'
+    author      = 'Greg Riker'
+
+    def set_metadata(self, stream, mi, type):
+        from calibre.ebooks.metadata.topaz import set_metadata
+        set_metadata(stream, mi)
+
+# }}}
 
 from calibre.ebooks.comic.input import ComicInput
 from calibre.ebooks.epub.input import EPUBInput
@@ -337,8 +417,10 @@ from calibre.ebooks.pml.input import PMLInput
 from calibre.ebooks.rb.input import RBInput
 from calibre.web.feeds.input import RecipeInput
 from calibre.ebooks.rtf.input import RTFInput
+from calibre.ebooks.tcr.input import TCRInput
 from calibre.ebooks.txt.input import TXTInput
 from calibre.ebooks.lrf.input import LRFInput
+from calibre.ebooks.chm.input import CHMInput
 
 from calibre.ebooks.epub.output import EPUBOutput
 from calibre.ebooks.fb2.output import FB2Output
@@ -351,26 +433,50 @@ from calibre.ebooks.pdf.output import PDFOutput
 from calibre.ebooks.pml.output import PMLOutput
 from calibre.ebooks.rb.output import RBOutput
 from calibre.ebooks.rtf.output import RTFOutput
+from calibre.ebooks.tcr.output import TCROutput
 from calibre.ebooks.txt.output import TXTOutput
 
 from calibre.customize.profiles import input_profiles, output_profiles
 
-
-from calibre.devices.bebook.driver import BEBOOK, BEBOOK_MINI
+from calibre.devices.apple.driver import ITUNES
+from calibre.devices.hanlin.driver import HANLINV3, HANLINV5, BOOX, SPECTRA
 from calibre.devices.blackberry.driver import BLACKBERRY
-from calibre.devices.cybookg3.driver import CYBOOKG3, CYBOOK_OPUS
-from calibre.devices.eb600.driver import EB600, COOL_ER
+from calibre.devices.cybook.driver import CYBOOK
+from calibre.devices.eb600.driver import EB600, COOL_ER, SHINEBOOK, \
+                POCKETBOOK360, GER2, ITALICA, ECLICTO, DBOOK, INVESBOOK, \
+                BOOQ, ELONEX, POCKETBOOK301, MENTOR
 from calibre.devices.iliad.driver import ILIAD
-from calibre.devices.irexdr.driver import IREXDR1000
-from calibre.devices.jetbook.driver import JETBOOK
+from calibre.devices.irexdr.driver import IREXDR1000, IREXDR800
+from calibre.devices.jetbook.driver import JETBOOK, MIBUK
 from calibre.devices.kindle.driver import KINDLE, KINDLE2, KINDLE_DX
-from calibre.devices.prs500.driver import PRS500
+from calibre.devices.nook.driver import NOOK
 from calibre.devices.prs505.driver import PRS505
-from calibre.devices.prs700.driver import PRS700
-from calibre.devices.android.driver import ANDROID
-from calibre.devices.eslick.driver import ESLICK
+from calibre.devices.android.driver import ANDROID, S60
+from calibre.devices.nokia.driver import N770, N810, E71X, E52
+from calibre.devices.eslick.driver import ESLICK, EBK52
+from calibre.devices.nuut2.driver import NUUT2
+from calibre.devices.iriver.driver import IRIVER_STORY
+from calibre.devices.binatone.driver import README
+from calibre.devices.hanvon.driver import N516, EB511, ALEX, AZBOOKA, THEBOOK
+from calibre.devices.edge.driver import EDGE
+from calibre.devices.teclast.driver import TECLAST_K3, NEWSMY, IPAPYRUS
+from calibre.devices.sne.driver import SNE
+from calibre.devices.misc import PALMPRE, AVANT, SWEEX, PDNOVEL, KOGAN
+from calibre.devices.folder_device.driver import FOLDER_DEVICE_FOR_CONFIG
+from calibre.devices.kobo.driver import KOBO
 
-plugins = [HTML2ZIP]
+from calibre.ebooks.metadata.fetch import GoogleBooks, ISBNDB, Amazon, \
+    LibraryThing
+from calibre.ebooks.metadata.douban import DoubanBooks
+from calibre.ebooks.metadata.covers import OpenLibraryCovers, \
+        LibraryThingCovers
+from calibre.library.catalog import CSV_XML, EPUB_MOBI, BIBTEX
+from calibre.ebooks.epub.fix.unmanifested import Unmanifested
+from calibre.ebooks.epub.fix.epubcheck import Epubcheck
+
+plugins = [HTML2ZIP, PML2PMLZ, ArchiveExtract, GoogleBooks, ISBNDB, Amazon,
+        LibraryThing, DoubanBooks, CSV_XML, EPUB_MOBI, BIBTEX, Unmanifested,
+        Epubcheck, OpenLibraryCovers, LibraryThingCovers]
 plugins += [
     ComicInput,
     EPUBInput,
@@ -385,8 +491,10 @@ plugins += [
     RBInput,
     RecipeInput,
     RTFInput,
+    TCRInput,
     TXTInput,
     LRFInput,
+    CHMInput,
 ]
 plugins += [
     EPUBOutput,
@@ -400,30 +508,193 @@ plugins += [
     PMLOutput,
     RBOutput,
     RTFOutput,
+    TCROutput,
     TXTOutput,
 ]
+# Order here matters. The first matched device is the one used.
 plugins += [
-    BEBOOK,
-    BEBOOK_MINI,
+    HANLINV3,
+    HANLINV5,
     BLACKBERRY,
-    CYBOOKG3,
-    EB600,
+    CYBOOK,
     ILIAD,
     IREXDR1000,
+    IREXDR800,
     JETBOOK,
+    MIBUK,
+    SHINEBOOK,
+    POCKETBOOK360,
+    POCKETBOOK301,
     KINDLE,
     KINDLE2,
     KINDLE_DX,
-    PRS500,
+    NOOK,
     PRS505,
-    PRS700,
     ANDROID,
-    CYBOOK_OPUS,
+    S60,
+    N770,
+    E71X,
+    E52,
+    N810,
     COOL_ER,
-    ESLICK
+    ESLICK,
+    EBK52,
+    NUUT2,
+    IRIVER_STORY,
+    GER2,
+    ITALICA,
+    ECLICTO,
+    DBOOK,
+    INVESBOOK,
+    BOOX,
+    BOOQ,
+    EB600,
+    README,
+    N516,
+    THEBOOK,
+    EB511,
+    ELONEX,
+    TECLAST_K3,
+    NEWSMY,
+    IPAPYRUS,
+    EDGE,
+    SNE,
+    ALEX,
+    PALMPRE,
+    KOBO,
+    AZBOOKA,
+    FOLDER_DEVICE_FOR_CONFIG,
+    AVANT,
+    MENTOR,
+    SWEEX,
+    KOGAN,
+    PDNOVEL,
+    SPECTRA,
+    ITUNES,
 ]
 plugins += [x for x in list(locals().values()) if isinstance(x, type) and \
                                         x.__name__.endswith('MetadataReader')]
 plugins += [x for x in list(locals().values()) if isinstance(x, type) and \
                                         x.__name__.endswith('MetadataWriter')]
 plugins += input_profiles + output_profiles
+
+# Interface Actions {{{
+
+class ActionAdd(InterfaceActionBase):
+    name = 'Add Books'
+    actual_plugin = 'calibre.gui2.actions.add:AddAction'
+
+class ActionFetchAnnotations(InterfaceActionBase):
+    name = 'Fetch Annotations'
+    actual_plugin = 'calibre.gui2.actions.annotate:FetchAnnotationsAction'
+
+class ActionGenerateCatalog(InterfaceActionBase):
+    name = 'Generate Catalog'
+    actual_plugin = 'calibre.gui2.actions.catalog:GenerateCatalogAction'
+
+class ActionConvert(InterfaceActionBase):
+    name = 'Convert Books'
+    actual_plugin = 'calibre.gui2.actions.convert:ConvertAction'
+
+class ActionDelete(InterfaceActionBase):
+    name = 'Remove Books'
+    actual_plugin = 'calibre.gui2.actions.delete:DeleteAction'
+
+class ActionEditMetadata(InterfaceActionBase):
+    name = 'Edit Metadata'
+    actual_plugin = 'calibre.gui2.actions.edit_metadata:EditMetadataAction'
+
+class ActionView(InterfaceActionBase):
+    name = 'View'
+    actual_plugin = 'calibre.gui2.actions.view:ViewAction'
+
+class ActionFetchNews(InterfaceActionBase):
+    name = 'Fetch News'
+    actual_plugin = 'calibre.gui2.actions.fetch_news:FetchNewsAction'
+
+class ActionSaveToDisk(InterfaceActionBase):
+    name = 'Save To Disk'
+    actual_plugin = 'calibre.gui2.actions.save_to_disk:SaveToDiskAction'
+
+class ActionShowBookDetails(InterfaceActionBase):
+    name = 'Show Book Details'
+    actual_plugin = 'calibre.gui2.actions.show_book_details:ShowBookDetailsAction'
+
+class ActionRestart(InterfaceActionBase):
+    name = 'Restart'
+    actual_plugin = 'calibre.gui2.actions.restart:RestartAction'
+
+class ActionOpenFolder(InterfaceActionBase):
+    name = 'Open Folder'
+    actual_plugin = 'calibre.gui2.actions.open:OpenFolderAction'
+
+class ActionSendToDevice(InterfaceActionBase):
+    name = 'Send To Device'
+    actual_plugin = 'calibre.gui2.actions.device:SendToDeviceAction'
+
+class ActionConnectShare(InterfaceActionBase):
+    name = 'Connect Share'
+    actual_plugin = 'calibre.gui2.actions.device:ConnectShareAction'
+
+class ActionHelp(InterfaceActionBase):
+    name = 'Help'
+    actual_plugin = 'calibre.gui2.actions.help:HelpAction'
+
+class ActionPreferences(InterfaceActionBase):
+    name = 'Preferences'
+    actual_plugin = 'calibre.gui2.actions.preferences:PreferencesAction'
+
+class ActionSimilarBooks(InterfaceActionBase):
+    name = 'Similar Books'
+    actual_plugin = 'calibre.gui2.actions.similar_books:SimilarBooksAction'
+
+class ActionChooseLibrary(InterfaceActionBase):
+    name = 'Choose Library'
+    actual_plugin = 'calibre.gui2.actions.choose_library:ChooseLibraryAction'
+
+class ActionAddToLibrary(InterfaceActionBase):
+    name = 'Add To Library'
+    actual_plugin = 'calibre.gui2.actions.add_to_library:AddToLibraryAction'
+
+class ActionEditCollections(InterfaceActionBase):
+    name = 'Edit Collections'
+    actual_plugin = 'calibre.gui2.actions.edit_collections:EditCollectionsAction'
+
+class ActionCopyToLibrary(InterfaceActionBase):
+    name = 'Copy To Library'
+    actual_plugin = 'calibre.gui2.actions.copy_to_library:CopyToLibraryAction'
+
+plugins += [ActionAdd, ActionFetchAnnotations, ActionGenerateCatalog,
+        ActionConvert, ActionDelete, ActionEditMetadata, ActionView,
+        ActionFetchNews, ActionSaveToDisk, ActionShowBookDetails,
+        ActionRestart, ActionOpenFolder, ActionConnectShare,
+        ActionSendToDevice, ActionHelp, ActionPreferences, ActionSimilarBooks,
+        ActionAddToLibrary, ActionEditCollections, ActionChooseLibrary,
+        ActionCopyToLibrary]
+
+# }}}
+
+# Preferences Plugins {{{
+
+class LookAndFeel(PreferencesPlugin):
+    name = 'Look & Feel'
+    gui_name = _('Look and Feel')
+    category = 'Interface'
+    gui_category = _('Interface')
+    category_order = 1
+    name_order = 1
+    config_widget = 'calibre.gui2.preferences.look_feel'
+
+class Behavior(PreferencesPlugin):
+    name = 'Behavior'
+    gui_name = _('Behavior')
+    category = 'Interface'
+    gui_category = _('Interface')
+    category_order = 1
+    name_order = 2
+    config_widget = 'calibre.gui2.preferences.behavior'
+
+plugins += [LookAndFeel, Behavior]
+
+#}}}
+
