@@ -2,9 +2,9 @@ __license__   = 'GPL v3'
 __copyright__ = '2008, Ashish Kulkarni <kulkarni.ashish@gmail.com>'
 '''Read meta information from IMP files'''
 
-import sys, os
+import sys
 
-from calibre.ebooks.metadata import MetaInformation
+from calibre.ebooks.metadata import MetaInformation, string_to_authors
 
 MAGIC = ['\x00\x01BOOKDOUG', '\x00\x02BOOKDOUG']
 
@@ -17,7 +17,7 @@ def get_metadata(stream):
         if stream.read(10) not in MAGIC:
             print >>sys.stderr, u'Couldn\'t read IMP header from file'
             return mi
-        
+
         def cString(skip=0):
             result = ''
             while 1:
@@ -30,15 +30,11 @@ def get_metadata(stream):
 
         stream.read(38) # skip past some uninteresting headers
         _, category, title, author = cString(), cString(), cString(1), cString(2)
-        
+
         if title:
             mi.title = title
         if author:
-            src = author.split('&')
-            authors = []
-            for au in src:
-                authors += au.split(',')
-            mi.authors = authors
+            mi.authors = string_to_authors(author)
             mi.author = author
         if category:
             mi.category = category
@@ -46,17 +42,3 @@ def get_metadata(stream):
         msg = u'Couldn\'t read metadata from imp: %s with error %s'%(mi.title, unicode(err))
         print >>sys.stderr, msg.encode('utf8')
     return mi
-        
-            
-def main(args=sys.argv):
-    if len(args) != 2:
-        print >>sys.stderr, _('Usage: imp-meta file.imp')
-        print >>sys.stderr, _('No filename specified.')
-        return 1
-    
-    path = os.path.abspath(os.path.expanduser(args[1]))
-    print get_metadata(open(path, 'rb'))
-    return 0
-
-if __name__ == '__main__':
-    sys.exit(main())
