@@ -12,13 +12,13 @@ from urllib import unquote
 from urlparse import urlparse
 
 from lxml import etree
-from dateutil import parser
 
 from calibre.ebooks.chardet import xml_to_unicode
 from calibre.constants import __appname__, __version__, filesystem_encoding
 from calibre.ebooks.metadata.toc import TOC
 from calibre.ebooks.metadata import MetaInformation, string_to_authors
-
+from calibre.utils.date import parse_date, isoformat
+from calibre.utils.localization import get_lang
 
 class Resource(object):
     '''
@@ -270,6 +270,11 @@ class Spine(ResourceCollection):
             Resource.__init__(self, *args, **kwargs)
             self.is_linear = True
             self.id = idfunc(self.path)
+            self.idref = None
+
+        def __repr__(self):
+            return 'Spine.Item(path=%r, id=%s, is_linear=%s)' % \
+                    (self.path, self.id, self.is_linear)
 
     @staticmethod
     def from_opf_spine_element(itemrefs, manifest):
@@ -279,8 +284,9 @@ class Spine(ResourceCollection):
             if idref is not None:
                 path = s.manifest.path_for_id(idref)
                 if path:
-                    r = Spine.Item(s.manifest.id_for_path, path, is_path=True)
+                    r = Spine.Item(lambda x:idref, path, is_path=True)
                     r.is_linear = itemref.get('linear', 'yes') == 'yes'
+                    r.idref = idref
                     s.append(r)
         return s
 
@@ -398,6 +404,10 @@ class MetadataField(object):
 
     def __set__(self, obj, val):
         elem = obj.get_metadata_element(self.name)
+        if val is None:
+            if elem is not None:
+                elem.getparent().remove(elem)
+            return
         if elem is None:
             elem = obj.create_metadata_element(self.name, is_dc=self.is_dc)
         obj.set_text(elem, unicode(val))
@@ -427,9 +437,14 @@ class OPF(object):
     tags_path       = XPath('descendant::*[re:match(name(), "subject", "i")]')
     isbn_path       = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
                             '(re:match(@scheme, "isbn", "i") or re:match(@opf:scheme, "isbn", "i"))]')
+    raster_cover_path = XPath('descendant::*[re:match(name(), "meta", "i") and ' +
+            're:match(@name, "cover", "i") and @content]')
     identifier_path = XPath('descendant::*[re:match(name(), "identifier", "i")]')
     application_id_path = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
                             '(re:match(@opf:scheme, "calibre|libprs500", "i") or re:match(@scheme, "calibre|libprs500", "i"))]')
+    uuid_id_path    = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
+                            '(re:match(@opf:scheme, "uuid", "i") or re:match(@scheme, "uuid", "i"))]')
+
     manifest_path   = XPath('descendant::*[re:match(name(), "manifest", "i")]/*[re:match(name(), "item", "i")]')
     manifest_ppath  = XPath('descendant::*[re:match(name(), "manifest", "i")]')
     spine_path      = XPath('descendant::*[re:match(name(), "spine", "i")]/*[re:match(name(), "itemref", "i")]')
@@ -444,12 +459,14 @@ class OPF(object):
     series          = MetadataField('series', is_dc=False)
     series_index    = MetadataField('series_index', is_dc=False, formatter=float, none_is=1)
     rating          = MetadataField('rating', is_dc=False, formatter=int)
-    pubdate         = MetadataField('date', formatter=parser.parse)
+    pubdate         = MetadataField('date', formatter=parse_date)
     publication_type = MetadataField('publication_type', is_dc=False)
-    timestamp       = MetadataField('timestamp', is_dc=False, formatter=parser.parse)
+    timestamp       = MetadataField('timestamp', is_dc=False,
+                                    formatter=parse_date)
 
 
-    def __init__(self, stream, basedir=os.getcwdu(), unquote_urls=True):
+    def __init__(self, stream, basedir=os.getcwdu(), unquote_urls=True,
+            populate_spine=True):
         if not hasattr(stream, 'read'):
             stream = open(stream, 'rb')
         raw = stream.read()
@@ -457,7 +474,8 @@ class OPF(object):
             raise ValueError('Empty file: '+getattr(stream, 'name', 'stream'))
         self.basedir  = self.base_dir = basedir
         self.path_to_html_toc = self.html_toc_fragment = None
-        raw, self.encoding = xml_to_unicode(raw, strip_encoding_pats=True, resolve_entities=True)
+        raw, self.encoding = xml_to_unicode(raw, strip_encoding_pats=True,
+                resolve_entities=True, assume_utf8=True)
         raw = raw[raw.find('<'):]
         self.root     = etree.fromstring(raw, self.PARSER)
         self.metadata = self.metadata_path(self.root)
@@ -472,7 +490,7 @@ class OPF(object):
             self.manifest = Manifest.from_opf_manifest_element(m, basedir)
         self.spine = None
         s = self.spine_path(self.root)
-        if s:
+        if populate_spine and s:
             self.spine = Spine.from_opf_spine_element(s, self.manifest)
         self.guide = None
         guide = self.guide_path(self.root)
@@ -579,6 +597,15 @@ class OPF(object):
                 if x.get('id', None) == idref:
                     yield x.get('href', '')
 
+    def first_spine_item(self):
+        items = self.iterspine()
+        if not items:
+            return None
+        idref = items[0].get('idref', '')
+        for x in self.itermanifest():
+            if x.get('id', None) == idref:
+                return x.get('href', None)
+
     def create_spine_item(self, idref):
         ans = etree.Element('{%s}itemref'%self.NAMESPACES['opf'], idref=idref)
         ans.tail = '\n\t\t'
@@ -640,10 +667,19 @@ class OPF(object):
             remove = list(self.authors_path(self.metadata))
             for elem in remove:
                 elem.getparent().remove(elem)
+            elems = []
             for author in val:
                 attrib = {'{%s}role'%self.NAMESPACES['opf']: 'aut'}
                 elem = self.create_metadata_element('creator', attrib=attrib)
                 self.set_text(elem, author.strip())
+                # Ensure new author element is at the top of the list
+                # for broken implementations that always use the first
+                # <dc:creator> element with no attention to the role
+                elems.append(elem)
+            for elem in reversed(elems):
+                parent = elem.getparent()
+                parent.remove(elem)
+                parent.insert(0, elem)
 
         return property(fget=fget, fset=fset)
 
@@ -666,7 +702,7 @@ class OPF(object):
                 for key in matches[0].attrib:
                     if key.endswith('file-as'):
                         matches[0].attrib.pop(key)
-                matches[0].set('file-as', unicode(val))
+                matches[0].set('{%s}file-as'%self.NAMESPACES['opf'], unicode(val))
 
         return property(fget=fget, fset=fset)
 
@@ -689,22 +725,25 @@ class OPF(object):
                 for key in matches[0].attrib:
                     if key.endswith('file-as'):
                         matches[0].attrib.pop(key)
-                matches[0].set('file-as', unicode(val))
+                matches[0].set('{%s}file-as'%self.NAMESPACES['opf'], unicode(val))
 
         return property(fget=fget, fset=fset)
 
+
     @dynamic_property
     def tags(self):
 
         def fget(self):
             ans = []
             for tag in self.tags_path(self.metadata):
-                ans.append(self.get_text(tag))
+                text = self.get_text(tag)
+                if text and text.strip():
+                    ans.extend([x.strip() for x in text.split(',')])
             return ans
 
         def fset(self, val):
             for tag in list(self.tags_path(self.metadata)):
-                self.metadata.remove(tag)
+                tag.getparent().remove(tag)
             for tag in val:
                 elem = self.create_metadata_element('subject')
                 self.set_text(elem, unicode(tag))
@@ -720,6 +759,11 @@ class OPF(object):
 
         def fset(self, val):
             matches = self.isbn_path(self.metadata)
+            if val is None:
+                if matches:
+                    for x in matches:
+                        x.getparent().remove(x)
+                    return
             if not matches:
                 attrib = {'{%s}scheme'%self.NAMESPACES['opf']: 'ISBN'}
                 matches = [self.create_metadata_element('identifier',
@@ -745,6 +789,25 @@ class OPF(object):
 
         return property(fget=fget, fset=fset)
 
+    @dynamic_property
+    def uuid(self):
+
+        def fget(self):
+            for match in self.uuid_id_path(self.metadata):
+                return self.get_text(match) or None
+
+        def fset(self, val):
+            matches = self.uuid_id_path(self.metadata)
+            if not matches:
+                attrib = {'{%s}scheme'%self.NAMESPACES['opf']: 'uuid'}
+                matches = [self.create_metadata_element('identifier',
+                                                        attrib=attrib)]
+            self.set_text(matches[0], unicode(val))
+
+        return property(fget=fget, fset=fset)
+
+
+
     @dynamic_property
     def book_producer(self):
 
@@ -761,6 +824,9 @@ class OPF(object):
             self.set_text(matches[0], unicode(val))
         return property(fget=fget, fset=fset)
 
+    def identifier_iter(self):
+        for item in self.identifier_path(self.metadata):
+            yield item
 
     def guess_cover(self):
         '''
@@ -782,6 +848,14 @@ class OPF(object):
                         if os.access(os.path.join(self.base_dir, prefix+suffix), os.R_OK):
                             return cpath
 
+    @property
+    def raster_cover(self):
+        covers = self.raster_cover_path(self.metadata)
+        if covers:
+            cover_id = covers[0].get('content')
+            for item in self.itermanifest():
+                if item.get('id', None) == cover_id:
+                    return item.get('href', None)
 
     @dynamic_property
     def cover(self):
@@ -842,10 +916,11 @@ class OPF(object):
             raw = '<?xml version="1.0"  encoding="%s"?>\n'%encoding.upper()+raw
         return raw
 
-    def smart_update(self, mi):
+    def smart_update(self, mi, replace_metadata=False):
         for attr in ('title', 'authors', 'author_sort', 'title_sort',
                      'publisher', 'series', 'series_index', 'rating',
-                     'isbn', 'language', 'tags', 'category', 'comments'):
+                     'isbn', 'language', 'tags', 'category', 'comments',
+                     'pubdate'):
             val = getattr(mi, attr, None)
             if val is not None and val != [] and val != (None, None):
                 setattr(self, attr, val)
@@ -930,11 +1005,8 @@ class OPFCreator(MetaInformation):
 
     def render(self, opf_stream=sys.stdout, ncx_stream=None,
                ncx_manifest_entry=None, encoding=None):
-        from calibre.utils.genshi.template import MarkupTemplate
-        opf_template = open(P('templates/opf.xml'), 'rb').read()
         if encoding is None:
             encoding = 'utf-8'
-        template = MarkupTemplate(opf_template)
         toc = getattr(self, 'toc', None)
         if self.manifest:
             self.manifest.set_basedir(self.base_path)
@@ -955,12 +1027,104 @@ class OPFCreator(MetaInformation):
                 cover = os.path.abspath(os.path.join(self.base_path, cover))
             self.guide.set_cover(cover)
         self.guide.set_basedir(self.base_path)
-        opf = template.generate(
-                __appname__=__appname__, mi=self,
-                __version__=__version__).render('xml', encoding=encoding)
-        opf_stream.write('<?xml version="1.0" encoding="%s" ?>\n'
-                %encoding.upper())
-        opf_stream.write(opf)
+
+        # Actual rendering
+        from lxml.builder import ElementMaker
+        from calibre.ebooks.oeb.base import OPF2_NS, DC11_NS, CALIBRE_NS
+        DNS = OPF2_NS+'___xx___'
+        E = ElementMaker(namespace=DNS, nsmap={None:DNS})
+        M = ElementMaker(namespace=DNS,
+                nsmap={'dc':DC11_NS, 'calibre':CALIBRE_NS, 'opf':OPF2_NS})
+        DC = ElementMaker(namespace=DC11_NS)
+
+        def DC_ELEM(tag, text, dc_attrs={}, opf_attrs={}):
+            if text:
+                elem = getattr(DC, tag)(text, **dc_attrs)
+            else:
+                elem = getattr(DC, tag)(**dc_attrs)
+            for k, v in opf_attrs.items():
+                elem.set('{%s}%s'%(OPF2_NS, k), v)
+            return elem
+
+        def CAL_ELEM(name, content):
+            return M.meta(name=name, content=content)
+
+        metadata = M.metadata()
+        a = metadata.append
+        role = {}
+        if self.title_sort:
+            role = {'file-as':self.title_sort}
+        a(DC_ELEM('title', self.title if self.title else _('Unknown'),
+            opf_attrs=role))
+        for i, author in enumerate(self.authors):
+            fa = {'role':'aut'}
+            if i == 0 and self.author_sort:
+                fa['file-as'] = self.author_sort
+            a(DC_ELEM('creator', author, opf_attrs=fa))
+        a(DC_ELEM('contributor', '%s (%s) [%s]'%(__appname__, __version__,
+            'http://calibre-ebook.com'), opf_attrs={'role':'bkp',
+                'file-as':__appname__}))
+        a(DC_ELEM('identifier', str(self.application_id),
+            opf_attrs={'scheme':__appname__},
+            dc_attrs={'id':__appname__+'_id'}))
+        if getattr(self, 'pubdate', None) is not None:
+            a(DC_ELEM('date', self.pubdate.isoformat()))
+        lang = self.language
+        if not lang or lang.lower() == 'und':
+            lang = get_lang().replace('_', '-')
+        a(DC_ELEM('language', lang))
+        if self.comments:
+            a(DC_ELEM('description', self.comments))
+        if self.publisher:
+            a(DC_ELEM('publisher', self.publisher))
+        if self.isbn:
+            a(DC_ELEM('identifier', self.isbn, opf_attrs={'scheme':'ISBN'}))
+        if self.rights:
+            a(DC_ELEM('rights', self.rights))
+        if self.tags:
+            for tag in self.tags:
+                a(DC_ELEM('subject', tag))
+        if self.series:
+            a(CAL_ELEM('calibre:series', self.series))
+            if self.series_index is not None:
+                a(CAL_ELEM('calibre:series_index', self.format_series_index()))
+        if self.rating is not None:
+            a(CAL_ELEM('calibre:rating', str(self.rating)))
+        if self.timestamp is not None:
+            a(CAL_ELEM('calibre:timestamp', self.timestamp.isoformat()))
+        if self.publication_type is not None:
+            a(CAL_ELEM('calibre:publication_type', self.publication_type))
+        manifest = E.manifest()
+        if self.manifest is not None:
+            for ref in self.manifest:
+                item = E.item(id=str(ref.id), href=ref.href())
+                item.set('media-type', ref.mime_type)
+                manifest.append(item)
+        spine = E.spine()
+        if self.toc is not None:
+            spine.set('toc', 'ncx')
+        if self.spine is not None:
+            for ref in self.spine:
+                spine.append(E.itemref(idref=ref.id))
+        guide = E.guide()
+        if self.guide is not None:
+            for ref in self.guide:
+                item = E.reference(type=ref.type, href=ref.href())
+                if ref.title:
+                    item.set('title', ref.title)
+                guide.append(item)
+
+        root = E.package(
+                metadata,
+                manifest,
+                spine,
+                guide
+        )
+        root.set('unique-identifier', __appname__+'_id')
+        raw = etree.tostring(root, pretty_print=True, xml_declaration=True,
+                encoding=encoding)
+        raw = raw.replace(DNS, OPF2_NS)
+        opf_stream.write(raw)
         opf_stream.flush()
         if toc is not None and ncx_stream is not None:
             toc.render(ncx_stream, self.application_id)
@@ -975,6 +1139,9 @@ def metadata_to_opf(mi, as_string=True):
     if not mi.application_id:
         mi.application_id = str(uuid.uuid4())
 
+    if not mi.uuid:
+        mi.uuid = str(uuid.uuid4())
+
     if not mi.book_producer:
         mi.book_producer = __appname__ + ' (%s) '%__version__ + \
             '[http://calibre-ebook.com]'
@@ -984,13 +1151,14 @@ def metadata_to_opf(mi, as_string=True):
 
     root = etree.fromstring(textwrap.dedent(
     '''
-    <package xmlns="http://www.idpf.org/2007/opf" unique-identifier="%(a)s_id">
+    <package xmlns="http://www.idpf.org/2007/opf" unique-identifier="uuid_id">
         <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
             <dc:identifier opf:scheme="%(a)s" id="%(a)s_id">%(id)s</dc:identifier>
+            <dc:identifier opf:scheme="uuid" id="uuid_id">%(uuid)s</dc:identifier>
         </metadata>
         <guide/>
     </package>
-    '''%dict(a=__appname__, id=mi.application_id)))
+    '''%dict(a=__appname__, id=mi.application_id, uuid=mi.uuid)))
     metadata = root[0]
     guide = root[1]
     metadata[0].tail = '\n'+(' '*8)
@@ -1013,13 +1181,12 @@ def metadata_to_opf(mi, as_string=True):
             elem.text = text.strip()
         metadata.append(elem)
 
-    factory(DC('title'), mi.title, mi.title_sort)
+    factory(DC('title'), mi.title)
     for au in mi.authors:
         factory(DC('creator'), au, mi.author_sort, 'aut')
     factory(DC('contributor'), mi.book_producer, __appname__, 'bkp')
     if hasattr(mi.pubdate, 'isoformat'):
-        factory(DC('date'), mi.pubdate.isoformat())
-    factory(DC('language'), mi.language)
+        factory(DC('date'), isoformat(mi.pubdate))
     if mi.category:
         factory(DC('type'), mi.category)
     if mi.comments:
@@ -1030,6 +1197,8 @@ def metadata_to_opf(mi, as_string=True):
         factory(DC('identifier'), mi.isbn, scheme='ISBN')
     if mi.rights:
         factory(DC('rights'), mi.rights)
+    factory(DC('language'), mi.language if mi.language and mi.language.lower()
+            != 'und' else get_lang().replace('_', '-'))
     if mi.tags:
         for tag in mi.tags:
             factory(DC('subject'), tag)
@@ -1041,9 +1210,11 @@ def metadata_to_opf(mi, as_string=True):
     if mi.rating is not None:
         meta('rating', str(mi.rating))
     if hasattr(mi.timestamp, 'isoformat'):
-        meta('timestamp', mi.timestamp.isoformat())
+        meta('timestamp', isoformat(mi.timestamp))
     if mi.publication_type:
         meta('publication_type', mi.publication_type)
+    if mi.title_sort:
+        meta('title_sort', mi.title_sort)
 
     metadata[-1].tail = '\n' +(' '*4)
 
@@ -1060,12 +1231,12 @@ def metadata_to_opf(mi, as_string=True):
 
 
 def test_m2o():
-    from datetime import datetime
+    from calibre.utils.date import now as nowf
     from cStringIO import StringIO
     mi = MetaInformation('test & title', ['a"1', "a'2"])
     mi.title_sort = 'a\'"b'
     mi.author_sort = 'author sort'
-    mi.pubdate = datetime.now()
+    mi.pubdate = nowf()
     mi.language = 'en'
     mi.category = 'test'
     mi.comments = 'what a fun book\n\n'
@@ -1075,7 +1246,7 @@ def test_m2o():
     mi.series = 's"c\'l&<>'
     mi.series_index = 3.34
     mi.rating = 3
-    mi.timestamp = datetime.now()
+    mi.timestamp = nowf()
     mi.publication_type = 'ooooo'
     mi.rights = 'yes'
     mi.cover = 'asd.jpg'
@@ -1102,12 +1273,14 @@ class OPFTest(unittest.TestCase):
 <package version="2.0" xmlns="http://www.idpf.org/2007/opf" >
 <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
     <dc:title opf:file-as="Wow">A Cool &amp; &copy; &#223; Title</dc:title>
-    <creator opf:role="aut" file-as="Monkey">Monkey Kitchen, Next</creator>
+    <creator opf:role="aut" file-as="Monkey">Monkey Kitchen</creator>
+    <creator opf:role="aut">Next</creator>
     <dc:subject>One</dc:subject><dc:subject>Two</dc:subject>
     <dc:identifier scheme="ISBN">123456789</dc:identifier>
-    <x-metadata>
-        <series>A one book series</series>
-    </x-metadata>
+    <meta name="calibre:series" content="A one book series" />
+    <meta name="calibre:rating" content="4"/>
+    <meta name="calibre:publication_type" content="test"/>
+    <meta name="calibre:series_index" content="2.5" />
 </metadata>
 <manifest>
     <item id="1" href="a%20%7E%20b" media-type="text/txt" />
@@ -1127,7 +1300,9 @@ class OPFTest(unittest.TestCase):
         self.assertEqual(opf.tags, ['One', 'Two'])
         self.assertEqual(opf.isbn, '123456789')
         self.assertEqual(opf.series, 'A one book series')
-        self.assertEqual(opf.series_index, 1)
+        self.assertEqual(opf.series_index, 2.5)
+        self.assertEqual(opf.rating, 4)
+        self.assertEqual(opf.publication_type, 'test')
         self.assertEqual(list(opf.itermanifest())[0].get('href'), 'a ~ b')
 
     def testWriting(self):
@@ -1157,3 +1332,6 @@ def suite():
 
 def test():
     unittest.TextTestRunner(verbosity=2).run(suite())
+
+if __name__ == '__main__':
+    test()
