@@ -25,10 +25,17 @@ from calibre.ebooks.oeb.base import XHTML, XHTML_NS, CSS_MIME, OEB_STYLES
 from calibre.ebooks.oeb.base import XPNSMAP, xpath, urlnormalize
 from calibre.ebooks.oeb.profile import PROFILES
 
-html_css = open(P('templates/html.css'), 'rb').read()
+_html_css_stylesheet = None
+
+def html_css_stylesheet():
+    global _html_css_stylesheet
+    if _html_css_stylesheet is None:
+        html_css = open(P('templates/html.css'), 'rb').read()
+        _html_css_stylesheet = cssutils.parseString(html_css)
+        _html_css_stylesheet.namespaces['h'] = XHTML_NS
+    return _html_css_stylesheet
+
 XHTML_CSS_NAMESPACE = '@namespace "%s";\n' % XHTML_NS
-HTML_CSS_STYLESHEET = cssutils.parseString(html_css)
-HTML_CSS_STYLESHEET.namespaces['h'] = XHTML_NS
 
 INHERITED = set(['azimuth', 'border-collapse', 'border-spacing',
                  'caption-side', 'color', 'cursor', 'direction', 'elevation',
@@ -96,6 +103,8 @@ class CSSSelector(etree.XPath):
             path = css_to_xpath(css)
         except UnicodeEncodeError: # Bug in css_to_xpath
             path = '/'
+        except NotImplementedError: # Probably a subselect like :hover
+            path = '/'
         path = self.LOCAL_NAME_RE.sub(r"local-name() = '", path)
         etree.XPath.__init__(self, path, namespaces=namespaces)
         self.css = css
@@ -110,16 +119,27 @@ class CSSSelector(etree.XPath):
 class Stylizer(object):
     STYLESHEETS = WeakKeyDictionary()
 
-    def __init__(self, tree, path, oeb, profile=PROFILES['PRS505'],
+    def __init__(self, tree, path, oeb, opts, profile=PROFILES['PRS505'],
             extra_css='', user_css=''):
-        self.oeb = oeb
+        self.oeb, self.opts = oeb, opts
         self.profile = profile
         self.logger = oeb.logger
         item = oeb.manifest.hrefs[path]
         basename = os.path.basename(path)
         cssname = os.path.splitext(basename)[0] + '.css'
-        stylesheets = [HTML_CSS_STYLESHEET]
-        head = xpath(tree, '/h:html/h:head')[0]
+        stylesheets = [html_css_stylesheet()]
+        head = xpath(tree, '/h:html/h:head')
+        if head:
+            head = head[0]
+        else:
+            head = []
+
+        # Add cssutils parsing profiles from output_profile
+        for profile in self.opts.output_profile.extra_css_modules:
+            cssutils.profile.addProfile(profile['name'],
+                                        profile['props'],
+                                        profile['macros'])
+
         parser = cssutils.CSSParser(fetcher=self._fetch_css_file,
                 log=logging.getLogger('calibre.css'))
         self.font_face_rules = []
@@ -148,12 +168,18 @@ class Stylizer(object):
                         item.href))
                     continue
                 stylesheets.append(sitem.data)
-        for x in (extra_css, user_css):
+        csses = {'extra_css':extra_css, 'user_css':user_css}
+        for w, x in csses.items():
             if x:
-                text = XHTML_CSS_NAMESPACE + x
-                stylesheet = parser.parseString(text, href=cssname)
-                stylesheet.namespaces['h'] = XHTML_NS
-                stylesheets.append(stylesheet)
+                try:
+                    text = XHTML_CSS_NAMESPACE + x
+                    stylesheet = parser.parseString(text, href=cssname)
+                    stylesheet.namespaces['h'] = XHTML_NS
+                    stylesheets.append(stylesheet)
+                except:
+                    self.logger.exception('Failed to parse %s, ignoring.'%w)
+                    self.logger.debug('Bad css: ')
+                    self.logger.debug(x)
         rules = []
         index = 0
         self.stylesheets = set()
@@ -168,7 +194,11 @@ class Stylizer(object):
         self.rules = rules
         self._styles = {}
         class_sel_pat = re.compile(r'\.[a-z]+', re.IGNORECASE)
+        capital_sel_pat = re.compile(r'h|[A-Z]+')
         for _, _, cssdict, text, _ in rules:
+            fl = ':first-letter' in text
+            if fl:
+                text = text.replace(':first-letter', '')
             try:
                 selector = CSSSelector(text)
             except (AssertionError, ExpressionError, etree.XPathSyntaxError,
@@ -176,18 +206,50 @@ class Stylizer(object):
                     SelectorSyntaxError):
                 continue
             matches = selector(tree)
-            if not matches and class_sel_pat.match(text):
+
+            if not matches:
+                ntext = capital_sel_pat.sub(lambda m: m.group().lower(), text)
+                if ntext != text:
+                    self.logger.warn('Transformed CSS selector', text, 'to',
+                            ntext)
+                    selector = CSSSelector(ntext)
+                    matches = selector(tree)
+
+            if not matches and class_sel_pat.match(text) and text.lower() != text:
                 found = False
+                ltext = text.lower()
                 for x in tree.xpath('//*[@class]'):
-                    if text.lower().endswith('.'+x.get('class').lower()) and \
-                            text.lower() != text:
+                    if ltext.endswith('.'+x.get('class').lower()):
                         matches.append(x)
                         found = True
                 if found:
                     self.logger.warn('Ignoring case mismatches for CSS selector: %s in %s'
                         %(text, item.href))
-            for elem in matches:
-                self.style(elem)._update_cssdict(cssdict)
+            if fl:
+                from lxml.builder import ElementMaker
+                E = ElementMaker(namespace=XHTML_NS)
+                for elem in matches:
+                    for x in elem.iter():
+                        if x.text:
+                            span = E.span(x.text[0])
+                            span.tail = x.text[1:]
+                            x.text = None
+                            x.insert(0, span)
+                            self.style(span)._update_cssdict(cssdict)
+                            break
+            else:
+                for elem in matches:
+                    self.style(elem)._update_cssdict(cssdict)
+        for elem in xpath(tree, '//h:img[@width or @height]'):
+            base = elem.get('style', '').strip()
+            if base:
+                base += ';'
+            for prop in ('width', 'height'):
+                val = elem.get(prop, False)
+                if val:
+                    base += '%s: %s;'%(prop, val)
+                    del elem.attrib[prop]
+            elem.set('style', base)
         for elem in xpath(tree, '//h:*[@style]'):
             self.style(elem)._apply_style_attr()
 
@@ -227,6 +289,10 @@ class Stylizer(object):
                 style.update(self._normalize_edge(prop.cssValue, name))
             elif name == 'font':
                 style.update(self._normalize_font(prop.cssValue))
+            elif name == 'list-style':
+                style.update(self._normalize_list_style(prop.cssValue))
+            elif name == 'text-align':
+                style.update(self._normalize_text_align(prop.cssValue))
             else:
                 style[name] = prop.value
         if 'font-size' in style:
@@ -259,6 +325,44 @@ class Stylizer(object):
             style["%s-%s" % (name, edge)] = value
         return style
 
+    def _normalize_list_style(self, cssvalue):
+        composition = ('list-style-type', 'list-style-position',
+                       'list-style-image')
+        style = {}
+        if cssvalue.cssText == 'inherit':
+            for key in composition:
+                style[key] = 'inherit'
+        else:
+            try:
+                primitives = [v.cssText for v in cssvalue]
+            except TypeError:
+                primitives = [cssvalue.cssText]
+            primitives.reverse()
+            value = primitives.pop()
+            for key in composition:
+                if cssprofiles.validate(key, value):
+                    style[key] = value
+                    if not primitives: break
+                    value = primitives.pop()
+            for key in composition:
+                if key not in style:
+                    style[key] = DEFAULTS[key]
+
+        return style
+
+    def _normalize_text_align(self, cssvalue):
+        style = {}
+        text = cssvalue.cssText
+        if text == 'inherit':
+            style['text-align'] = 'inherit'
+        else:
+            if text in ('left', 'justify') and self.opts.change_justification in ('left', 'justify'):
+                val = self.opts.change_justification
+                style['text-align'] = val
+            else:
+                style['text-align'] = text
+        return style
+
     def _normalize_font(self, cssvalue):
         composition = ('font-style', 'font-variant', 'font-weight',
                        'font-size', 'line-height', 'font-family')
@@ -364,6 +468,7 @@ class Style(object):
         return result
 
     def _unit_convert(self, value, base=None, font=None):
+        ' Return value in pts'
         if isinstance(value, (int, long, float)):
             return value
         try:
@@ -376,7 +481,8 @@ class Style(object):
             value = float(m.group(1))
             unit = m.group(2)
             if unit == '%':
-                base = base or self.width
+                if base is None:
+                    base = self.width
                 result = (value / 100.0) * base
             elif unit == 'px':
                 result = value * 72.0 / self._profile.dpi
@@ -400,6 +506,9 @@ class Style(object):
                 result = value * 0.40
         return result
 
+    def pt_to_px(self, value):
+        return (self._profile.dpi / 72.0) * value
+
     @property
     def fontSize(self):
         def normalize_fontsize(value, base):
@@ -456,7 +565,7 @@ class Style(object):
                 base = parent.width
             else:
                 base = self._profile.width
-            if 'width' is self._element.attrib:
+            if 'width' in self._element.attrib:
                 width = self._element.attrib['width']
             elif 'width' in self._style:
                 width = self._style['width']
@@ -464,6 +573,8 @@ class Style(object):
                 result = base
             else:
                 result = self._unit_convert(width, base=base)
+            if isinstance(result, (unicode, str, bytes)):
+                result = self._profile.width
             self._width = result
         return self._width
 
@@ -477,7 +588,7 @@ class Style(object):
                 base = parent.height
             else:
                 base = self._profile.height
-            if 'height' is self._element.attrib:
+            if 'height' in self._element.attrib:
                 height = self._element.attrib['height']
             elif 'height' in self._style:
                 height = self._style['height']
@@ -485,6 +596,8 @@ class Style(object):
                 result = base
             else:
                 result = self._unit_convert(height, base=base)
+            if isinstance(result, (unicode, str, bytes)):
+                result = self._profile.height
             self._height = result
         return self._height
 
