@@ -1,24 +1,62 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 """ The GUI """
-import os
-from PyQt4.QtCore import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, QSize, \
-                         QByteArray, QTranslator, QCoreApplication, QThread
-from PyQt4.QtGui import QFileDialog, QMessageBox, QPixmap, QFileIconProvider, \
-                        QIcon, QTableView, QApplication, QDialog, QPushButton
+import os, sys
+from threading import RLock
+
+from PyQt4.Qt import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, \
+                         QByteArray, QTranslator, QCoreApplication, QThread, \
+                         QEvent, QTimer, pyqtSignal, QDate, QDesktopServices, \
+                         QFileDialog, QMessageBox, QPixmap, QFileIconProvider, \
+                         QIcon, QApplication, QDialog, QPushButton, QUrl
 
 ORG_NAME = 'KovidsBrain'
 APP_UID  = 'libprs500'
-from calibre import islinux, iswindows, isosx
-from calibre.utils.config import Config, ConfigProxy, dynamic
+from calibre.constants import islinux, iswindows, isosx, isfreebsd, isfrozen
+from calibre.utils.config import Config, ConfigProxy, dynamic, JSONConfig
 from calibre.utils.localization import set_qt_translator
 from calibre.ebooks.metadata.meta import get_metadata, metadata_from_formats
 from calibre.ebooks.metadata import MetaInformation
+from calibre.utils.date import UNDEFINED_DATE
 
+# Setup gprefs {{{
+gprefs = JSONConfig('gui')
+
+gprefs.defaults['action-layout-toolbar'] = (
+        'Add Books', 'Edit Metadata', None, 'Convert Books', 'View', None,
+        'Choose Library', 'Donate', None, 'Fetch News', 'Save To Disk',
+        'Connect Share', None, 'Remove Books', None, 'Help', 'Preferences',
+        )
+
+gprefs.defaults['action-layout-toolbar-device'] = (
+        'Add Books', 'Edit Metadata', None, 'Convert Books', 'View',
+        'Send To Device', None, None, 'Location Manager', None, None,
+        'Fetch News', 'Save To Disk', 'Connect Share', None,
+        'Remove Books', None, 'Help', 'Preferences',
+        )
+
+gprefs.defaults['action-layout-context-menu'] = (
+        'Edit Metadata', 'Send To Device', 'Save To Disk',
+        'Connect Share', 'Copy To Library', None,
+        'Convert Books', 'View', 'Open Folder', 'Show Book Details',
+        'Similar Books', None, 'Remove Books',
+        )
+
+gprefs.defaults['action-layout-context-menu-device'] = (
+        'View', 'Save To Disk', None, 'Remove Books', None,
+        'Add To Library', 'Edit Collections',
+        )
+
+gprefs.defaults['show_splash_screen'] = True
+gprefs.defaults['toolbar_icon_size'] = 'medium'
+gprefs.defaults['toolbar_text'] = 'auto'
+
+# }}}
 
 NONE = QVariant() #: Null value to return from the data function of item models
+UNDEFINED_QDATE = QDate(UNDEFINED_DATE)
 
-ALL_COLUMNS = ['title', 'authors', 'size', 'timestamp', 'rating', 'publisher',
+ALL_COLUMNS = ['title', 'ondevice', 'authors', 'size', 'timestamp', 'rating', 'publisher',
         'tags', 'series', 'pubdate']
 
 def _config():
@@ -27,18 +65,14 @@ def _config():
               help=_('Send file to storage card instead of main memory by default'))
     c.add_opt('confirm_delete', default=False,
               help=_('Confirm before deleting'))
-    c.add_opt('toolbar_icon_size', default=QSize(48, 48),
-              help=_('Toolbar icon size')) # value QVariant.toSize
-    c.add_opt('show_text_in_toolbar', default=True,
-              help=_('Show button labels in the toolbar'))
     c.add_opt('main_window_geometry', default=None,
               help=_('Main window geometry')) # value QVariant.toByteArray
     c.add_opt('new_version_notification', default=True,
               help=_('Notify when a new version is available'))
     c.add_opt('use_roman_numerals_for_series_number', default=True,
               help=_('Use Roman numerals for series number'))
-    c.add_opt('sort_by_popularity', default=False,
-              help=_('Sort tags list by popularity'))
+    c.add_opt('sort_tags_by', default='name',
+              help=_('Sort tags list by name, popularity, or rating'))
     c.add_opt('cover_flow_queue_length', default=6,
               help=_('Number of covers to show in the cover browsing mode'))
     c.add_opt('LRF_conversion_defaults', default=[],
@@ -52,7 +86,7 @@ def _config():
               help=_('Columns to be displayed in the book list'))
     c.add_opt('autolaunch_server', default=False, help=_('Automatically launch content server on application startup'))
     c.add_opt('oldest_news', default=60, help=_('Oldest news kept in database'))
-    c.add_opt('systray_icon', default=True, help=_('Show system tray icon'))
+    c.add_opt('systray_icon', default=False, help=_('Show system tray icon'))
     c.add_opt('upload_news_to_device', default=True,
               help=_('Upload downloaded news to device'))
     c.add_opt('delete_news_from_library_on_upload', default=False,
@@ -64,8 +98,6 @@ def _config():
     c.add_opt('default_send_to_device_action', default=None,
             help=_('Default action to perform when send to device button is '
                 'clicked'))
-    c.add_opt('show_donate_button', default=True,
-            help='Show donation button')
     c.add_opt('asked_library_thing_password', default=False,
             help='Asked library thing password at least once.')
     c.add_opt('search_as_you_type', default=True,
@@ -73,6 +105,8 @@ def _config():
             'only take place when the Enter or Return key is pressed.')
     c.add_opt('save_to_disk_template_history', default=[],
         help='Previously used Save to Disk templates')
+    c.add_opt('send_to_device_template_history', default=[],
+        help='Previously used Send to Device templates')
     c.add_opt('main_search_history', default=[],
         help='Search history for the main GUI')
     c.add_opt('viewer_search_history', default=[],
@@ -83,7 +117,20 @@ def _config():
         help='Search history for the recipe scheduler')
     c.add_opt('worker_limit', default=6,
             help=_('Maximum number of waiting worker processes'))
-
+    c.add_opt('get_social_metadata', default=True,
+            help=_('Download social metadata (tags/rating/etc.)'))
+    c.add_opt('overwrite_author_title_metadata', default=True,
+            help=_('Overwrite author and title with new metadata'))
+    c.add_opt('enforce_cpu_limit', default=True,
+            help=_('Limit max simultaneous jobs to number of CPUs'))
+    c.add_opt('tag_browser_hidden_categories', default=set(),
+            help=_('tag browser categories not to display'))
+    c.add_opt('gui_layout', choices=['wide', 'narrow'],
+            help=_('The layout of the user interface'), default='wide')
+    c.add_opt('show_avg_rating', default=True,
+            help=_('Show the average rating per item indication in the tag browser'))
+    c.add_opt('disable_animations', default=False,
+            help=_('Disable UI animations'))
     return ConfigProxy(c)
 
 config = _config()
@@ -110,6 +157,17 @@ def available_width():
     desktop       = QCoreApplication.instance().desktop()
     return desktop.availableGeometry().width()
 
+_is_widescreen = None
+
+def is_widescreen():
+    global _is_widescreen
+    if _is_widescreen is None:
+        try:
+            _is_widescreen = float(available_width())/available_height() > 1.4
+        except:
+            _is_widescreen = False
+    return _is_widescreen
+
 def extension(path):
     return os.path.splitext(path)[1][1:].lower()
 
@@ -124,17 +182,22 @@ class CopyButton(QPushButton):
 
 
     def keyPressEvent(self, ev):
-        if ev.key() in self.ACTION_KEYS:
-            self.copied()
-        else:
-            QPushButton.event(self, ev)
+        try:
+            if ev.key() in self.ACTION_KEYS:
+                self.copied()
+                return
+        except:
+            pass
+        QPushButton.keyPressEvent(self, ev)
 
 
     def keyReleaseEvent(self, ev):
-        if ev.key() in self.ACTION_KEYS:
+        try:
+            if ev.key() in self.ACTION_KEYS:
+                return
+        except:
             pass
-        else:
-            QPushButton.event(self, ev)
+        QPushButton.keyReleaseEvent(self, ev)
 
     def mouseReleaseEvent(self, ev):
         ev.accept()
@@ -165,29 +228,39 @@ class MessageBox(QMessageBox):
 
 
 
-def warning_dialog(parent, title, msg, det_msg='', show=False):
+def warning_dialog(parent, title, msg, det_msg='', show=False,
+        show_copy_button=True):
     d = MessageBox(QMessageBox.Warning, 'WARNING: '+title, msg, QMessageBox.Ok,
                     parent, det_msg)
     d.setEscapeButton(QMessageBox.Ok)
     d.setIconPixmap(QPixmap(I('dialog_warning.svg')))
+    if not show_copy_button:
+        d.cb.setVisible(False)
     if show:
         return d.exec_()
     return d
 
-def error_dialog(parent, title, msg, det_msg='', show=False):
+def error_dialog(parent, title, msg, det_msg='', show=False,
+        show_copy_button=True):
     d = MessageBox(QMessageBox.Critical, 'ERROR: '+title, msg, QMessageBox.Ok,
                     parent, det_msg)
     d.setIconPixmap(QPixmap(I('dialog_error.svg')))
     d.setEscapeButton(QMessageBox.Ok)
+    if not show_copy_button:
+        d.cb.setVisible(False)
     if show:
         return d.exec_()
     return d
 
-def question_dialog(parent, title, msg, det_msg=''):
-    d = MessageBox(QMessageBox.Question, title, msg, QMessageBox.Yes|QMessageBox.No,
+def question_dialog(parent, title, msg, det_msg='', show_copy_button=True,
+        buttons=QMessageBox.Yes|QMessageBox.No):
+    d = MessageBox(QMessageBox.Question, title, msg, buttons,
                     parent, det_msg)
-    d.setIconPixmap(QPixmap(I('dialog_information.svg')))
+    d.setIconPixmap(QPixmap(I('dialog_question.svg')))
     d.setEscapeButton(QMessageBox.No)
+    if not show_copy_button:
+        d.cb.setVisible(False)
+
     return d.exec_() == QMessageBox.Yes
 
 def info_dialog(parent, title, msg, det_msg='', show=False):
@@ -199,36 +272,25 @@ def info_dialog(parent, title, msg, det_msg='', show=False):
     return d
 
 
-def qstring_to_unicode(q):
-    return unicode(q)
-
-def human_readable(size):
-    """ Convert a size in bytes into a human readable form """
-    divisor, suffix = 1, "B"
-    if size < 1024*1024:
-        divisor, suffix = 1024., "KB"
-    elif size < 1024*1024*1024:
-        divisor, suffix = 1024*1024, "MB"
-    elif size < 1024*1024*1024*1024:
-        divisor, suffix = 1024*1024*1024, "GB"
-    size = str(float(size)/divisor)
-    if size.find(".") > -1:
-        size = size[:size.find(".")+2]
-    if size.endswith('.0'):
-        size = size[:-2]
-    return size + " " + suffix
 
 class Dispatcher(QObject):
-    '''Convenience class to ensure that a function call always happens in the GUI thread'''
-    SIGNAL = SIGNAL('dispatcher(PyQt_PyObject,PyQt_PyObject)')
+    '''
+    Convenience class to use Qt signals with arbitrary python callables.
+    By default, ensures that a function call always happens in the
+    thread this Dispatcher was created in.
+    '''
+    dispatch_signal = pyqtSignal(object, object)
 
-    def __init__(self, func):
-        QObject.__init__(self)
+    def __init__(self, func, queued=True, parent=None):
+        QObject.__init__(self, parent)
         self.func = func
-        self.connect(self, self.SIGNAL, self.dispatch, Qt.QueuedConnection)
+        typ = Qt.QueuedConnection
+        if not queued:
+            typ = Qt.AutoConnection if queued is None else Qt.DirectConnection
+        self.dispatch_signal.connect(self.dispatch, type=typ)
 
     def __call__(self, *args, **kwargs):
-        self.emit(self.SIGNAL, args, kwargs)
+        self.dispatch_signal.emit(args, kwargs)
 
     def dispatch(self, args, kwargs):
         self.func(*args, **kwargs)
@@ -268,24 +330,6 @@ class GetMetadata(QObject):
             mi = MetaInformation('', [_('Unknown')])
         self.emit(SIGNAL('metadata(PyQt_PyObject, PyQt_PyObject)'), id, mi)
 
-class TableView(QTableView):
-    def __init__(self, parent):
-        QTableView.__init__(self, parent)
-        self.read_settings()
-
-    def read_settings(self):
-        self.cw = dynamic[self.__class__.__name__+'column widths']
-
-    def write_settings(self):
-        dynamic[self.__class__.__name__+'column widths'] = \
-         tuple([int(self.columnWidth(i)) for i in range(self.model().columnCount(None))])
-
-    def restore_column_widths(self):
-        if self.cw and len(self.cw):
-            for i in range(len(self.cw)):
-                self.setColumnWidth(i, self.cw[i])
-            return True
-
 class FileIconProvider(QFileIconProvider):
 
     ICONS = {
@@ -307,14 +351,21 @@ class FileIconProvider(QFileIconProvider):
              'lrf'     : 'lrf',
              'lrx'     : 'lrx',
              'pdf'     : 'pdf',
+             'pdr'     : 'zero',
              'rar'     : 'rar',
              'zip'     : 'zip',
              'txt'     : 'txt',
              'prc'     : 'mobi',
              'azw'     : 'mobi',
              'mobi'    : 'mobi',
+             'mbp'     : 'zero',
+             'azw1'    : 'mobi',
+             'tpz'     : 'mobi',
+             'tan'     : 'zero',
              'epub'    : 'epub',
              'fb2'     : 'fb2',
+             'rtf'     : 'rtf',
+             'odt'     : 'odt',
              }
 
     def __init__(self):
@@ -354,7 +405,7 @@ class FileIconProvider(QFileIconProvider):
         if fileinfo.isDir():
             key = 'dir'
         else:
-            ext = qstring_to_unicode(fileinfo.completeSuffix()).lower()
+            ext = unicode(fileinfo.completeSuffix()).lower()
             key = self.key_from_ext(ext)
         return self.cached_icon(key)
 
@@ -375,6 +426,7 @@ def initialize_file_icon_provider():
 
 def file_icon_provider():
     global _file_icon_provider
+    initialize_file_icon_provider()
     return _file_icon_provider
 
 class FileDialog(QObject):
@@ -385,13 +437,15 @@ class FileDialog(QObject):
                        modal = True,
                        name = '',
                        mode = QFileDialog.ExistingFiles,
+                       default_dir='~'
                        ):
         QObject.__init__(self)
         ftext = ''
         if filters:
             for filter in filters:
                 text, extensions = filter
-                extensions = ['*.'+i if not i.startswith('.') else i for i in extensions]
+                extensions = ['*'+(i if i.startswith('.') else '.'+i) for i in
+                        extensions]
                 ftext += '%s (%s);;'%(text, ' '.join(extensions))
         if add_all_files_filter or not ftext:
             ftext += 'All files (*)'
@@ -402,9 +456,10 @@ class FileDialog(QObject):
         self.selected_files = None
         self.fd = None
 
-        initial_dir = dynamic.get(self.dialog_name, os.path.expanduser('~'))
+        initial_dir = dynamic.get(self.dialog_name,
+                os.path.expanduser(default_dir))
         if not isinstance(initial_dir, basestring):
-            initial_dir = os.path.expanduser('~')
+            initial_dir = os.path.expanduser(default_dir)
         self.selected_files = []
         if mode == QFileDialog.AnyFile:
             f = unicode(QFileDialog.getSaveFileName(parent, title, initial_dir, ftext, ""))
@@ -435,13 +490,14 @@ class FileDialog(QObject):
 
     def get_files(self):
         if self.selected_files is None:
-            return tuple(os.path.abspath(qstring_to_unicode(i)) for i in self.fd.selectedFiles())
+            return tuple(os.path.abspath(unicode(i)) for i in self.fd.selectedFiles())
         return tuple(self.selected_files)
 
 
-def choose_dir(window, name, title):
-    fd = FileDialog(title, [], False, window, name=name,
-                    mode=QFileDialog.DirectoryOnly)
+def choose_dir(window, name, title, default_dir='~'):
+    fd = FileDialog(title=title, filters=[], add_all_files_filter=False,
+            parent=window, name=name, mode=QFileDialog.DirectoryOnly,
+            default_dir=default_dir)
     dir = fd.get_files()
     if dir:
         return dir[0]
@@ -485,7 +541,7 @@ def pixmap_to_data(pixmap, format='JPEG'):
     buf = QBuffer(ba)
     buf.open(QBuffer.WriteOnly)
     pixmap.save(buf, format)
-    return str(ba.data())
+    return bytes(ba.data())
 
 class ResizableDialog(QDialog):
 
@@ -503,25 +559,146 @@ class ResizableDialog(QDialog):
 
 gui_thread = None
 
-
+qt_app = None
 class Application(QApplication):
 
     def __init__(self, args):
         qargs = [i.encode('utf-8') if isinstance(i, unicode) else i for i in args]
         QApplication.__init__(self, qargs)
-        global gui_thread
+        self.file_event_hook = None
+        global gui_thread, qt_app
         gui_thread = QThread.currentThread()
+        self._translator = None
+        self.load_translations()
+        qt_app = self
+        self._file_open_paths = []
+        self._file_open_lock = RLock()
+
+        if islinux:
+            self.setStyleSheet('''
+                    QToolTip {
+                        border: 2px solid black;
+                        padding: 5px;
+                        border-radius: 10px;
+                        opacity: 200;
+                        background-color: #e1e1ff;
+                        color: black;
+                    }
+            ''')
+
+    def _send_file_open_events(self):
+        with self._file_open_lock:
+            if self._file_open_paths:
+                self.file_event_hook(self._file_open_paths)
+                self._file_open_paths = []
+
+
+    def load_translations(self):
+        if self._translator is not None:
+            self.removeTranslator(self._translator)
         self._translator = QTranslator(self)
         if set_qt_translator(self._translator):
             self.installTranslator(self._translator)
 
+    def event(self, e):
+        if callable(self.file_event_hook) and e.type() == QEvent.FileOpen:
+            path = unicode(e.file())
+            if os.access(path, os.R_OK):
+                with self._file_open_lock:
+                    self._file_open_paths.append(path)
+                QTimer.singleShot(1000, self._send_file_open_events)
+            return True
+        else:
+            return QApplication.event(self, e)
+
+_store_app = None
+
+def open_url(qurl):
+    paths = os.environ.get('LD_LIBRARY_PATH',
+                '').split(os.pathsep)
+    paths = [x for x in paths if x]
+    if isfrozen and islinux and paths:
+        npaths = [x for x in paths if x != sys.frozen_path]
+        os.environ['LD_LIBRARY_PATH'] = os.pathsep.join(npaths)
+    QDesktopServices.openUrl(qurl)
+    if isfrozen and islinux and paths:
+        os.environ['LD_LIBRARY_PATH'] = os.pathsep.join(paths)
+
+
+def open_local_file(path):
+    if iswindows:
+        os.startfile(os.path.normpath(path))
+    else:
+        url = QUrl.fromLocalFile(path)
+        open_url(url)
+
 def is_ok_to_use_qt():
-    global gui_thread
-    if islinux and ':' not in os.environ.get('DISPLAY', ''):
+    global gui_thread, _store_app
+    if (islinux or isfreebsd) and ':' not in os.environ.get('DISPLAY', ''):
         return False
-    if QApplication.instance() is None:
-        QApplication([])
+    if _store_app is None and QApplication.instance() is None:
+        _store_app = QApplication([])
     if gui_thread is None:
         gui_thread = QThread.currentThread()
     return gui_thread is QThread.currentThread()
 
+def is_gui_thread():
+    global gui_thread
+    return gui_thread is QThread.currentThread()
+
+
+def find_forms(srcdir):
+    base = os.path.join(srcdir, 'calibre', 'gui2')
+    forms = []
+    for root, _, files in os.walk(base):
+        for name in files:
+            if name.endswith('.ui'):
+                forms.append(os.path.abspath(os.path.join(root, name)))
+
+    return forms
+
+def form_to_compiled_form(form):
+    return form.rpartition('.')[0]+'_ui.py'
+
+def build_forms(srcdir, info=None):
+    import re, cStringIO
+    from PyQt4.uic import compileUi
+    forms = find_forms(srcdir)
+    if info is None:
+        from calibre import prints
+        info = prints
+    pat = re.compile(r'''(['"]):/images/([^'"]+)\1''')
+    def sub(match):
+        ans = 'I(%s%s%s)'%(match.group(1), match.group(2), match.group(1))
+        return ans
+
+    for form in forms:
+        compiled_form = form_to_compiled_form(form)
+        if not os.path.exists(compiled_form) or os.stat(form).st_mtime > os.stat(compiled_form).st_mtime:
+            info('\tCompiling form', form)
+            buf = cStringIO.StringIO()
+            compileUi(form, buf)
+            dat = buf.getvalue()
+            dat = dat.replace('__appname__', 'calibre')
+            dat = dat.replace('import images_rc', '')
+            dat = dat.replace('from library import', 'from calibre.gui2.library import')
+            dat = dat.replace('from widgets import', 'from calibre.gui2.widgets import')
+            dat = dat.replace('from convert.xpath_wizard import',
+                'from calibre.gui2.convert.xpath_wizard import')
+            dat = re.compile(r'QtGui.QApplication.translate\(.+?,\s+"(.+?)(?<!\\)",.+?\)', re.DOTALL).sub(r'_("\1")', dat)
+            dat = dat.replace('_("MMM yyyy")', '"MMM yyyy"')
+            dat = pat.sub(sub, dat)
+
+            if form.endswith('viewer%smain.ui'%os.sep):
+                info('\t\tPromoting WebView')
+                dat = dat.replace('self.view = QtWebKit.QWebView(', 'self.view = DocumentView(')
+                dat = dat.replace('self.view = QWebView(', 'self.view = DocumentView(')
+                dat = dat.replace('from QtWebKit.QWebView import QWebView',
+                        'from PyQt4 import QtWebKit\nfrom PyQt4.QtWebKit import QWebView')
+                dat += '\n\nfrom calibre.gui2.viewer.documentview import DocumentView'
+
+            open(compiled_form, 'wb').write(dat)
+
+_df = os.environ.get('CALIBRE_DEVELOP_FROM', None)
+if _df and os.path.exists(_df):
+    build_forms(_df)
