@@ -14,12 +14,14 @@ class PluginWidget(Widget, Ui_Form):
 
     TITLE = _('Comic Input')
     HELP = _('Options specific to')+' comic '+_('input')
+    COMMIT_NAME = 'comic_input'
 
     def __init__(self, parent, get_option, get_help, db=None, book_id=None):
-        Widget.__init__(self, parent, 'comic_input',
+        Widget.__init__(self, parent,
                 ['colors', 'dont_normalize', 'keep_aspect_ratio', 'right2left',
                     'despeckle', 'no_sort', 'no_process', 'landscape',
-                    'dont_sharpen', 'disable_trim', 'wide', 'output_format']
+                    'dont_sharpen', 'disable_trim', 'wide', 'output_format',
+                    'dont_grayscale']
                 )
         self.db, self.book_id = db, book_id
         for x in get_option('output_format').option.choices:
