@@ -11,9 +11,11 @@ class PluginWidget(Widget, Ui_Form):
 
     TITLE = _('TXT Input')
     HELP = _('Options specific to')+' TXT '+_('input')
+    COMMIT_NAME = 'txt_input'
 
     def __init__(self, parent, get_option, get_help, db=None, book_id=None):
-        Widget.__init__(self, parent, 'txt_input',
-            ['single_line_paras', 'print_formatted_paras', 'markdown'])
+        Widget.__init__(self, parent,
+            ['single_line_paras', 'print_formatted_paras', 'markdown',
+                'markdown_disable_toc', 'preserve_spaces'])
         self.db, self.book_id = db, book_id
         self.initialize_options(get_option, get_help, db, book_id)
