@@ -1,25 +1,28 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
-import os, re, time, textwrap
+
+import os, re, time, textwrap, copy, sys
 
 from PyQt4.Qt import    QDialog, QListWidgetItem, QIcon, \
-                        QDesktopServices, QVBoxLayout, QLabel, QPlainTextEdit, \
+                        QVBoxLayout, QLabel, QPlainTextEdit, \
                         QStringListModel, QAbstractItemModel, QFont, \
                         SIGNAL, QThread, Qt, QSize, QVariant, QUrl, \
-                        QModelIndex, QInputDialog, QAbstractTableModel, \
+                        QModelIndex, QAbstractTableModel, \
                         QDialogButtonBox, QTabWidget, QBrush, QLineEdit, \
                         QProgressDialog
 
 from calibre.constants import iswindows, isosx
 from calibre.gui2.dialogs.config.config_ui import Ui_Dialog
-from calibre.gui2 import qstring_to_unicode, choose_dir, error_dialog, config, \
-                         ALL_COLUMNS, NONE, info_dialog, choose_files, \
-                         warning_dialog
-from calibre.utils.config import prefs
-from calibre.gui2.library import BooksModel
+from calibre.gui2.dialogs.config.create_custom_column import CreateCustomColumn
+from calibre.gui2.dialogs.config.toolbar import ToolbarLayout
+from calibre.gui2 import error_dialog, config, gprefs, \
+        open_url, open_local_file, \
+        ALL_COLUMNS, NONE, info_dialog, choose_files, \
+        warning_dialog, ResizableDialog, question_dialog
+from calibre.utils.config import prefs, read_raw_tweaks, write_tweaks
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.oeb.iterator import is_supported
-from calibre.library import server_config
+from calibre.library.server import server_config
 from calibre.customize.ui import initialized_plugins, is_disabled, enable_plugin, \
                                  disable_plugin, customize_plugin, \
                                  plugin_customization, add_plugin, \
@@ -27,12 +30,15 @@ from calibre.customize.ui import initialized_plugins, is_disabled, enable_plugin
                                  input_format_plugins, \
                                  output_format_plugins, available_output_formats
 from calibre.utils.smtp import config as smtp_prefs
+from calibre.gui2.convert import config_widget_for_input_plugin
 from calibre.gui2.convert.look_and_feel import LookAndFeelWidget
 from calibre.gui2.convert.page_setup import PageSetupWidget
 from calibre.gui2.convert.structure_detection import StructureDetectionWidget
 from calibre.ebooks.conversion.plumber import Plumber
 from calibre.utils.logging import Log
 from calibre.gui2.convert.toc import TOCWidget
+from calibre.utils.search_query_parser import saved_searches
+
 
 class ConfigTabs(QTabWidget):
 
@@ -56,15 +62,10 @@ class ConfigTabs(QTabWidget):
         self.widgets = [lf, ps, sd, toc]
 
         for plugin in input_format_plugins():
-            name = plugin.name.lower().replace(' ', '_')
-            try:
-                input_widget = __import__('calibre.gui2.convert.'+name,
-                        fromlist=[1])
-                pw = input_widget.PluginWidget
+            pw = config_widget_for_input_plugin(plugin)
+            if pw is not None:
                 pw.ICON = I('forward.svg')
                 self.widgets.append(widget_factory(pw))
-            except ImportError:
-                continue
 
         for plugin in output_format_plugins():
             name = plugin.name.lower().replace(' ', '_')
@@ -90,7 +91,6 @@ class ConfigTabs(QTabWidget):
             widget.commit(save_defaults=True)
         return True
 
-
 class PluginModel(QAbstractItemModel):
 
     def __init__(self, *args):
@@ -110,6 +110,9 @@ class PluginModel(QAbstractItemModel):
                 self._data[plugin.type].append(plugin)
         self.categories = sorted(self._data.keys())
 
+        for plugins in self._data.values():
+            plugins.sort(cmp=lambda x, y: cmp(x.name.lower(), y.name.lower()))
+
     def index(self, row, column, parent):
         if not self.hasIndex(row, column, parent):
             return QModelIndex()
@@ -177,7 +180,7 @@ class PluginModel(QAbstractItemModel):
                 ans='%s (%s) %s %s\n%s'%(plugin.name, ver, _('by'), plugin.author, desc)
                 c = plugin_customization(plugin)
                 if c:
-                    ans += '\nCustomization: '+c
+                    ans += _('\nCustomization: ')+c
                 return QVariant(ans)
             if role == Qt.DecorationRole:
                 return self.disabled_icon if is_disabled(plugin) else self.icon
@@ -191,22 +194,32 @@ class PluginModel(QAbstractItemModel):
 
 class CategoryModel(QStringListModel):
 
+    CATEGORIES = [
+             ('general', _('General'), 'dialog_information.svg'),
+             ('interface', _('Interface'), 'lookfeel.svg'),
+             ('conversion', _('Conversion'), 'convert.svg'),
+             ('email', _('Email\nDelivery'), 'mail.svg'),
+             ('add/save', _('Add/Save'), 'save.svg'),
+             ('advanced', _('Advanced'), 'view.svg'),
+             ('server', _('Content\nServer'), 'network-server.svg'),
+             ('plugins', _('Plugins'), 'plugins.svg'),
+    ]
+
     def __init__(self, *args):
         QStringListModel.__init__(self, *args)
-        self.setStringList([_('General'), _('Interface'), _('Conversion'),
-                            _('Email\nDelivery'), _('Add/Save'),
-                            _('Advanced'), _('Content\nServer'), _('Plugins')])
-        self.icons = list(map(QVariant, map(QIcon,
-            [I('dialog_information.svg'), I('lookfeel.svg'),
-                I('convert.svg'),
-                I('mail.svg'), I('save.svg'), I('view.svg'),
-             I('network-server.svg'), I('plugins.svg')])))
+        self.setStringList([x[1] for x in self.CATEGORIES])
 
     def data(self, index, role):
         if role == Qt.DecorationRole:
-            return self.icons[index.row()]
+            return QVariant(QIcon(I(self.CATEGORIES[index.row()][2])))
         return QStringListModel.data(self, index, role)
 
+    def index_for_name(self, name):
+        for i, x in enumerate(self.CATEGORIES):
+            if x[0] == name:
+                return self.index(i)
+        return self.index(0)
+
 class EmailAccounts(QAbstractTableModel):
 
     def __init__(self, accounts):
@@ -323,28 +336,28 @@ class EmailAccounts(QAbstractTableModel):
             self.reset()
 
 
-class ConfigDialog(QDialog, Ui_Dialog):
+class ConfigDialog(ResizableDialog, Ui_Dialog):
 
-    def __init__(self, window, db, server=None):
-        QDialog.__init__(self, window)
-        Ui_Dialog.__init__(self)
-        self.ICON_SIZES = {0:QSize(48, 48), 1:QSize(32,32), 2:QSize(24,24)}
-        self.setupUi(self)
+    def category_current_changed(self, n, p):
+        self.stackedWidget.setCurrentIndex(n.row())
+
+    def __init__(self, parent, library_view, server=None,
+            initial_category='general'):
+        ResizableDialog.__init__(self, parent)
         self._category_model = CategoryModel()
 
-        self.category_view.currentChanged = \
-            lambda n, p: self.stackedWidget.setCurrentIndex(n.row())
+        self.category_view.currentChanged = self.category_current_changed
         self.category_view.setModel(self._category_model)
-        self.db = db
+        self.parent = parent
+        self.library_view = library_view
+        self.model = library_view.model()
+        self.db = self.model.db
         self.server = server
-        path = prefs['library_path']
-        self.location.setText(path if path else '')
-        self.connect(self.browse_button, SIGNAL('clicked(bool)'), self.browse)
         self.connect(self.compact_button, SIGNAL('clicked(bool)'), self.compact)
 
         input_map = prefs['input_format_order']
         all_formats = set()
-        for fmt in all_input_formats():
+        for fmt in all_input_formats().union(set(['ZIP', 'RAR'])):
             all_formats.add(fmt.upper())
         for format in input_map + list(all_formats.difference(input_map)):
             item = QListWidgetItem(format, self.input_order)
@@ -359,19 +372,28 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.roman_numerals.setChecked(rn)
         self.new_version_notification.setChecked(config['new_version_notification'])
 
-        column_map = config['column_map']
-        for col in column_map + [i for i in ALL_COLUMNS if i not in column_map]:
-            item = QListWidgetItem(BooksModel.headers[col], self.columns)
+        # Set up columns
+        colmap = list(self.model.column_map)
+        state = self.library_view.get_state()
+        hidden_cols = state['hidden_columns']
+        positions = state['column_positions']
+        colmap.sort(cmp=lambda x,y: cmp(positions[x], positions[y]))
+        self.custcols = copy.deepcopy(self.db.field_metadata.get_custom_field_metadata())
+        for col in colmap:
+            item = QListWidgetItem(self.model.headers[col], self.columns)
             item.setData(Qt.UserRole, QVariant(col))
-            item.setFlags(Qt.ItemIsEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsSelectable)
-            item.setCheckState(Qt.Checked if col in column_map else Qt.Unchecked)
-
-        self.connect(self.column_up, SIGNAL('clicked()'), self.up_column)
-        self.connect(self.column_down, SIGNAL('clicked()'), self.down_column)
-
-        icons = config['toolbar_icon_size']
-        self.toolbar_button_size.setCurrentIndex(0 if icons == self.ICON_SIZES[0] else 1 if icons == self.ICON_SIZES[1] else 2)
-        self.show_toolbar_text.setChecked(config['show_text_in_toolbar'])
+            flags = Qt.ItemIsEnabled|Qt.ItemIsSelectable
+            if col != 'ondevice':
+                flags |= Qt.ItemIsUserCheckable
+            item.setFlags(flags)
+            if col != 'ondevice':
+                item.setCheckState(Qt.Unchecked if col in hidden_cols else
+                        Qt.Checked)
+        self.column_up.clicked.connect(self.up_column)
+        self.column_down.clicked.connect(self.down_column)
+        self.del_custcol_button.clicked.connect(self.del_custcol)
+        self.add_custcol_button.clicked.connect(self.add_custcol)
+        self.edit_custcol_button.clicked.connect(self.edit_custcol)
 
         output_formats = sorted(available_output_formats())
         output_formats.remove('oeb')
@@ -398,7 +420,6 @@ class ConfigDialog(QDialog, Ui_Dialog):
         for item in items:
             self.language.addItem(item[1], QVariant(item[0]))
 
-
         exts = set([])
         for ext in BOOK_EXTENSIONS:
             ext = ext.lower()
@@ -427,6 +448,7 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.username.setText(opts.username)
         self.password.setText(opts.password if opts.password else '')
         self.opt_max_opds_items.setValue(opts.max_opds_items)
+        self.opt_max_opds_ungrouped_items.setValue(opts.max_opds_ungrouped_items)
         self.auto_launch.setChecked(config['autolaunch_server'])
         self.systray_icon.setChecked(config['systray_icon'])
         self.sync_news.setChecked(config['upload_news_to_device'])
@@ -435,8 +457,11 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.priority.setCurrentIndex(p)
         self.priority.setVisible(iswindows)
         self.priority_label.setVisible(iswindows)
+        self.new_book_tags.setText(', '.join(prefs['new_book_tags']))
         self._plugin_model = PluginModel()
         self.plugin_view.setModel(self._plugin_model)
+        self.plugin_view.setStyleSheet(
+                "QTreeView::item { padding-bottom: 10px;}")
         self.connect(self.toggle_plugin, SIGNAL('clicked()'), lambda : self.modify_plugin(op='toggle'))
         self.connect(self.customize_plugin, SIGNAL('clicked()'), lambda : self.modify_plugin(op='customize'))
         self.connect(self.remove_plugin, SIGNAL('clicked()'), lambda : self.modify_plugin(op='remove'))
@@ -447,12 +472,96 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.button_osx_symlinks.setVisible(isosx)
         self.separate_cover_flow.setChecked(config['separate_cover_flow'])
         self.setup_email_page()
-        self.category_view.setCurrentIndex(self.category_view.model().index(0))
         self.delete_news.setEnabled(bool(self.sync_news.isChecked()))
         self.connect(self.sync_news, SIGNAL('toggled(bool)'),
                 self.delete_news.setEnabled)
         self.setup_conversion_options()
         self.opt_worker_limit.setValue(config['worker_limit'])
+        self.connect(self.button_open_config_dir, SIGNAL('clicked()'),
+                self.open_config_dir)
+        self.opt_get_social_metadata.setChecked(config['get_social_metadata'])
+        self.opt_overwrite_author_title_metadata.setChecked(config['overwrite_author_title_metadata'])
+        self.opt_enforce_cpu_limit.setChecked(config['enforce_cpu_limit'])
+        self.device_detection_button.clicked.connect(self.debug_device_detection)
+        self.port.editingFinished.connect(self.check_port_value)
+        self.search_as_you_type.setChecked(config['search_as_you_type'])
+        self.show_avg_rating.setChecked(config['show_avg_rating'])
+        self.show_splash_screen.setChecked(gprefs.get('show_splash_screen',
+            True))
+        li = None
+        for i, z in enumerate([('wide', _('Wide')),
+            ('narrow', _('Narrow'))]):
+            x, y = z
+            self.opt_gui_layout.addItem(y, QVariant(x))
+            if x == config['gui_layout']:
+                li = i
+        self.opt_gui_layout.setCurrentIndex(li)
+        restrictions = sorted(saved_searches().names(),
+                              cmp=lambda x,y: cmp(x.lower(), y.lower()))
+        restrictions.insert(0, '')
+        for x in ('gui', 'cs'):
+            w = getattr(self, 'opt_%s_restriction'%x)
+            w.addItems(restrictions)
+            idx = w.findText(self.db.prefs.get(x+'_restriction', ''))
+            w.setCurrentIndex(0 if idx < 0 else idx)
+        self.opt_disable_animations.setChecked(config['disable_animations'])
+        idx = 0
+        for i, x in enumerate([(_('Small'), 'small'), (_('Medium'), 'medium'),
+            (_('Large'), 'large')]):
+            if x[1] == gprefs.get('toolbar_icon_size', 'medium'):
+                idx = i
+            self.opt_toolbar_icon_size.addItem(x[0], x[1])
+        self.opt_toolbar_icon_size.setCurrentIndex(idx)
+        idx = 0
+        for i, x in enumerate([(_('Automatic'), 'auto'), (_('Always'), 'always'),
+            (_('Never'), 'never')]):
+            if x[1] == gprefs.get('toolbar_text', 'auto'):
+                idx = i
+            self.opt_toolbar_text.addItem(x[0], x[1])
+        self.opt_toolbar_text.setCurrentIndex(idx)
+        self.reset_confirmation_button.clicked.connect(self.reset_confirmation)
+
+        deft, curt = read_raw_tweaks()
+        self.current_tweaks.setPlainText(curt.decode('utf-8'))
+        self.default_tweaks.setPlainText(deft.decode('utf-8'))
+        self.restore_tweaks_to_default_button.clicked.connect(self.restore_tweaks_to_default)
+        self.toolbar_cm_widget = ToolbarLayout(parent, parent)
+        self.toolbar_cm_tab.addTab(self.toolbar_cm_widget,
+                _('Toolbars/Context menus'))
+
+        self.category_view.setCurrentIndex(self.category_view.model().index_for_name(initial_category))
+
+    def restore_tweaks_to_default(self, *args):
+        deft, curt = read_raw_tweaks()
+        self.current_tweaks.setPlainText(deft.decode('utf-8'))
+
+
+    def reset_confirmation(self):
+        from calibre.gui2 import dynamic
+        for key in dynamic.keys():
+            if key.endswith('_again') and dynamic[key] is False:
+                dynamic[key] = True
+        info_dialog(self, _('Done'),
+                _('Confirmation dialogs have all been reset'), show=True)
+
+    def check_port_value(self, *args):
+        port = self.port.value()
+        if port < 1025:
+            warning_dialog(self, _('System port selected'), '<p>'+
+                    _('The value <b>%d</b> you have chosen for the content '
+                        'server port is a system port. Your operating '
+                        'system <b>may</b> not allow the server to run on this '
+                        'port. To be safe choose a port number larger than '
+                        '1024.')%port, show=True)
+
+    def debug_device_detection(self):
+        from calibre.gui2.dialogs.config.device_debug import DebugDevice
+        d = DebugDevice(self)
+        d.exec_()
+
+    def open_config_dir(self):
+        from calibre.utils.config import config_dir
+        open_local_file(config_dir)
 
     def create_symlinks(self):
         from calibre.utils.osx_symlinks import create_symlinks
@@ -544,15 +653,16 @@ class ConfigDialog(QDialog, Ui_Dialog):
                     info_dialog(self, _('Plugin not customizable'),
                         _('Plugin: %s does not need customization')%plugin.name).exec_()
                     return
+                config_dialog = QDialog(self)
+                button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+                v = QVBoxLayout(config_dialog)
+
+                config_dialog.connect(button_box, SIGNAL('accepted()'), config_dialog.accept)
+                config_dialog.connect(button_box, SIGNAL('rejected()'), config_dialog.reject)
+                config_dialog.setWindowTitle(_('Customize') + ' ' + plugin.name)
+
                 if hasattr(plugin, 'config_widget'):
-                    config_dialog = QDialog(self)
-                    button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
-
-                    config_dialog.connect(button_box, SIGNAL('accepted()'), config_dialog.accept)
-                    config_dialog.connect(button_box, SIGNAL('rejected()'), config_dialog.reject)
-
                     config_widget = plugin.config_widget()
-                    v = QVBoxLayout(config_dialog)
                     v.addWidget(config_widget)
                     v.addWidget(button_box)
                     config_dialog.exec_()
@@ -561,17 +671,28 @@ class ConfigDialog(QDialog, Ui_Dialog):
                         plugin.save_settings(config_widget)
                         self._plugin_model.refresh_plugin(plugin)
                 else:
-                    help = plugin.customization_help()
+                    help_text = plugin.customization_help(gui=True)
+                    help_text = QLabel(help_text, config_dialog)
+                    help_text.setWordWrap(True)
+                    help_text.setTextInteractionFlags(Qt.LinksAccessibleByMouse
+                            | Qt.LinksAccessibleByKeyboard)
+                    help_text.setOpenExternalLinks(True)
+                    v.addWidget(help_text)
                     sc = plugin_customization(plugin)
                     if not sc:
                         sc = ''
                     sc = sc.strip()
-                    text, ok = QInputDialog.getText(self, _('Customize %s')%plugin.name,
-                                                    help, QLineEdit.Normal, sc)
-                    if ok:
-                        customize_plugin(plugin, unicode(text).strip())
+                    sc = QLineEdit(sc, config_dialog)
+                    v.addWidget(sc)
+                    v.addWidget(button_box)
+                    config_dialog.exec_()
+
+                    if config_dialog.result() == QDialog.Accepted:
+                        sc = unicode(sc.text()).strip()
+                        customize_plugin(plugin, sc)
+
                     self._plugin_model.refresh_plugin(plugin)
-            if op == 'remove':
+            elif op == 'remove':
                 if remove_plugin(plugin):
                     self._plugin_model.populate()
                     self._plugin_model.reset()
@@ -586,12 +707,28 @@ class ConfigDialog(QDialog, Ui_Dialog):
             self.input_order.insertItem(idx-1, self.input_order.takeItem(idx))
             self.input_order.setCurrentRow(idx-1)
 
+    def set_tweaks(self):
+        raw = unicode(self.current_tweaks.toPlainText()).encode('utf-8')
+        try:
+            exec raw
+        except:
+            import traceback
+            error_dialog(self, _('Invalid tweaks'),
+                    _('The tweaks you entered are invalid, try resetting the'
+                        ' tweaks to default and changing them one by one until'
+                        ' you find the invalid setting.'),
+                    det_msg=traceback.format_exc(), show=True)
+            return False
+        write_tweaks(raw)
+        return True
+
     def down_input(self):
         idx = self.input_order.currentRow()
         if idx < self.input_order.count()-1:
             self.input_order.insertItem(idx+1, self.input_order.takeItem(idx))
             self.input_order.setCurrentRow(idx+1)
 
+    # Column settings {{{
     def up_column(self):
         idx = self.columns.currentRow()
         if idx > 0:
@@ -604,6 +741,77 @@ class ConfigDialog(QDialog, Ui_Dialog):
             self.columns.insertItem(idx+1, self.columns.takeItem(idx))
             self.columns.setCurrentRow(idx+1)
 
+    def del_custcol(self):
+        idx = self.columns.currentRow()
+        if idx < 0:
+            return error_dialog(self, '', _('You must select a column to delete it'),
+                    show=True)
+        col = unicode(self.columns.item(idx).data(Qt.UserRole).toString())
+        if col not in self.custcols:
+            return error_dialog(self, '',
+                    _('The selected column is not a custom column'), show=True)
+        if not question_dialog(self, _('Are you sure?'),
+            _('Do you really want to delete column %s and all its data?') %
+            self.custcols[col]['name'], show_copy_button=False):
+            return
+        self.columns.item(idx).setCheckState(False)
+        self.columns.takeItem(idx)
+        self.custcols[col]['*deleteme'] = True
+        return
+
+    def add_custcol(self):
+        CreateCustomColumn(self, False, self.model.orig_headers, ALL_COLUMNS)
+
+    def edit_custcol(self):
+        CreateCustomColumn(self, True, self.model.orig_headers, ALL_COLUMNS)
+
+    def apply_custom_column_changes(self):
+        config_cols = [unicode(self.columns.item(i).data(Qt.UserRole).toString())\
+                 for i in range(self.columns.count())]
+        if not config_cols:
+            config_cols = ['title']
+        removed_cols = set(self.model.column_map) - set(config_cols)
+        hidden_cols = set([unicode(self.columns.item(i).data(Qt.UserRole).toString())\
+                 for i in range(self.columns.count()) \
+                 if self.columns.item(i).checkState()==Qt.Unchecked])
+        hidden_cols = hidden_cols.union(removed_cols) # Hide removed cols
+        hidden_cols = list(hidden_cols.intersection(set(self.model.column_map)))
+        if 'ondevice' in hidden_cols:
+            hidden_cols.remove('ondevice')
+        def col_pos(x, y):
+            xidx = config_cols.index(x) if x in config_cols else sys.maxint
+            yidx = config_cols.index(y) if y in config_cols else sys.maxint
+            return cmp(xidx, yidx)
+        positions = {}
+        for i, col in enumerate((sorted(self.model.column_map, cmp=col_pos))):
+            positions[col] = i
+        state = {'hidden_columns': hidden_cols, 'column_positions':positions}
+        self.library_view.apply_state(state)
+        self.library_view.save_state()
+
+        must_restart = False
+        for c in self.custcols:
+            if self.custcols[c]['colnum'] is None:
+                self.db.create_custom_column(
+                                label=self.custcols[c]['label'],
+                                name=self.custcols[c]['name'],
+                                datatype=self.custcols[c]['datatype'],
+                                is_multiple=self.custcols[c]['is_multiple'],
+                                display = self.custcols[c]['display'])
+                must_restart = True
+            elif '*deleteme' in self.custcols[c]:
+                self.db.delete_custom_column(label=self.custcols[c]['label'])
+                must_restart = True
+            elif '*edited' in self.custcols[c]:
+                cc = self.custcols[c]
+                self.db.set_custom_column_metadata(cc['colnum'], name=cc['name'],
+                                                   label=cc['label'],
+                                                   display = self.custcols[c]['display'])
+                if '*must_restart' in self.custcols[c]:
+                    must_restart = True
+        return must_restart
+    # }}}
+
     def view_server_logs(self):
         from calibre.library.server import log_access_file, log_error_file
         d = QDialog(self)
@@ -623,7 +831,10 @@ class ConfigDialog(QDialog, Ui_Dialog):
         try:
             al.setPlainText(open(log_access_file, 'rb').read().decode('utf8', 'replace'))
         except IOError:
-            el.setPlainText('No access log found')
+            al.setPlainText('No access log found')
+        bx = QDialogButtonBox(QDialogButtonBox.Ok)
+        layout.addWidget(bx)
+        self.connect(bx, SIGNAL('accepted()'), d.accept)
         d.show()
 
     def set_server_options(self):
@@ -637,7 +848,7 @@ class ConfigDialog(QDialog, Ui_Dialog):
 
     def start_server(self):
         self.set_server_options()
-        from calibre.library.server import start_threaded_server
+        from calibre.library.server.main import start_threaded_server
         self.server = start_threaded_server(self.db, server_config().parse())
         while not self.server.is_running and self.server.exception is None:
             time.sleep(1)
@@ -650,7 +861,7 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.stop.setEnabled(True)
 
     def stop_server(self):
-        from calibre.library.server import stop_threaded_server
+        from calibre.library.server.main import stop_threaded_server
         stop_threaded_server(self.server)
         self.server = None
         self.start.setEnabled(True)
@@ -658,19 +869,12 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.stop.setEnabled(False)
 
     def test_server(self):
-        QDesktopServices.openUrl(QUrl('http://127.0.0.1:'+str(self.port.value())))
+        open_url(QUrl('http://127.0.0.1:'+str(self.port.value())))
 
     def compact(self, toggled):
         d = CheckIntegrity(self.db, self)
         d.exec_()
 
-    def browse(self):
-        dir = choose_dir(self, 'database location dialog',
-                         _('Select location for books'))
-        if dir:
-            self.location.setText(dir)
-
-
     def accept(self):
         mcs = unicode(self.max_cover_size.text()).strip()
         if not re.match(r'\d+x\d+', mcs):
@@ -683,28 +887,29 @@ class ConfigDialog(QDialog, Ui_Dialog):
             return
         if not self.add_save.save_settings():
             return
+        if not self.set_tweaks():
+            return
         wl = self.opt_worker_limit.value()
         if wl%2 != 0:
             wl += 1
+        self.toolbar_cm_widget.commit()
         config['worker_limit'] = wl
 
-
         config['use_roman_numerals_for_series_number'] = bool(self.roman_numerals.isChecked())
         config['new_version_notification'] = bool(self.new_version_notification.isChecked())
         prefs['network_timeout'] = int(self.timeout.value())
-        path = qstring_to_unicode(self.location.text())
         input_cols = [unicode(self.input_order.item(i).data(Qt.UserRole).toString()) for i in range(self.input_order.count())]
         prefs['input_format_order'] = input_cols
-        cols = [unicode(self.columns.item(i).data(Qt.UserRole).toString()) for i in range(self.columns.count()) if self.columns.item(i).checkState()==Qt.Checked]
-        if not cols:
-            cols = ['title']
-        config['column_map'] = cols
-        config['toolbar_icon_size'] = self.ICON_SIZES[self.toolbar_button_size.currentIndex()]
-        config['show_text_in_toolbar'] = bool(self.show_toolbar_text.isChecked())
+
+        must_restart = self.apply_custom_column_changes()
+
         config['separate_cover_flow'] = bool(self.separate_cover_flow.isChecked())
         config['disable_tray_notification'] = not self.systray_notifications.isChecked()
         p = {0:'normal', 1:'high', 2:'low'}[self.priority.currentIndex()]
         prefs['worker_process_priority'] = p
+        nbt = [x.strip() for x in
+            unicode(self.new_book_tags.text()).strip().split(',')]
+        prefs['new_book_tags'] = [x for x in nbt if x]
         prefs['output_format'] = unicode(self.output_format.currentText()).upper()
         config['cover_flow_queue_length'] = self.cover_browse.value()
         prefs['language'] = str(self.language.itemData(self.language.currentIndex()).toString())
@@ -716,27 +921,39 @@ class ConfigDialog(QDialog, Ui_Dialog):
         sc.set('port', self.port.value())
         sc.set('max_cover', mcs)
         sc.set('max_opds_items', self.opt_max_opds_items.value())
+        sc.set('max_opds_ungrouped_items',
+                self.opt_max_opds_ungrouped_items.value())
         config['delete_news_from_library_on_upload'] = self.delete_news.isChecked()
         config['upload_news_to_device'] = self.sync_news.isChecked()
         config['search_as_you_type'] = self.search_as_you_type.isChecked()
+        config['show_avg_rating'] = self.show_avg_rating.isChecked()
+        config['get_social_metadata'] = self.opt_get_social_metadata.isChecked()
+        config['overwrite_author_title_metadata'] = self.opt_overwrite_author_title_metadata.isChecked()
+        config['enforce_cpu_limit'] = bool(self.opt_enforce_cpu_limit.isChecked())
+        config['disable_animations'] = bool(self.opt_disable_animations.isChecked())
+        gprefs['show_splash_screen'] = bool(self.show_splash_screen.isChecked())
+        for x in ('toolbar_icon_size', 'toolbar_text'):
+            w = getattr(self, 'opt_'+x)
+            data = w.itemData(w.currentIndex()).toString()
+            gprefs[x] = unicode(data)
         fmts = []
         for i in range(self.viewer.count()):
             if self.viewer.item(i).checkState() == Qt.Checked:
                 fmts.append(str(self.viewer.item(i).text()))
         config['internally_viewed_formats'] = fmts
+        val = self.opt_gui_layout.itemData(self.opt_gui_layout.currentIndex()).toString()
+        config['gui_layout'] = unicode(val)
+        for x in ('gui', 'cs'):
+            w = getattr(self, 'opt_%s_restriction'%x)
+            self.db.prefs.set(x+'_restriction', unicode(w.currentText()))
 
-        if not path or not os.path.exists(path) or not os.path.isdir(path):
-            d = error_dialog(self, _('Invalid database location'),
-                             _('Invalid database location ')+path+
-                             _('<br>Must be a directory.'))
-            d.exec_()
-        elif not os.access(path, os.W_OK):
-            d = error_dialog(self, _('Invalid database location'),
-                     _('Invalid database location.<br>Cannot write to ')+path)
-            d.exec_()
-        else:
-            self.database_location = os.path.abspath(path)
-            QDialog.accept(self)
+        if must_restart:
+            warning_dialog(self, _('Must restart'),
+                    _('The changes you made require that Calibre be '
+                        'restarted. Please restart as soon as practical.'),
+                    show=True, show_copy_button=False)
+            self.parent.must_restart_before_config = True
+        QDialog.accept(self)
 
 class VacThread(QThread):
 
@@ -807,6 +1024,5 @@ if __name__ == '__main__':
     from PyQt4.Qt import QApplication
     app = QApplication([])
     d=ConfigDialog(None, LibraryDatabase2('/tmp'))
-    d.category_view.setCurrentIndex(d.category_view.model().index(2))
     d.show()
     app.exec_()
