@@ -10,24 +10,29 @@ import os
 import re
 import time
 import traceback
-from datetime import datetime
 
-from PyQt4.QtCore import SIGNAL, QObject, QCoreApplication, Qt, QTimer, QThread, QDate
-from PyQt4.QtGui import QPixmap, QListWidgetItem, QErrorMessage, QDialog
+from PyQt4.Qt import SIGNAL, QObject, Qt, QTimer, QThread, QDate, \
+    QPixmap, QListWidgetItem, QDialog, pyqtSignal
 
-from calibre.gui2 import qstring_to_unicode, error_dialog, file_icon_provider, \
-                           choose_files, choose_images, ResizableDialog
+from calibre.gui2 import error_dialog, file_icon_provider, dynamic, \
+                           choose_files, choose_images, ResizableDialog, \
+                           warning_dialog
 from calibre.gui2.dialogs.metadata_single_ui import Ui_MetadataSingleDialog
 from calibre.gui2.dialogs.fetch_metadata import FetchMetadata
 from calibre.gui2.dialogs.tag_editor import TagEditor
 from calibre.gui2.widgets import ProgressIndicator
 from calibre.ebooks import BOOK_EXTENSIONS
-from calibre.ebooks.metadata import authors_to_sort_string, string_to_authors, authors_to_string
-from calibre.ebooks.metadata.library_thing import cover_from_isbn
-from calibre import islinux
+from calibre.ebooks.metadata import string_to_authors, \
+        authors_to_string, check_isbn
+from calibre.ebooks.metadata.covers import download_cover
 from calibre.ebooks.metadata.meta import get_metadata
-from calibre.utils.config import prefs
-from calibre.customize.ui import run_plugins_on_import
+from calibre.ebooks.metadata import MetaInformation
+from calibre.utils.config import prefs, tweaks
+from calibre.utils.date import qt_to_dt, local_tz, utcfromtimestamp
+from calibre.customize.ui import run_plugins_on_import, get_isbndb_key
+from calibre.gui2.dialogs.config.social import SocialMetadata
+from calibre.gui2.custom_column_widgets import populate_metadata_page
+from calibre import strftime
 
 class CoverFetcher(QThread):
 
@@ -44,13 +49,14 @@ class CoverFetcher(QThread):
 
     def run(self):
         try:
+            au = self.author if self.author else None
+            mi = MetaInformation(self.title, [au])
             if not self.isbn:
                 from calibre.ebooks.metadata.fetch import search
                 if not self.title:
                     self.needs_isbn = True
                     return
-                au = self.author if self.author else None
-                key = prefs['isbndb_com_key']
+                key = get_isbndb_key()
                 if not key:
                     key = None
                 results = search(title=self.title, author=au,
@@ -62,8 +68,10 @@ class CoverFetcher(QThread):
                     return
                 self.isbn = results[0]
 
-            self.cover_data = cover_from_isbn(self.isbn, timeout=self.timeout,
-                    username=self.username, password=self.password)[0]
+            mi.isbn = self.isbn
+
+            self.cover_data, self.errors = download_cover(mi,
+                    timeout=self.timeout)
         except Exception, e:
             self.exception = e
             self.traceback = traceback.format_exc()
@@ -72,20 +80,29 @@ class CoverFetcher(QThread):
 
 
 class Format(QListWidgetItem):
-    def __init__(self, parent, ext, size, path=None):
+
+    def __init__(self, parent, ext, size, path=None, timestamp=None):
         self.path = path
         self.ext = ext
         self.size = float(size)/(1024*1024)
         text = '%s (%.2f MB)'%(self.ext.upper(), self.size)
         QListWidgetItem.__init__(self, file_icon_provider().icon_from_ext(ext),
                                  text, parent, QListWidgetItem.UserType)
+        if timestamp is not None:
+            ts = timestamp.astimezone(local_tz)
+            t = strftime('%a, %d %b %Y [%H:%M:%S]', ts.timetuple())
+            text = _('Last modified: %s')%t
+            self.setToolTip(text)
+            self.setStatusTip(text)
+
 
 class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
     COVER_FETCH_TIMEOUT = 240 # seconds
+    view_format = pyqtSignal(object)
 
     def do_reset_cover(self, *args):
-        pix = QPixmap(I('book.svg'))
+        pix = QPixmap(I('default_cover.svg'))
         self.cover.setPixmap(pix)
         self.cover_changed = True
         self.cover_data = None
@@ -99,7 +116,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         if _file:
             _file = os.path.abspath(_file)
             if not os.access(_file, os.R_OK):
-                d = error_dialog(self.window, _('Cannot read'),
+                d = error_dialog(self, _('Cannot read'),
                         _('You do not have permission to read the file: ') + _file)
                 d.exec_()
                 return
@@ -108,14 +125,14 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 cf = open(_file, "rb")
                 cover = cf.read()
             except IOError, e:
-                d = error_dialog(self.window, _('Error reading file'),
+                d = error_dialog(self, _('Error reading file'),
                         _("<p>There was an error reading from file: <br /><b>") + _file + "</b></p><br />"+str(e))
                 d.exec_()
             if cover:
                 pix = QPixmap()
                 pix.loadFromData(cover)
                 if pix.isNull():
-                    d = error_dialog(self.window,
+                    d = error_dialog(self,
                         _("Not a valid picture"),
                             _file + _(" is not a valid picture"))
                     d.exec_()
@@ -126,58 +143,101 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                     self.cpixmap = pix
                     self.cover_data = cover
 
+    def generate_cover(self, *args):
+        from calibre.utils.magick.draw import create_cover_page, TextLine
+        title = unicode(self.title.text()).strip()
+        author = unicode(self.authors.text()).strip()
+        if not title or not author:
+            return error_dialog(self, _('Specify title and author'),
+                    _('You must specify a title and author before generating '
+                        'a cover'), show=True)
+        lines = [TextLine(title, 44), TextLine(author, 32)]
+        self.cover_data = create_cover_page(lines, I('library.png'))
+        pix = QPixmap()
+        pix.loadFromData(self.cover_data)
+        self.cover.setPixmap(pix)
+        self.cover_changed = True
+        self.cpixmap = pix
 
     def add_format(self, x):
         files = choose_files(self, 'add formats dialog',
-                             "Choose formats for " + qstring_to_unicode((self.title.text())),
-                             [('Books', BOOK_EXTENSIONS)])
-        if not files:
-            return
-        for _file in files:
+                             _("Choose formats for ") + unicode((self.title.text())),
+                             [(_('Books'), BOOK_EXTENSIONS)])
+        self._add_formats(files)
+
+    def _add_formats(self, paths):
+        added = False
+        if not paths:
+            return added
+        bad_perms = []
+        for _file in paths:
             _file = os.path.abspath(_file)
             if not os.access(_file, os.R_OK):
-                QErrorMessage(self.window).showMessage("You do not have "+\
-                                    "permission to read the file: " + _file)
+                bad_perms.append(_file)
                 continue
-            _file = run_plugins_on_import(_file)
-            size = os.stat(_file).st_size
+
+            nfile = run_plugins_on_import(_file)
+            if nfile is not None:
+                _file = nfile
+            stat = os.stat(_file)
+            size = stat.st_size
             ext = os.path.splitext(_file)[1].lower().replace('.', '')
+            timestamp = utcfromtimestamp(stat.st_mtime)
             for row in range(self.formats.count()):
                 fmt = self.formats.item(row)
                 if fmt.ext.lower() == ext:
                     self.formats.takeItem(row)
                     break
-            Format(self.formats, ext, size, path=_file)
+            Format(self.formats, ext, size, path=_file, timestamp=timestamp)
             self.formats_changed = True
+            added = True
+        if bad_perms:
+            error_dialog(self, _('No permission'),
+                    _('You do not have '
+                'permission to read the following files:'),
+                det_msg='\n'.join(bad_perms), show=True)
 
-    def remove_format(self, x):
+        return added
+
+    def formats_dropped(self, event, paths):
+        if self._add_formats(paths):
+            event.accept()
+
+    def remove_format(self, *args):
         rows = self.formats.selectionModel().selectedRows(0)
         for row in rows:
             self.formats.takeItem(row.row())
             self.formats_changed = True
 
     def get_selected_format_metadata(self):
-        row = self.formats.currentRow()
-        fmt = self.formats.item(row)
-        if fmt is None:
-            if self.formats.count() == 1:
-                fmt = self.formats.item(0)
-            if fmt is None:
-                error_dialog(self, _('No format selected'),
-                    _('No format selected')).exec_()
-                return None, None
-        ext = fmt.ext.lower()
-        if fmt.path is None:
-            stream = self.db.format(self.row, ext, as_file=True)
-        else:
-            stream = open(fmt.path, 'r+b')
+        old = prefs['read_file_metadata']
+        if not old:
+            prefs['read_file_metadata'] = True
         try:
-            mi = get_metadata(stream, ext)
-            return mi, ext
-        except:
-            error_dialog(self, _('Could not read metadata'),
-                         _('Could not read metadata from %s format')%ext).exec_()
-        return None, None
+            row = self.formats.currentRow()
+            fmt = self.formats.item(row)
+            if fmt is None:
+                if self.formats.count() == 1:
+                    fmt = self.formats.item(0)
+                if fmt is None:
+                    error_dialog(self, _('No format selected'),
+                        _('No format selected')).exec_()
+                    return None, None
+            ext = fmt.ext.lower()
+            if fmt.path is None:
+                stream = self.db.format(self.row, ext, as_file=True)
+            else:
+                stream = open(fmt.path, 'r+b')
+            try:
+                mi = get_metadata(stream, ext)
+                return mi, ext
+            except:
+                error_dialog(self, _('Could not read metadata'),
+                            _('Could not read metadata from %s format')%ext).exec_()
+            return None, None
+        finally:
+            if old != prefs['read_file_metadata']:
+                prefs['read_file_metadata'] = old
 
     def set_metadata_from_format(self):
         mi, ext = self.get_selected_format_metadata()
@@ -208,7 +268,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             if mi.series_index is not None:
                 self.series_index.setValue(float(mi.series_index))
         if mi.comments and mi.comments.strip():
-            self.comments.setText(mi.comments)
+            self.comments.setPlainText(mi.comments)
 
 
     def set_cover(self):
@@ -269,6 +329,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.connect(self.__abort_button, SIGNAL('clicked()'),
                     self.do_cancel_all)
         self.splitter.setStretchFactor(100, 1)
+        self.read_state()
         self.db = db
         self.pi = ProgressIndicator(self)
         self.accepted_callback = accepted_callback
@@ -276,15 +337,23 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         self.row = row
         self.cover_data = None
         self.formats_changed = False
+        self.formats.setAcceptDrops(True)
         self.cover_changed = False
         self.cpixmap = None
-        self.cover.setAcceptDrops(True)
         self.pubdate.setMinimumDate(QDate(100,1,1))
+        pubdate_format = tweaks['gui_pubdate_display_format']
+        if pubdate_format is not None:
+            self.pubdate.setDisplayFormat(pubdate_format)
+        self.date.setMinimumDate(QDate(100,1,1))
+
         self.connect(self.cover, SIGNAL('cover_changed(PyQt_PyObject)'), self.cover_dropped)
         QObject.connect(self.cover_button, SIGNAL("clicked(bool)"), \
                                                     self.select_cover)
         QObject.connect(self.add_format_button, SIGNAL("clicked(bool)"), \
                                                     self.add_format)
+        self.connect(self.formats,
+                SIGNAL('formats_dropped(PyQt_PyObject,PyQt_PyObject)'),
+                self.formats_dropped)
         QObject.connect(self.remove_format_button, SIGNAL("clicked(bool)"), \
                                                 self.remove_format)
         QObject.connect(self.fetch_metadata_button, SIGNAL('clicked()'),
@@ -300,6 +369,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                         self.deduce_author_sort)
         self.connect(self.formats, SIGNAL('itemDoubleClicked(QListWidgetItem*)'),
                 self.show_format)
+        self.connect(self.formats, SIGNAL('delete_format()'), self.remove_format)
         self.connect(self.button_set_cover, SIGNAL('clicked()'), self.set_cover)
         self.connect(self.button_set_metadata, SIGNAL('clicked()'),
                 self.set_metadata_from_format)
@@ -310,11 +380,13 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         isbn = db.isbn(self.id, index_is_id=True)
         if not isbn:
             isbn = ''
+        self.isbn.textChanged.connect(self.validate_isbn)
         self.isbn.setText(isbn)
         aus = self.db.author_sort(row)
         self.author_sort.setText(aus if aus else '')
         tags = self.db.tags(row)
-        self.tags.setText(', '.join(tags.split(',')) if tags else '')
+        self.original_tags = ', '.join(tags.split(',')) if tags else ''
+        self.tags.setText(self.original_tags)
         self.tags.update_tags_cache(self.db.all_tags())
         rating = self.db.rating(row)
         if rating > 0:
@@ -325,6 +397,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         pubdate = db.pubdate(self.id, index_is_id=True)
         self.pubdate.setDate(QDate(pubdate.year, pubdate.month,
             pubdate.day))
+        timestamp = db.timestamp(self.id, index_is_id=True)
+        self.orig_timestamp = timestamp
+        self.date.setDate(QDate(timestamp.year, timestamp.month,
+            timestamp.day))
 
         exts = self.db.formats(row)
         if exts:
@@ -333,9 +409,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 if not ext:
                     ext = ''
                 size = self.db.sizeof_format(row, ext)
+                timestamp = self.db.format_last_modified(self.id, ext)
                 if size is None:
                     continue
-                Format(self.formats, ext, size)
+                Format(self.formats, ext, size, timestamp=timestamp)
 
 
         self.initialize_combos()
@@ -348,29 +425,63 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.series_index.setValue(1.0)
         QObject.connect(self.series, SIGNAL('currentIndexChanged(int)'), self.enable_series_index)
         QObject.connect(self.series, SIGNAL('editTextChanged(QString)'), self.enable_series_index)
+        self.series.lineEdit().editingFinished.connect(self.increment_series_index)
 
         self.show()
-        height_of_rest = self.frameGeometry().height() - self.cover.height()
-        width_of_rest  = self.frameGeometry().width() - self.cover.width()
-        ag = QCoreApplication.instance().desktop().availableGeometry(self)
-        self.cover.MAX_HEIGHT = ag.height()-(25 if islinux else 0)-height_of_rest
-        self.cover.MAX_WIDTH = ag.width()-(25 if islinux else 0)-width_of_rest
+        pm = QPixmap()
         if cover:
-            pm = QPixmap()
             pm.loadFromData(cover)
-            if not pm.isNull():
-                self.cover.setPixmap(pm)
+        if pm.isNull():
+            pm = QPixmap(I('default_cover.svg'))
+        else:
             self.cover_data = cover
+        self.cover.setPixmap(pm)
+        self.original_series_name = unicode(self.series.text()).strip()
+        if len(db.custom_column_label_map) == 0:
+            self.central_widget.tabBar().setVisible(False)
+        else:
+            self.create_custom_column_editors()
+        self.generate_cover_button.clicked.connect(self.generate_cover)
+
+    def create_custom_column_editors(self):
+        w = self.central_widget.widget(1)
+        layout = w.layout()
+        self.custom_column_widgets, self.__cc_spacers = \
+                    populate_metadata_page(layout, self.db, self.id,
+                                           parent=w, bulk=False, two_column=True)
+        self.__custom_col_layouts = [layout]
+        ans = self.custom_column_widgets
+        for i in range(len(ans)-1):
+            if len(ans[i+1].widgets) == 2:
+                w.setTabOrder(ans[i].widgets[-1], ans[i+1].widgets[1])
+            else:
+                w.setTabOrder(ans[i].widgets[-1], ans[i+1].widgets[0])
+            for c in range(2, len(ans[i].widgets), 2):
+                w.setTabOrder(ans[i].widgets[c-1], ans[i].widgets[c+1])
+
+    def validate_isbn(self, isbn):
+        isbn = unicode(isbn).strip()
+        if not isbn:
+            self.isbn.setStyleSheet('QLineEdit { background-color: rgba(0,255,0,0%) }')
+            self.isbn.setToolTip(_('This ISBN number is valid'))
+            return
+
+        if check_isbn(isbn):
+            self.isbn.setStyleSheet('QLineEdit { background-color: rgba(0,255,0,20%) }')
+            self.isbn.setToolTip(_('This ISBN number is valid'))
+        else:
+            self.isbn.setStyleSheet('QLineEdit { background-color: rgba(255,0,0,20%) }')
+            self.isbn.setToolTip(_('This ISBN number is invalid'))
 
     def show_format(self, item, *args):
         fmt = item.ext
-        self.emit(SIGNAL('view_format(PyQt_PyObject)'), fmt)
+        self.view_format.emit(fmt)
 
     def deduce_author_sort(self):
         au = unicode(self.authors.text())
         au = re.sub(r'\s+et al\.$', '', au)
         authors = string_to_authors(au)
-        self.author_sort.setText(authors_to_sort_string(authors))
+        self.author_sort.setText(self.db.author_sort_from_authors(authors))
 
     def swap_title_author(self):
         title = self.title.text()
@@ -378,9 +489,9 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         self.authors.setText(title)
         self.author_sort.setText('')
 
-    def cover_dropped(self, paths):
+    def cover_dropped(self, cover_data):
         self.cover_changed = True
-        self.cover_data = self.cover.cover_data
+        self.cover_data = cover_data
 
     def initialize_combos(self):
         self.initalize_authors()
@@ -438,6 +549,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.publisher.setCurrentIndex(idx)
 
     def edit_tags(self):
+        if self.tags.text() != self.original_tags:
+            error_dialog(self, _('Cannot use tag editor'),
+                _('The tags editor cannot be used if you have modified the tags')).exec_()
+            return
         d = TagEditor(self, self.db, self.row)
         d.exec_()
         if d.result() == QDialog.Accepted:
@@ -480,7 +595,14 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             if self.cover_fetcher.exception is not None:
                 err = self.cover_fetcher.exception
                 error_dialog(self, _('Cannot fetch cover'),
-                    _('<b>Could not fetch cover.</b><br/>')+repr(err)).exec_()
+                    _('<b>Could not fetch cover.</b><br/>')+unicode(err)).exec_()
+                return
+            if self.cover_fetcher.errors and self.cover_fetcher.cover_data is None:
+                details = u'\n\n'.join([e[-1] + ': ' + e[1] for e in self.cover_fetcher.errors])
+                error_dialog(self, _('Cannot fetch cover'),
+                    _('<b>Could not fetch cover.</b><br/>') +
+                    _('For the error message from each cover source, '
+                      'click Show details below.'), det_msg=details, show=True)
                 return
 
             pix = QPixmap()
@@ -501,9 +623,12 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
     def fetch_metadata(self):
         isbn   = re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text()))
-        title  = qstring_to_unicode(self.title.text())
-        author = string_to_authors(unicode(self.authors.text()))[0]
-        publisher = qstring_to_unicode(self.publisher.currentText())
+        title  = unicode(self.title.text())
+        try:
+            author = string_to_authors(unicode(self.authors.text()))[0]
+        except:
+            author = ''
+        publisher = unicode(self.publisher.currentText())
         if isbn or title or author or publisher:
             d = FetchMetadata(self, isbn, title, author, publisher, self.timeout)
             self._fetch_metadata_scope = d
@@ -511,9 +636,21 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 if d.exec_() == QDialog.Accepted:
                     book = d.selected_book()
                     if book:
-                        self.title.setText(book.title)
-                        self.authors.setText(authors_to_string(book.authors))
-                        if book.author_sort: self.author_sort.setText(book.author_sort)
+                        if d.opt_get_social_metadata.isChecked():
+                            d2 = SocialMetadata(book, self)
+                            d2.exec_()
+                            if d2.exceptions:
+                                det = '\n'.join([x[0]+'\n\n'+x[-1]+'\n\n\n' for
+                                    x in d2.exceptions])
+                                warning_dialog(self, _('There were errors'),
+                                       _('There were errors downloading social metadata'),
+                                       det_msg=det, show=True)
+                        else:
+                            book.tags = []
+                        if d.opt_overwrite_author_title_metadata.isChecked():
+                            self.title.setText(book.title)
+                            self.authors.setText(authors_to_string(book.authors))
+                            if book.author_sort: self.author_sort.setText(book.author_sort)
                         if book.publisher: self.publisher.setEditText(book.publisher)
                         if book.isbn: self.isbn.setText(book.isbn)
                         if book.pubdate:
@@ -521,31 +658,57 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                             self.pubdate.setDate(QDate(d.year, d.month, d.day))
                         summ = book.comments
                         if summ:
-                            prefix = qstring_to_unicode(self.comments.toPlainText())
+                            prefix = unicode(self.comments.toPlainText())
                             if prefix:
                                 prefix += '\n'
-                            self.comments.setText(prefix + summ)
+                            self.comments.setPlainText(prefix + summ)
+                        if book.rating is not None:
+                            self.rating.setValue(int(book.rating))
+                        if book.tags:
+                            self.tags.setText(', '.join(book.tags))
+                        if book.series is not None:
+                            if self.series.text() is None or self.series.text() == '':
+                                self.series.setText(book.series)
+                                if book.series_index is not None:
+                                    self.series_index.setValue(book.series_index)
         else:
             error_dialog(self, _('Cannot fetch metadata'),
                          _('You must specify at least one of ISBN, Title, '
-                           'Authors or Publisher'))
+                           'Authors or Publisher'), show=True)
 
     def enable_series_index(self, *args):
         self.series_index.setEnabled(True)
 
+    def increment_series_index(self):
+        if self.db is not None:
+            try:
+                series = unicode(self.series.text()).strip()
+                if series and series != self.original_series_name:
+                    ns = 1
+                    if tweaks['series_index_auto_increment'] == 'next':
+                        ns = self.db.get_next_series_num_for(series)
+                    self.series_index.setValue(ns)
+                    self.original_series_name = series
+            except:
+                traceback.print_exc()
+
     def remove_unused_series(self):
         self.db.remove_unused_series()
-        idx = qstring_to_unicode(self.series.currentText())
+        idx = unicode(self.series.currentText())
         self.series.clear()
         self.initialize_series()
         if idx:
             for i in range(self.series.count()):
-                if qstring_to_unicode(self.series.itemText(i)) == idx:
+                if unicode(self.series.itemText(i)) == idx:
                     self.series.setCurrentIndex(i)
                     break
 
 
     def accept(self):
+        cf = getattr(self, 'cover_fetcher', None)
+        if cf is not None and hasattr(cf, 'terminate'):
+            cf.terminate()
+            cf.wait()
         try:
             if self.formats_changed:
                 self.sync_formats()
@@ -556,20 +719,40 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 self.db.set_authors(self.id, string_to_authors(au), notify=False)
             aus = unicode(self.author_sort.text())
             if aus:
-                self.db.set_author_sort(self.id, aus, notify=False)
+                self.db.set_author_sort(self.id, aus, notify=False, commit=False)
             self.db.set_isbn(self.id,
-                    re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text())), notify=False)
-            self.db.set_rating(self.id, 2*self.rating.value(), notify=False)
-            self.db.set_publisher(self.id, qstring_to_unicode(self.publisher.currentText()), notify=False)
-            self.db.set_tags(self.id, qstring_to_unicode(self.tags.text()).split(','), notify=False)
-            self.db.set_series(self.id, qstring_to_unicode(self.series.currentText()), notify=False)
-            self.db.set_series_index(self.id, self.series_index.value(), notify=False)
-            self.db.set_comment(self.id, qstring_to_unicode(self.comments.toPlainText()), notify=False)
+                             re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text())),
+                             notify=False, commit=False)
+            self.db.set_rating(self.id, 2*self.rating.value(), notify=False,
+                               commit=False)
+            self.db.set_publisher(self.id, unicode(self.publisher.currentText()),
+                                  notify=False, commit=False)
+            self.db.set_tags(self.id, [x.strip() for x in
+                unicode(self.tags.text()).split(',')], notify=False, commit=False)
+            self.db.set_series(self.id,
+                    unicode(self.series.currentText()).strip(), notify=False,
+                    commit=False)
+            self.db.set_series_index(self.id, self.series_index.value(),
+                                     notify=False, commit=False)
+            self.db.set_comment(self.id, unicode(self.comments.toPlainText()),
+                                notify=False, commit=False)
             d = self.pubdate.date()
-            self.db.set_pubdate(self.id, datetime(d.year(), d.month(), d.day()))
+            d = qt_to_dt(d)
+            self.db.set_pubdate(self.id, d, notify=False, commit=False)
+            d = self.date.date()
+            d = qt_to_dt(d)
+            if d.date() != self.orig_timestamp.date():
+                self.db.set_timestamp(self.id, d, notify=False, commit=False)
+            self.db.commit()
 
-            if self.cover_changed and self.cover_data is not None:
-                self.db.set_cover(self.id, self.cover_data)
+            if self.cover_changed:
+                if self.cover_data is not None:
+                    self.db.set_cover(self.id, self.cover_data)
+                else:
+                    self.db.remove_cover(self.id)
+            for w in getattr(self, 'custom_column_widgets', []):
+                w.commit(self.id)
+            self.db.commit()
         except IOError, err:
             if err.errno == 13: # Permission denied
                 fname = err.filename if err.filename else 'file'
@@ -577,7 +760,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                         _('Could not open %s. Is it being used by another'
                         ' program?')%fname, show=True)
             raise
-
+        self.save_state()
         QDialog.accept(self)
         if callable(self.accepted_callback):
             self.accepted_callback(self.id)
@@ -589,3 +772,16 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             cf.wait()
 
         QDialog.reject(self, *args)
+
+    def read_state(self):
+        wg = dynamic.get('metasingle_window_geometry', None)
+        ss = dynamic.get('metasingle_splitter_state', None)
+        if wg is not None:
+            self.restoreGeometry(wg)
+        if ss is not None:
+            self.splitter.restoreState(ss)
+
+    def save_state(self):
+        dynamic.set('metasingle_window_geometry', bytes(self.saveGeometry()))
+        dynamic.set('metasingle_splitter_state',
+                bytes(self.splitter.saveState()))
