@@ -5,7 +5,7 @@ import sys, logging, os, traceback, time
 from PyQt4.QtGui import QKeySequence, QPainter, QDialog, QSpinBox, QSlider, QIcon
 from PyQt4.QtCore import Qt, QObject, SIGNAL, QCoreApplication, QThread
 
-from calibre import __appname__, setup_cli_handlers, islinux
+from calibre import __appname__, setup_cli_handlers, islinux, isfreebsd
 from calibre.ebooks.lrf.lrfparser import LRFDocument
 
 from calibre.gui2 import ORG_NAME, APP_UID, error_dialog, \
@@ -56,6 +56,12 @@ class Config(QDialog, Ui_ViewerConfig):
 
 class Main(MainWindow, Ui_MainWindow):
 
+    def create_document(self):
+        self.document = Document(self.logger, self.opts)
+        QObject.connect(self.document, SIGNAL('chapter_rendered(int)'), self.chapter_rendered)
+        QObject.connect(self.document, SIGNAL('page_changed(PyQt_PyObject)'), self.page_changed)
+
+
     def __init__(self, logger, opts, parent=None):
         MainWindow.__init__(self, opts, parent)
         Ui_MainWindow.__init__(self)
@@ -65,8 +71,7 @@ class Main(MainWindow, Ui_MainWindow):
 
         self.logger = logger
         self.opts = opts
-        self.document = None
-        self.document = Document(self.logger, self.opts)
+        self.create_document()
         self.spin_box_action = self.spin_box = QSpinBox()
         self.tool_bar.addWidget(self.spin_box)
         self.tool_bar.addSeparator()
@@ -76,10 +81,7 @@ class Main(MainWindow, Ui_MainWindow):
         self.search = SearchBox2(self)
         self.search.initialize('lrf_viewer_search_history')
         self.search_action = self.tool_bar.addWidget(self.search)
-        QObject.connect(self.document, SIGNAL('chapter_rendered(int)'), self.chapter_rendered)
-        QObject.connect(self.document, SIGNAL('page_changed(PyQt_PyObject)'), self.page_changed)
-
-        QObject.connect(self.search, SIGNAL('search(PyQt_PyObject, PyQt_PyObject)'), self.find)
+        self.search.search.connect(self.find)
 
         self.action_next_page.setShortcuts([QKeySequence.MoveToNextPage, QKeySequence(Qt.Key_Space)])
         self.action_previous_page.setShortcuts([QKeySequence.MoveToPreviousPage, QKeySequence(Qt.Key_Backspace)])
@@ -118,6 +120,7 @@ class Main(MainWindow, Ui_MainWindow):
         self.progress_bar.setMinimum(0)
         self.progress_bar.setMaximum(0)
         self.progress_bar.setValue(0)
+        self.create_document()
 
         if stream is not None:
             self.file_name = os.path.basename(stream.name) if hasattr(stream, 'name') else ''
@@ -149,7 +152,7 @@ class Main(MainWindow, Ui_MainWindow):
             self.stack.setCurrentIndex(1)
             self.renderer.start()
 
-    def find(self, search, refinement):
+    def find(self, search):
         self.last_search = search
         try:
             self.document.search(search)
@@ -255,7 +258,7 @@ def file_renderer(stream, opts, parent=None, logger=None):
         level = logging.DEBUG if opts.verbose else logging.INFO
         logger = logging.getLogger('lrfviewer')
         setup_cli_handlers(logger, level)
-    if islinux:
+    if islinux or isfreebsd:
         try: # Set lrfviewer as the default for LRF files for this user
             from subprocess import call
             call('xdg-mime default calibre-lrfviewer.desktop application/lrf', shell=True)
@@ -304,7 +307,7 @@ def main(args=sys.argv, logger=None):
     if hasattr(opts, 'help'):
         parser.print_help()
         return 1
-    pid = os.fork() if islinux else -1
+    pid = os.fork() if (islinux or isfreebsd) else -1
     if pid <= 0:
         app = Application(args)
         app.setWindowIcon(QIcon(I('viewer.svg')))
