@@ -7,7 +7,7 @@ __docformat__ = 'restructuredtext en'
 Logic for setting up conversion jobs
 '''
 
-import cPickle
+import cPickle, os
 
 from PyQt4.Qt import QDialog, QProgressDialog, QString, QTimer, SIGNAL
 
@@ -22,6 +22,7 @@ from calibre.customize.conversion import OptionRecommendation
 from calibre.utils.config import prefs
 from calibre.ebooks.conversion.config import GuiRecommendations, \
     load_defaults, load_specifics, save_specifics
+from calibre.gui2.convert import bulk_defaults_for_input_format
 
 def convert_single_ebook(parent, db, book_ids, auto_conversion=False, out_format=None):
     changed = False
@@ -31,7 +32,6 @@ def convert_single_ebook(parent, db, book_ids, auto_conversion=False, out_format
     total = len(book_ids)
     if total == 0:
         return None, None, None
-    parent.status_bar.showMessage(_('Starting conversion of %d books') % total, 2000)
 
     for i, book_id in enumerate(book_ids):
         temp_files = []
@@ -96,34 +96,33 @@ def convert_single_ebook(parent, db, book_ids, auto_conversion=False, out_format
     return jobs, changed, bad
 
 def convert_bulk_ebook(parent, queue, db, book_ids, out_format=None, args=[]):
-    changed = False
-    jobs = []
-    bad = []
-
     total = len(book_ids)
     if total == 0:
         return None, None, None
-    parent.status_bar.showMessage(_('Starting conversion of %d books') % total, 2000)
 
     d = BulkConfig(parent, db, out_format)
     if d.exec_() != QDialog.Accepted:
-        return jobs, changed, bad
+        return None
 
     output_format = d.output_format
     user_recs = cPickle.loads(d.recommendations)
 
     book_ids = convert_existing(parent, db, book_ids, output_format)
-    return QueueBulk(parent, book_ids, output_format, queue, db, user_recs, args)
+    use_saved_single_settings = d.opt_individual_saved_settings.isChecked()
+    return QueueBulk(parent, book_ids, output_format, queue, db, user_recs,
+            args, use_saved_single_settings=use_saved_single_settings)
 
 class QueueBulk(QProgressDialog):
 
-    def __init__(self, parent, book_ids, output_format, queue, db, user_recs, args):
+    def __init__(self, parent, book_ids, output_format, queue, db, user_recs,
+            args, use_saved_single_settings=True):
         QProgressDialog.__init__(self, '',
                 QString(), 0, len(book_ids), parent)
         self.setWindowTitle(_('Queueing books for bulk conversion'))
         self.book_ids, self.output_format, self.queue, self.db, self.args, self.user_recs = \
                 book_ids, output_format, queue, db, args, user_recs
         self.parent = parent
+        self.use_saved_single_settings = use_saved_single_settings
         self.i, self.bad, self.jobs, self.changed = 0, [], [], False
         self.timer = QTimer(self)
         self.connect(self.timer, SIGNAL('timeout()'), self.do_book)
@@ -150,12 +149,13 @@ class QueueBulk(QProgressDialog):
             temp_files = []
 
             combined_recs = GuiRecommendations()
-            default_recs = load_defaults('%s_input' % input_format)
-            specific_recs = load_specifics(self.db, book_id)
+            default_recs = bulk_defaults_for_input_format(input_format)
             for key in default_recs:
                 combined_recs[key] = default_recs[key]
-            for key in specific_recs:
-                combined_recs[key] = specific_recs[key]
+            if self.use_saved_single_settings:
+                specific_recs = load_specifics(self.db, book_id)
+                for key in specific_recs:
+                    combined_recs[key] = specific_recs[key]
             for item in self.user_recs:
                 combined_recs[item[0]] = item[1]
             save_specifics(self.db, book_id, combined_recs)
@@ -175,7 +175,6 @@ class QueueBulk(QProgressDialog):
             for x in list(lrecs):
                 if x[0] == 'debug_pipeline':
                     lrecs.remove(x)
-
             try:
                 dtitle = unicode(mi.title)
             except:
@@ -209,9 +208,7 @@ class QueueBulk(QProgressDialog):
         self.jobs.reverse()
         self.queue(self.jobs, self.changed, self.bad, *self.args)
 
-def fetch_scheduled_recipe(recipe, script):
-    from calibre.gui2.dialogs.scheduler import config
-    from calibre.ebooks.conversion.config import load_defaults
+def fetch_scheduled_recipe(arg):
     fmt = prefs['output_format'].lower()
     pt = PersistentTemporaryFile(suffix='_recipe_out.%s'%fmt.lower())
     pt.close()
@@ -224,22 +221,73 @@ def fetch_scheduled_recipe(recipe, script):
     if lf.get('base_font_size', 0.0) != 0.0:
         recs.append(('base_font_size', lf['base_font_size'],
             OptionRecommendation.HIGH))
+        recs.append(('keep_ligatures', lf.get('keep_ligatures', False),
+            OptionRecommendation.HIGH))
 
     lr = load_defaults('lrf_output')
     if lr.get('header', False):
         recs.append(('header', True, OptionRecommendation.HIGH))
         recs.append(('header_format', '%t', OptionRecommendation.HIGH))
 
-    args = [script, pt.name, recs]
-    if recipe.needs_subscription:
-        x = config.get('recipe_account_info_%s'%recipe.id, False)
-        if not x:
-            raise ValueError(_('You must set a username and password for %s')%recipe.title)
-        recs.append(('username', x[0], OptionRecommendation.HIGH))
-        recs.append(('password', x[1], OptionRecommendation.HIGH))
+    args = [arg['recipe'], pt.name, recs]
+    if arg['username'] is not None:
+        recs.append(('username', arg['username'], OptionRecommendation.HIGH))
+    if arg['password'] is not None:
+        recs.append(('password', arg['password'], OptionRecommendation.HIGH))
 
 
-    return 'gui_convert', args, _('Fetch news from ')+recipe.title, fmt.upper(), [pt]
+    return 'gui_convert', args, _('Fetch news from ')+arg['title'], fmt.upper(), [pt]
+
+def generate_catalog(parent, dbspec, ids, device):
+    from calibre.gui2.dialogs.catalog import Catalog
+
+    # Build the Catalog dialog in gui2.dialogs.catalog
+    d = Catalog(parent, dbspec, ids)
+
+    if d.exec_() != d.Accepted:
+        return None
+
+    # Create the output file
+    out = PersistentTemporaryFile(suffix='_catalog_out.'+d.catalog_format.lower())
+
+    # Profile the connected device
+    # Parallel initialization in calibre.library.cli:command_catalog()
+    connected_device = { 'storage':None,'serial':None,'save_template':None,'name':None}
+
+    if device:
+        try:
+            storage = []
+            if device._main_prefix:
+                storage.append(os.path.join(device._main_prefix, device.EBOOK_DIR_MAIN))
+            if device._card_a_prefix:
+                storage.append(os.path.join(device._card_a_prefix, device.EBOOK_DIR_CARD_A))
+            if device._card_b_prefix:
+                storage.append(os.path.join(device._card_b_prefix, device.EBOOK_DIR_CARD_B))
+            connected_device = {      'storage': storage,
+                                       'serial': device.detected_device.serial if \
+                                                 hasattr(device.detected_device,'serial') else None,
+                                'save_template': device.save_template(),
+                                         'name': device.gui_name}
+        except:
+            pass
+
+    # These args are passed inline to gui2.convert.gui_conversion:gui_catalog
+    args = [
+        d.catalog_format,
+        d.catalog_title,
+        dbspec,
+        ids,
+        out.name,
+        d.catalog_sync,
+        d.fmt_options,
+        connected_device
+        ]
+    out.close()
+
+    # This returns to gui2.ui:generate_catalog()
+    # Which then calls gui2.convert.gui_conversion:gui_catalog() with the args inline
+    return 'gui_catalog', args, _('Generate catalog'), out.name, d.catalog_sync, \
+            d.catalog_title
 
 def convert_existing(parent, db, book_ids, output_format):
     already_converted_ids = []
