@@ -5,20 +5,25 @@ __docformat__ = 'restructuredtext en'
 
 '''
 '''
-import os, math, re, glob
-from PyQt4.Qt import QWidget, QSize, QSizePolicy, QUrl, SIGNAL, Qt, QTimer, \
+import os, math, re, glob, sys
+from base64 import b64encode
+from functools import partial
+
+from PyQt4.Qt import QSize, QSizePolicy, QUrl, SIGNAL, Qt, QTimer, \
                      QPainter, QPalette, QBrush, QFontDatabase, QDialog, \
-                     QColor, QPoint, QImage, QRegion, QVariant, \
-                     QFont, QObject, QApplication, pyqtSignature
+                     QColor, QPoint, QImage, QRegion, QVariant, QIcon, \
+                     QFont, pyqtSignature, QAction, QByteArray, QMenu
 from PyQt4.QtWebKit import QWebPage, QWebView, QWebSettings
 
 from calibre.utils.config import Config, StringConfig
 from calibre.utils.localization import get_language
 from calibre.gui2.viewer.config_ui import Ui_Dialog
-from calibre.gui2.viewer.js import bookmarks, referencing, hyphenation
-from calibre.ptempfile import PersistentTemporaryFile
+from calibre.gui2.shortcuts import Shortcuts, ShortcutConfig
 from calibre.constants import iswindows
-from calibre import prints
+from calibre import prints, guess_type
+from calibre.gui2.viewer.keys import SHORTCUTS
+
+bookmarks = referencing = hyphenation = jquery = jquery_scrollTo = hyphenator = images =None
 
 def load_builtin_fonts():
     base = P('fonts/liberation/*.ttf')
@@ -39,6 +44,8 @@ def config(defaults=None):
               help=_('Set the user CSS stylesheet. This can be used to customize the look of all books.'))
     c.add_opt('max_view_width', default=6000,
             help=_('Maximum width of the viewer window, in pixels.'))
+    c.add_opt('fit_images', default=True,
+            help=_('Resize images larger than the viewer window to fit inside it'))
     c.add_opt('hyphenate', default=False, help=_('Hyphenate text'))
     c.add_opt('hyphenate_default_lang', default='en',
             help=_('Default language for hyphenation rules'))
@@ -56,24 +63,10 @@ def config(defaults=None):
 
     return c
 
-class PythonJS(QObject):
-
-    def __init__(self, callback):
-        QObject.__init__(self, QApplication.instance())
-        self.setObjectName("py_bridge")
-        self._callback = callback
-
-    @pyqtSignature("QString")
-    def callback(self, msg):
-        print "callback called"
-        self._callback(msg)
-
-
-
 class ConfigDialog(QDialog, Ui_Dialog):
 
-    def __init__(self, *args):
-        QDialog.__init__(self, *args)
+    def __init__(self, shortcuts, parent=None):
+        QDialog.__init__(self, parent)
         self.setupUi(self)
 
         opts = config().parse()
@@ -88,7 +81,8 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.css.setToolTip(_('Set the user CSS stylesheet. This can be used to customize the look of all books.'))
         self.max_view_width.setValue(opts.max_view_width)
         pats = [os.path.basename(x).split('.')[0] for x in
-            glob.glob(P('viewer/hyphenate/patterns/*.js'))]
+            glob.glob(P('viewer/hyphenate/patterns/*.js',
+                allow_user_override=False))]
         names = list(map(get_language, pats))
         pmap = {}
         for i in range(len(pats)):
@@ -103,6 +97,11 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.hyphenate_default_lang.setCurrentIndex(idx)
         self.hyphenate.setChecked(opts.hyphenate)
         self.hyphenate_default_lang.setEnabled(opts.hyphenate)
+        self.shortcuts = shortcuts
+        self.shortcut_config = ShortcutConfig(shortcuts, parent=self)
+        p = self.tabs.widget(1)
+        p.layout().addWidget(self.shortcut_config)
+        self.opt_fit_images.setChecked(opts.fit_images)
 
 
     def accept(self, *args):
@@ -115,6 +114,7 @@ class ConfigDialog(QDialog, Ui_Dialog):
         c.set('standard_font', {0:'serif', 1:'sans', 2:'mono'}[self.standard_font.currentIndex()])
         c.set('user_css', unicode(self.css.toPlainText()))
         c.set('remember_window_size', self.opt_remember_window_size.isChecked())
+        c.set('fit_images', self.opt_fit_images.isChecked())
         c.set('max_view_width', int(self.max_view_width.value()))
         c.set('hyphenate', self.hyphenate.isChecked())
         idx = self.hyphenate_default_lang.currentIndex()
@@ -138,22 +138,22 @@ class Document(QWebPage):
         settings.setFontFamily(QWebSettings.FixedFont, opts.mono_family)
 
     def do_config(self, parent=None):
-        d = ConfigDialog(parent)
+        d = ConfigDialog(self.shortcuts, parent)
         if d.exec_() == QDialog.Accepted:
             self.set_font_settings()
             self.set_user_stylesheet()
             self.misc_config()
             self.triggerAction(QWebPage.Reload)
 
-    def __init__(self, *args):
-        QWebPage.__init__(self, *args)
+    def __init__(self, shortcuts, parent=None):
+        QWebPage.__init__(self, parent)
         self.setObjectName("py_bridge")
         self.debug_javascript = False
         self.current_language = None
-        #self.js_bridge = PythonJS(self.js_callback)
 
         self.setLinkDelegationPolicy(self.DelegateAllLinks)
         self.scroll_marks = []
+        self.shortcuts = shortcuts
         pal = self.palette()
         pal.setBrush(QPalette.Background, QColor(0xee, 0xee, 0xee))
         self.setPalette(pal)
@@ -181,26 +181,43 @@ class Document(QWebPage):
 
     def set_user_stylesheet(self):
         raw = config().parse().user_css
-        pt = PersistentTemporaryFile('_user_stylesheet.css')
-        pt.write(raw.encode('utf-8'))
-        pt.close()
-        self.settings().setUserStyleSheetUrl(QUrl.fromLocalFile(pt.name))
+        raw = '::selection {background:#ffff00; color:#000;}\nbody {background-color: white;}\n'+raw
+        data = 'data:text/css;charset=utf-8;base64,'
+        data += b64encode(raw.encode('utf-8'))
+        self.settings().setUserStyleSheetUrl(QUrl(data))
 
     def misc_config(self):
         opts = config().parse()
         self.hyphenate = opts.hyphenate
         self.hyphenate_default_lang = opts.hyphenate_default_lang
+        self.do_fit_images = opts.fit_images
+
+    def fit_images(self):
+        if self.do_fit_images:
+            self.javascript('setup_image_scaling_handlers()')
 
     def load_javascript_libraries(self):
+        global bookmarks, referencing, hyphenation, jquery, jquery_scrollTo, hyphenator, images
         self.mainFrame().addToJavaScriptWindowObject("py_bridge", self)
-        jquery = open(P('content_server/jquery.js'), 'rb').read()
-        jquery_scrollTo = open(P('viewer/jquery_scrollTo.js'), 'rb').read()
-        hyphenator = open(P('viewer/hyphenate/Hyphenator.js'),
-                'rb').read().decode('utf-8')
+        if jquery is None:
+            jquery = P('content_server/jquery.js', data=True)
+        if jquery_scrollTo is None:
+            jquery_scrollTo = P('viewer/jquery_scrollTo.js', data=True)
+        if hyphenator is None:
+            hyphenator = P('viewer/hyphenate/Hyphenator.js', data=True).decode('utf-8')
         self.javascript(jquery)
         self.javascript(jquery_scrollTo)
+        if bookmarks is None:
+            bookmarks = P('viewer/bookmarks.js', data=True)
         self.javascript(bookmarks)
+        if referencing is None:
+            referencing = P('viewer/referencing.js', data=True)
         self.javascript(referencing)
+        if images is None:
+            images = P('viewer/images.js', data=True)
+        self.javascript(images)
+        if hyphenation is None:
+            hyphenation = P('viewer/hyphenation.js', data=True)
         self.javascript(hyphenation)
         default_lang = self.hyphenate_default_lang
         lang = self.current_language
@@ -281,8 +298,49 @@ class Document(QWebPage):
             if r > 0:
                 self.javascript('document.body.style.paddingBottom = "%dpx"'%r)
 
+    def element_ypos(self, elem):
+        ans, ok = elem.evaluateJavaScript('$(this).offset().top').toInt()
+        if not ok:
+            raise ValueError('No ypos found')
+        return ans
+
+    def elem_outer_xml(self, elem):
+        return unicode(elem.toOuterXml())
+
+    def find_bookmark_element(self):
+        mf = self.mainFrame()
+        doc_pos = self.ypos
+        min_delta, min_elem = sys.maxint, None
+        for y in range(10, -500, -10):
+            for x in range(-50, 500, 10):
+                pos = QPoint(x, y)
+                result = mf.hitTestContent(pos)
+                if result.isNull(): continue
+                elem = result.enclosingBlockElement()
+                if elem.isNull(): continue
+                try:
+                    ypos = self.element_ypos(elem)
+                except:
+                    continue
+                delta = abs(ypos - doc_pos)
+                if delta < 25:
+                    return elem
+                if delta < min_delta:
+                    min_elem, min_delta = elem, delta
+        return min_elem
+
+
     def bookmark(self):
-        return self.javascript('calculate_bookmark(%d)'%(self.ypos+25), 'string')
+        elem = self.find_bookmark_element()
+
+        if elem is None or self.element_ypos(elem) < 100:
+            bm = 'body|%f'%(float(self.ypos)/(self.height*0.7))
+        else:
+            bm = unicode(elem.evaluateJavaScript(
+                'calculate_bookmark(%d, this)'%self.ypos).toString())
+            if not bm:
+                bm = 'body|%f'%(float(self.ypos)/(self.height*0.7))
+        return bm
 
     @property
     def at_bottom(self):
@@ -327,12 +385,26 @@ class Document(QWebPage):
 
     @property
     def height(self):
-        return self.javascript('document.body.offsetHeight', 'int') # contentsSize gives inaccurate results
+        j = self.javascript('document.body.offsetHeight', 'int')
+        q = self.mainFrame().contentsSize().height()
+        if q == j:
+            return j
+        if min(j, q) <= 0:
+            return max(j, q)
+        window_height = self.window_height
+        if j == window_height:
+            return j if q < 1.2*j else q
+        return j
 
     @property
     def width(self):
         return self.mainFrame().contentsSize().width() # offsetWidth gives inaccurate results
 
+    def set_bottom_padding(self, amount):
+        s = QSize(-1, -1) if amount == 0 else QSize(self.width,
+                self.height+amount)
+        self.setPreferredContentsSize(s)
+
 class EntityDeclarationProcessor(object):
 
     def __init__(self, html):
@@ -350,28 +422,85 @@ class DocumentView(QWebView):
     DISABLED_BRUSH = QBrush(Qt.lightGray, Qt.Dense5Pattern)
 
     def __init__(self, *args):
-        QWidget.__init__(self, *args)
+        QWebView.__init__(self, *args)
         self.debug_javascript = False
+        self.shortcuts =  Shortcuts(SHORTCUTS, 'shortcuts/viewer')
+        self.self_closing_pat = re.compile(r'<([a-z1-6]+)\s+([^>]+)/>',
+                re.IGNORECASE)
         self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
         self._size_hint = QSize(510, 680)
         self.initial_pos = 0.0
         self.to_bottom = False
-        self.document = Document(self)
+        self.document = Document(self.shortcuts, parent=self)
         self.setPage(self.document)
         self.manager = None
         self._reference_mode = False
         self._ignore_scrollbar_signals = False
-        self.connect(self.document, SIGNAL('loadStarted()'), self.load_started)
-        self.connect(self.document, SIGNAL('loadFinished(bool)'), self.load_finished)
+        self.loading_url = None
+        self.loadFinished.connect(self.load_finished)
         self.connect(self.document, SIGNAL('linkClicked(QUrl)'), self.link_clicked)
         self.connect(self.document, SIGNAL('linkHovered(QString,QString,QString)'), self.link_hovered)
         self.connect(self.document, SIGNAL('selectionChanged()'), self.selection_changed)
         self.connect(self.document, SIGNAL('animated_scroll_done()'),
                 self.animated_scroll_done, Qt.QueuedConnection)
+        copy_action = self.pageAction(self.document.Copy)
+        copy_action.setIcon(QIcon(I('convert.svg')))
+        d = self.document
+        self.unimplemented_actions = list(map(self.pageAction,
+            [d.DownloadImageToDisk, d.OpenLinkInNewWindow, d.DownloadLinkToDisk,
+                d.OpenImageInNewWindow, d.OpenLink]))
+        self.dictionary_action = QAction(QIcon(I('dictionary.svg')),
+                _('&Lookup in dictionary'), self)
+        self.dictionary_action.setShortcut(Qt.CTRL+Qt.Key_L)
+        self.dictionary_action.triggered.connect(self.lookup)
+        self.goto_location_action = QAction(_('Go to...'), self)
+        self.goto_location_menu = m = QMenu(self)
+        self.goto_location_actions = a = {
+                'Next Page': self.next_page,
+                'Previous Page': self.previous_page,
+                'Section Top' : partial(self.scroll_to, 0),
+                'Document Top': self.goto_document_start,
+                'Section Bottom':partial(self.scroll_to, 1),
+                'Document Bottom': self.goto_document_end,
+                'Next Section': self.goto_next_section,
+                'Previous Section': self.goto_previous_section,
+        }
+        for name, key in [(_('Next Section'), 'Next Section'),
+                (_('Previous Section'), 'Previous Section'),
+                (None, None),
+                (_('Document Start'), 'Document Top'),
+                (_('Document End'), 'Document Bottom'),
+                (None, None),
+                (_('Section Start'), 'Section Top'),
+                (_('Section End'), 'Section Bottom'),
+                (None, None),
+                (_('Next Page'), 'Next Page'),
+                (_('Previous Page'), 'Previous Page')]:
+            if key is None:
+                m.addSeparator()
+            else:
+                m.addAction(name, a[key], self.shortcuts.get_sequences(key)[0])
+        self.goto_location_action.setMenu(self.goto_location_menu)
+
+    def goto_next_section(self, *args):
+        if self.manager is not None:
+            self.manager.goto_next_section()
+
+    def goto_previous_section(self, *args):
+        if self.manager is not None:
+            self.manager.goto_previous_section()
+
+    def goto_document_start(self, *args):
+        if self.manager is not None:
+            self.manager.goto_start()
+
+    def goto_document_end(self, *args):
+        if self.manager is not None:
+            self.manager.goto_end()
 
     @property
     def copy_action(self):
-        return self.document.action(QWebPage.Copy)
+        return self.pageAction(self.document.Copy)
 
     def animated_scroll_done(self):
         if self.manager is not None:
@@ -391,6 +520,7 @@ class DocumentView(QWebView):
         self.document.do_config(parent)
         if self.manager is not None:
             self.manager.set_max_width()
+        self.setFocus(Qt.OtherFocusReason)
 
     def bookmark(self):
         return self.document.bookmark()
@@ -399,6 +529,23 @@ class DocumentView(QWebView):
         if self.manager is not None:
             self.manager.selection_changed(unicode(self.document.selectedText()))
 
+    def contextMenuEvent(self, ev):
+        menu = self.document.createStandardContextMenu()
+        for action in self.unimplemented_actions:
+            menu.removeAction(action)
+        text = unicode(self.selectedText())
+        if text:
+            menu.insertAction(list(menu.actions())[0], self.dictionary_action)
+        menu.addSeparator()
+        menu.addAction(self.goto_location_action)
+        menu.exec_(ev.globalPos())
+
+    def lookup(self, *args):
+        if self.manager is not None:
+            t = unicode(self.selectedText()).strip()
+            if t:
+                self.manager.lookup(t.split()[0])
+
     def set_manager(self, manager):
         self.manager = manager
         self.scrollbar = manager.horizontal_scrollbar
@@ -439,22 +586,40 @@ class DocumentView(QWebView):
         def fset(self, val): self.document.current_language = val
         return property(fget=fget, fset=fset)
 
-    def search(self, text):
+    def search(self, text, backwards=False):
+        if backwards:
+            return self.findText(text, self.document.FindBackwards)
         return self.findText(text)
 
     def path(self):
         return os.path.abspath(unicode(self.url().toLocalFile()))
 
+    def self_closing_sub(self, match):
+        tag = match.group(1)
+        if tag.lower().strip() == 'br':
+            return match.group()
+        return '<%s %s></%s>'%(match.group(1), match.group(2), match.group(1))
+
     def load_path(self, path, pos=0.0):
         self.initial_pos = pos
+        mt = getattr(path, 'mime_type', None)
+        if mt is None:
+            mt = guess_type(path)[0]
         html = open(path, 'rb').read().decode(path.encoding, 'replace')
         html = EntityDeclarationProcessor(html).processed_html
-        self.setHtml(html, QUrl.fromLocalFile(path))
-        self.turn_off_internal_scrollbars()
+        has_svg = re.search(r'<[:a-zA-Z]*svg', html) is not None
 
-    def load_started(self):
+        if 'xhtml' in mt:
+            html = self.self_closing_pat.sub(self.self_closing_sub, html)
         if self.manager is not None:
             self.manager.load_started()
+        self.loading_url = QUrl.fromLocalFile(path)
+        if has_svg:
+            prints('Rendering as XHTML...')
+            self.setContent(QByteArray(html.encode(path.encoding)), mt, QUrl.fromLocalFile(path))
+        else:
+            self.setHtml(html, self.loading_url)
+        self.turn_off_internal_scrollbars()
 
     def initialize_scrollbar(self):
         if getattr(self, 'scrollbar', None) is not None:
@@ -472,6 +637,12 @@ class DocumentView(QWebView):
 
 
     def load_finished(self, ok):
+        if self.loading_url is None:
+            # An <iframe> finished loading
+            return
+        self.loading_url = None
+        self.document.set_bottom_padding(0)
+        self.document.fit_images()
         self._size_hint = self.document.mainFrame().contentsSize()
         scrolled = False
         if self.to_bottom:
@@ -500,6 +671,7 @@ class DocumentView(QWebView):
 
     @classmethod
     def test_line(cls, img, y):
+        'Test if line contains pixels of exactly the same color'
         start = img.pixel(0, y)
         for i in range(1, img.width()):
             if img.pixel(i, y) != start:
@@ -509,6 +681,7 @@ class DocumentView(QWebView):
     def find_next_blank_line(self, overlap):
         img = QImage(self.width(), overlap, QImage.Format_ARGB32)
         painter = QPainter(img)
+        # Render a region of width x overlap pixels atthe bottom of the current viewport
         self.document.mainFrame().render(painter, QRegion(0, 0, self.width(), overlap))
         painter.end()
         for i in range(overlap-1, -1, -1):
@@ -521,8 +694,8 @@ class DocumentView(QWebView):
         delta_y = self.document.window_height - 25
         if self.document.at_top:
             if self.manager is not None:
-                self.manager.previous_document()
                 self.to_bottom = True
+                self.manager.previous_document()
         else:
             opos = self.document.ypos
             upper_limit = opos - delta_y
@@ -534,20 +707,52 @@ class DocumentView(QWebView):
                 self.manager.scrolled(self.scroll_fraction)
 
     def next_page(self):
-        delta_y = self.document.window_height - 25
-        if self.document.at_bottom:
+        window_height = self.document.window_height
+        document_height = self.document.height
+        ddelta = document_height - window_height
+        #print '\nWindow height:', window_height
+        #print 'Document height:', self.document.height
+
+        delta_y = window_height - 25
+        if self.document.at_bottom or ddelta <= 0:
             if self.manager is not None:
                 self.manager.next_document()
+        elif ddelta < 25:
+            self.scroll_by(y=ddelta)
+            return
         else:
+            oopos = self.document.ypos
+            #print 'Original position:', oopos
+            self.document.set_bottom_padding(0)
             opos = self.document.ypos
-            lower_limit = opos + delta_y
-            max_y = self.document.height - self.document.window_height
+            #print 'After set padding=0:', self.document.ypos
+            if opos < oopos:
+                if self.manager is not None:
+                    self.manager.next_document()
+                return
+            lower_limit = opos + delta_y # Max value of top y co-ord after scrolling
+            max_y = self.document.height - window_height # The maximum possible top y co-ord
+            if max_y < lower_limit:
+                padding = lower_limit - max_y
+                if padding == window_height:
+                    if self.manager is not None:
+                        self.manager.next_document()
+                    return
+                #print 'Setting padding to:', lower_limit - max_y
+                self.document.set_bottom_padding(lower_limit - max_y)
+            #print 'Document height:', self.document.height
+            max_y = self.document.height - window_height
             lower_limit = min(max_y, lower_limit)
+            #print 'Scroll to:', lower_limit
             if lower_limit > opos:
                 self.document.scroll_to(self.document.xpos, lower_limit)
-            self.find_next_blank_line( self.height() - (self.document.ypos-opos) )
+            actually_scrolled = self.document.ypos - opos
+            #print 'After scroll pos:', self.document.ypos
+            self.find_next_blank_line(window_height - actually_scrolled)
+            #print 'After blank line pos:', self.document.ypos
             if self.manager is not None:
                 self.manager.scrolled(self.scroll_fraction)
+            #print 'After all:', self.document.ypos
 
     def scroll_by(self, x=0, y=0, notify=True):
         old_pos = self.document.ypos
@@ -623,30 +828,17 @@ class DocumentView(QWebView):
         return ret
 
     def keyPressEvent(self, event):
-        key = event.key()
-        if key in [Qt.Key_PageDown, Qt.Key_Space, Qt.Key_Down]:
-            self.next_page()
-        elif key in [Qt.Key_PageUp, Qt.Key_Backspace, Qt.Key_Up]:
-            self.previous_page()
-        elif key in [Qt.Key_Home]:
-            if event.modifiers() & Qt.ControlModifier:
-                if self.manager is not None:
-                    self.manager.goto_start()
-            else:
-                self.scroll_to(0)
-        elif key in [Qt.Key_End]:
-            if event.modifiers() & Qt.ControlModifier:
-                if self.manager is not None:
-                    self.manager.goto_end()
-            else:
-                self.scroll_to(1)
-        elif key in [Qt.Key_J]:
+        key = self.shortcuts.get_match(event)
+        func = self.goto_location_actions.get(key, None)
+        if func is not None:
+            func()
+        elif key == 'Down':
             self.scroll_by(y=15)
-        elif key in [Qt.Key_K]:
+        elif key == 'Up':
             self.scroll_by(y=-15)
-        elif key in [Qt.Key_H]:
+        elif key == 'Left':
             self.scroll_by(x=-15)
-        elif key in [Qt.Key_L]:
+        elif key == 'Right':
             self.scroll_by(x=15)
         else:
             return QWebView.keyPressEvent(self, event)
