@@ -5,8 +5,8 @@ import traceback, os, sys, functools, collections, re
 from functools import partial
 from threading import Thread
 
-from PyQt4.Qt import QMovie, QApplication, Qt, QIcon, QTimer, QWidget, SIGNAL, \
-                     QDesktopServices, QDoubleSpinBox, QLabel, QTextBrowser, \
+from PyQt4.Qt import QApplication, Qt, QIcon, QTimer, SIGNAL, QByteArray, \
+                     QDoubleSpinBox, QLabel, QTextBrowser, \
                      QPainter, QBrush, QColor, QStandardItemModel, QPalette, \
                      QStandardItem, QUrl, QRegExpValidator, QRegExp, QLineEdit, \
                      QToolButton, QMenu, QInputDialog, QAction, QKeySequence
@@ -14,16 +14,18 @@ from PyQt4.Qt import QMovie, QApplication, Qt, QIcon, QTimer, QWidget, SIGNAL, \
 from calibre.gui2.viewer.main_ui import Ui_EbookViewer
 from calibre.gui2.viewer.printing import Printing
 from calibre.gui2.viewer.bookmarkmanager import BookmarkManager
+from calibre.gui2.widgets import ProgressIndicator
 from calibre.gui2.main_window import MainWindow
 from calibre.gui2 import Application, ORG_NAME, APP_UID, choose_files, \
-                         info_dialog, error_dialog
+                         info_dialog, error_dialog, open_url
 from calibre.ebooks.oeb.iterator import EbookIterator
 from calibre.ebooks import DRMError
-from calibre.constants import islinux
+from calibre.constants import islinux, isfreebsd
 from calibre.utils.config import Config, StringConfig, dynamic
 from calibre.gui2.search_box import SearchBox2
 from calibre.ebooks.metadata import MetaInformation
 from calibre.customize.ui import available_input_formats
+from calibre.gui2.viewer.dictionary import Lookup
 
 class TOCItem(QStandardItem):
 
@@ -62,39 +64,6 @@ class Worker(Thread):
             self.exception = err
             self.traceback = traceback.format_exc()
 
-class ProgressIndicator(QWidget):
-
-    def __init__(self, *args):
-        QWidget.__init__(self, *args)
-        self.setGeometry(0, 0, 300, 500)
-        self.movie = QMovie(I('jobs-animated.mng'))
-        self.ml = QLabel(self)
-        self.ml.setMovie(self.movie)
-        self.movie.start()
-        self.movie.setPaused(True)
-        self.status = QLabel(self)
-        self.status.setWordWrap(True)
-        self.status.setAlignment(Qt.AlignHCenter|Qt.AlignTop)
-        self.setVisible(False)
-
-    def start(self, msg=''):
-        view = self.parent()
-        pwidth, pheight = view.size().width(), view.size().height()
-        self.resize(pwidth, min(pheight, 250))
-        self.move(0, (pheight-self.size().height())/2.)
-        self.ml.resize(self.ml.sizeHint())
-        self.ml.move(int((self.size().width()-self.ml.size().width())/2.), 0)
-        self.status.resize(self.size().width(), self.size().height()-self.ml.size().height()-10)
-        self.status.move(0, self.ml.size().height()+10)
-        self.status.setText('<h1>'+msg+'</h1>')
-        self.setVisible(True)
-        self.movie.setPaused(False)
-
-    def stop(self):
-        if self.movie.state() == self.movie.Running:
-            self.movie.setPaused(True)
-            self.setVisible(False)
-
 class History(collections.deque):
 
     def __init__(self, action_back, action_forward):
@@ -192,9 +161,12 @@ class HelpfulLineEdit(QLineEdit):
 
 class EbookViewer(MainWindow, Ui_EbookViewer):
 
+    STATE_VERSION = 1
+
     def __init__(self, pathtoebook=None, debug_javascript=False):
         MainWindow.__init__(self, None)
         self.setupUi(self)
+        self.base_window_title = unicode(self.windowTitle())
         self.iterator          = None
         self.current_page      = None
         self.pending_search    = None
@@ -203,6 +175,9 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.pending_bookmark  = None
         self.selected_text     = None
         self.read_settings()
+        self.dictionary_box.hide()
+        self.close_dictionary_view.clicked.connect(lambda
+                x:self.dictionary_box.hide())
         self.history = History(self.action_back, self.action_forward)
         self.metadata = Metadata(self)
         self.pos = DoubleSpinBox()
@@ -210,6 +185,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.pos.setToolTip(_('Position in book'))
         self.pos.setSuffix('/'+_('Unknown')+'     ')
         self.pos.setMinimum(1.)
+        self.pos.setMinimumWidth(150)
         self.tool_bar2.insertWidget(self.action_find_next, self.pos)
         self.reference = HelpfulLineEdit()
         self.reference.setValidator(QRegExpValidator(QRegExp(r'\d+\.\d+'), self.reference))
@@ -219,8 +195,10 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.tool_bar2.insertSeparator(self.action_find_next)
         self.setFocusPolicy(Qt.StrongFocus)
         self.search = SearchBox2(self)
+        self.search.setMinimumContentsLength(20)
         self.search.initialize('viewer_search_history')
         self.search.setToolTip(_('Search for text in book'))
+        self.search.setMinimumWidth(200)
         self.tool_bar2.insertWidget(self.action_find_next, self.search)
         self.view.set_manager(self)
         self.view.document.debug_javascript = debug_javascript
@@ -252,7 +230,11 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.connect(self.action_previous_page, SIGNAL('triggered(bool)'),
                      lambda x:self.view.previous_page())
         self.connect(self.action_find_next, SIGNAL('triggered(bool)'),
-                     lambda x:self.find(unicode(self.search.text()), True, repeat=True))
+                     lambda x:self.find(self.search.smart_text, repeat=True))
+        self.connect(self.action_find_previous, SIGNAL('triggered(bool)'),
+                     lambda x:self.find(self.search.smart_text,
+                         repeat=True, backwards=True))
+
         self.connect(self.action_full_screen, SIGNAL('triggered(bool)'),
                      self.toggle_fullscreen)
         self.action_full_screen.setShortcuts([Qt.Key_F11, Qt.CTRL+Qt.SHIFT+Qt.Key_F])
@@ -260,10 +242,10 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.connect(self.action_bookmark, SIGNAL('triggered(bool)'), self.bookmark)
         self.connect(self.action_forward, SIGNAL('triggered(bool)'), self.forward)
         self.connect(self.action_preferences, SIGNAL('triggered(bool)'), lambda x: self.view.config(self))
-        self.connect(self.pos, SIGNAL('valueChanged(double)'), self.goto_page)
+        self.pos.editingFinished.connect(self.goto_page_num)
         self.connect(self.vertical_scrollbar, SIGNAL('valueChanged(int)'),
                      lambda x: self.goto_page(x/100.))
-        self.connect(self.search, SIGNAL('search(PyQt_PyObject, PyQt_PyObject)'), self.find)
+        self.search.search.connect(self.find)
         self.connect(self.toc, SIGNAL('clicked(QModelIndex)'), self.toc_clicked)
         self.connect(self.reference, SIGNAL('goto(PyQt_PyObject)'), self.goto)
 
@@ -271,6 +253,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.action_bookmark.setMenu(self.bookmarks_menu)
         self.set_bookmarks([])
 
+
         if pathtoebook is not None:
             f = functools.partial(self.load_ebook, pathtoebook)
             QTimer.singleShot(50, f)
@@ -292,7 +275,39 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         ca = self.view.copy_action
         ca.setShortcut(QKeySequence.Copy)
         self.addAction(ca)
+        self.restore_state()
 
+    def closeEvent(self, e):
+        self.save_state()
+        return MainWindow.closeEvent(self, e)
+
+    def save_state(self):
+        state = str(self.saveState(self.STATE_VERSION))
+        dynamic['viewer_toolbar_state'] = state
+
+    def restore_state(self):
+        state = dynamic.get('viewer_toolbar_state', None)
+        if state is not None:
+            try:
+                state = QByteArray(state)
+                self.restoreState(state, self.STATE_VERSION)
+            except:
+                pass
+
+
+    def lookup(self, word):
+        self.dictionary_view.setHtml('<html><body><p>'+ \
+            _('Connecting to dict.org to lookup: <b>%s</b>&hellip;')%word + \
+            '</p></body></html>')
+        self.dictionary_box.show()
+        self._lookup = Lookup(word, parent=self)
+        self._lookup.finished.connect(self.looked_up)
+        self._lookup.start()
+
+    def looked_up(self, *args):
+        html = self._lookup.html_result
+        self._lookup = None
+        self.dictionary_view.setHtml(html)
 
     def set_max_width(self):
         from calibre.gui2.viewer.documentview import config
@@ -351,6 +366,9 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         if pos is not None:
             self.goto_page(pos)
 
+    def goto_page_num(self):
+        num = self.pos.value()
+        self.goto_page(num)
 
     def forward(self, x):
         pos = self.history.forward()
@@ -407,13 +425,15 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
             self.set_bookmarks(self.iterator.bookmarks)
 
 
-    def find(self, text, refinement, repeat=False):
+    def find(self, text, repeat=False, backwards=False):
         if not text:
+            self.view.search('')
             return self.search.search_done(False)
         if self.view.search(text):
             self.scrolled(self.view.scroll_fraction)
             return self.search.search_done(True)
-        index = self.iterator.search(text, self.current_index)
+        index = self.iterator.search(text, self.current_index,
+                backwards=backwards)
         if index is None:
             if self.current_index > 0:
                 index = self.iterator.search(text, 0)
@@ -431,10 +451,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
             self.scrolled(self.view.scroll_fraction)
 
     def keyPressEvent(self, event):
-        if event.key() == Qt.Key_F3:
-            text = unicode(self.search.text())
-            self.find(text, True, repeat=True)
-        elif event.key() == Qt.Key_Slash:
+        if event.key() == Qt.Key_Slash:
             self.search.setFocus(Qt.OtherFocusReason)
         else:
             return MainWindow.keyPressEvent(self, event)
@@ -456,7 +473,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
             elif frag:
                 self.view.scroll_to(frag)
         else:
-            QDesktopServices.openUrl(url)
+            open_url(url)
 
     def load_started(self):
         self.open_progress_indicator(_('Loading flow...'))
@@ -485,6 +502,15 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
             self.pending_bookmark = None
         return self.current_index
 
+    def goto_next_section(self):
+        nindex = (self.current_index + 1)%len(self.iterator.spine)
+        self.load_path(self.iterator.spine[nindex])
+
+    def goto_previous_section(self):
+        pindex = (self.current_index - 1 + len(self.iterator.spine)) \
+                % len(self.iterator.spine)
+        self.load_path(self.iterator.spine[pindex])
+
     def load_path(self, path, pos=0.0):
         self.open_progress_indicator(_('Laying out %s')%self.current_title)
         self.view.load_path(path, pos=pos)
@@ -577,7 +603,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
                 self.toc_model = TOC(self.iterator.toc)
                 self.toc.setModel(self.toc_model)
             self.current_title = title
-            self.setWindowTitle(unicode(self.windowTitle())+' - '+title)
+            self.setWindowTitle(self.base_window_title+' - '+title)
             self.pos.setMaximum(sum(self.iterator.pages))
             self.pos.setSuffix(' / %d'%sum(self.iterator.pages))
             self.vertical_scrollbar.setMinimum(100)
@@ -665,7 +691,7 @@ View an ebook.
 def main(args=sys.argv):
     parser = option_parser()
     opts, args = parser.parse_args(args)
-    pid = os.fork() if False and islinux else -1
+    pid = os.fork() if False and (islinux or isfreebsd) else -1
     if pid <= 0:
         app = Application(args)
         app.setWindowIcon(QIcon(I('viewer.svg')))
