@@ -4,56 +4,60 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 Miscellaneous widgets used in the GUI
 '''
 import re, os, traceback
-from PyQt4.Qt import QListView, QIcon, QFont, QLabel, QListWidget, \
+
+from PyQt4.Qt import QIcon, QFont, QLabel, QListWidget, QAction, \
                         QListWidgetItem, QTextCharFormat, QApplication, \
                         QSyntaxHighlighter, QCursor, QColor, QWidget, \
-                        QPixmap, QMovie, QPalette, QTimer, QDialog, \
-                        QAbstractListModel, QVariant, Qt, SIGNAL, \
-                        QRegExp, QSettings, QSize, QModelIndex, \
-                        QAbstractButton, QPainter, QLineEdit, QComboBox, \
-                        QMenu, QStringListModel, QCompleter
+                        QPixmap, QSplitterHandle, QToolButton, \
+                        QAbstractListModel, QVariant, Qt, SIGNAL, pyqtSignal, \
+                        QRegExp, QSettings, QSize, QSplitter, \
+                        QPainter, QLineEdit, QComboBox, QPen, \
+                        QMenu, QStringListModel, QCompleter, QStringList, \
+                        QTimer, QRect
+
+from calibre.gui2 import NONE, error_dialog, pixmap_to_data, gprefs
 
-from calibre.gui2 import human_readable, NONE, TableView, \
-                         qstring_to_unicode, error_dialog
-from calibre.gui2.dialogs.job_view_ui import Ui_Dialog
 from calibre.gui2.filename_pattern_ui import Ui_Form
 from calibre import fit_image
 from calibre.utils.fonts import fontconfig
+from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.metadata.meta import metadata_from_filename
-from calibre.utils.config import prefs
+from calibre.utils.config import prefs, XMLConfig
+from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndicator
+
+history = XMLConfig('history')
 
 class ProgressIndicator(QWidget):
 
     def __init__(self, *args):
         QWidget.__init__(self, *args)
         self.setGeometry(0, 0, 300, 350)
-        self.movie = QMovie(I('jobs-animated.mng'))
-        self.ml = QLabel(self)
-        self.ml.setMovie(self.movie)
-        self.movie.start()
-        self.movie.setPaused(True)
+        self.pi = _ProgressIndicator(self)
         self.status = QLabel(self)
         self.status.setWordWrap(True)
         self.status.setAlignment(Qt.AlignHCenter|Qt.AlignTop)
         self.setVisible(False)
+        self.pos = None
 
     def start(self, msg=''):
         view = self.parent()
         pwidth, pheight = view.size().width(), view.size().height()
         self.resize(pwidth, min(pheight, 250))
-        self.move(0, (pheight-self.size().height())/2.)
-        self.ml.resize(self.ml.sizeHint())
-        self.ml.move(int((self.size().width()-self.ml.size().width())/2.), 0)
-        self.status.resize(self.size().width(), self.size().height()-self.ml.size().height()-10)
-        self.status.move(0, self.ml.size().height()+10)
+        if self.pos is None:
+            self.move(0, (pheight-self.size().height())/2.)
+        else:
+            self.move(self.pos[0], self.pos[1])
+        self.pi.resize(self.pi.sizeHint())
+        self.pi.move(int((self.size().width()-self.pi.size().width())/2.), 0)
+        self.status.resize(self.size().width(), self.size().height()-self.pi.size().height()-10)
+        self.status.move(0, self.pi.size().height()+10)
         self.status.setText('<h1>'+msg+'</h1>')
         self.setVisible(True)
-        self.movie.setPaused(False)
+        self.pi.startAnimation()
 
     def stop(self):
-        if self.movie.state() == self.movie.Running:
-            self.movie.setPaused(True)
-            self.setVisible(False)
+        self.pi.stopAnimation()
+        self.setVisible(False)
 
 class FilenamePattern(QWidget, Ui_Form):
 
@@ -72,7 +76,7 @@ class FilenamePattern(QWidget, Ui_Form):
             error_dialog(self, _('Invalid regular expression'),
                          _('Invalid regular expression: %s')%err).exec_()
             return
-        mi = metadata_from_filename(qstring_to_unicode(self.filename.text()), pat)
+        mi = metadata_from_filename(unicode(self.filename.text()), pat)
         if mi.title:
             self.title.setText(mi.title)
         else:
@@ -96,7 +100,7 @@ class FilenamePattern(QWidget, Ui_Form):
 
 
     def pattern(self):
-        pat = qstring_to_unicode(self.re.text())
+        pat = unicode(self.re.text())
         return re.compile(pat)
 
     def commit(self):
@@ -105,13 +109,10 @@ class FilenamePattern(QWidget, Ui_Form):
         return pat
 
 
+IMAGE_EXTENSIONS = ['jpg', 'jpeg', 'gif', 'png', 'bmp']
 
-
-class ImageView(QLabel):
-
-    MAX_WIDTH  = 400
-    MAX_HEIGHT = 300
-    DROPABBLE_EXTENSIONS = ('jpg', 'jpeg', 'gif', 'png', 'bmp')
+class FormatList(QListWidget):
+    DROPABBLE_EXTENSIONS = BOOK_EXTENSIONS
 
     @classmethod
     def paths_from_event(cls, event):
@@ -120,7 +121,56 @@ class ImageView(QLabel):
         and represent files with extensions.
         '''
         if event.mimeData().hasFormat('text/uri-list'):
-            urls = [qstring_to_unicode(u.toLocalFile()) for u in event.mimeData().urls()]
+            urls = [unicode(u.toLocalFile()) for u in event.mimeData().urls()]
+            urls = [u for u in urls if os.path.splitext(u)[1] and os.access(u, os.R_OK)]
+            return [u for u in urls if os.path.splitext(u)[1][1:].lower() in cls.DROPABBLE_EXTENSIONS]
+
+    def dragEnterEvent(self, event):
+        if int(event.possibleActions() & Qt.CopyAction) + \
+           int(event.possibleActions() & Qt.MoveAction) == 0:
+            return
+        paths = self.paths_from_event(event)
+        if paths:
+            event.acceptProposedAction()
+
+    def dropEvent(self, event):
+        paths = self.paths_from_event(event)
+        event.setDropAction(Qt.CopyAction)
+        self.emit(SIGNAL('formats_dropped(PyQt_PyObject,PyQt_PyObject)'),
+                event, paths)
+
+    def dragMoveEvent(self, event):
+        event.acceptProposedAction()
+
+    def keyPressEvent(self, event):
+        if event.key() == Qt.Key_Delete:
+            self.emit(SIGNAL('delete_format()'))
+        else:
+            return QListWidget.keyPressEvent(self, event)
+
+
+class ImageView(QWidget):
+
+    BORDER_WIDTH = 1
+
+    def __init__(self, parent=None):
+        QWidget.__init__(self, parent)
+        self._pixmap = QPixmap(self)
+        self.setMinimumSize(QSize(150, 200))
+        self.setAcceptDrops(True)
+        self.draw_border = True
+
+    # Drag 'n drop {{{
+    DROPABBLE_EXTENSIONS = IMAGE_EXTENSIONS
+
+    @classmethod
+    def paths_from_event(cls, event):
+        '''
+        Accept a drop event and return a list of paths that can be read from
+        and represent files with extensions.
+        '''
+        if event.mimeData().hasFormat('text/uri-list'):
+            urls = [unicode(u.toLocalFile()) for u in event.mimeData().urls()]
             urls = [u for u in urls if os.path.splitext(u)[1] and os.access(u, os.R_OK)]
             return [u for u in urls if os.path.splitext(u)[1][1:].lower() in cls.DROPABBLE_EXTENSIONS]
 
@@ -141,225 +191,81 @@ class ImageView(QLabel):
             if not pmap.isNull():
                 self.setPixmap(pmap)
                 event.accept()
-                self.cover_data = open(path, 'rb').read()
-                self.emit(SIGNAL('cover_changed(PyQt_PyObject)'), paths)
+                self.emit(SIGNAL('cover_changed(PyQt_PyObject)'), open(path,
+                    'rb').read())
                 break
 
     def dragMoveEvent(self, event):
         event.acceptProposedAction()
+    # }}}
 
     def setPixmap(self, pixmap):
-        QLabel.setPixmap(self, pixmap)
-        width, height = fit_image(pixmap.width(), pixmap.height(), self.MAX_WIDTH, self.MAX_HEIGHT)[1:]
-        self.setMaximumWidth(width)
-        self.setMaximumHeight(height)
+        if not isinstance(pixmap, QPixmap):
+            raise TypeError('Must use a QPixmap')
+        self._pixmap = pixmap
+        self.updateGeometry()
+        self.update()
 
+    def pixmap(self):
+        return self._pixmap
 
-class LocationModel(QAbstractListModel):
-
-    def __init__(self, parent):
-        QAbstractListModel.__init__(self, parent)
-        self.icons = [QVariant(QIcon(I('library.png'))),
-                      QVariant(QIcon(I('reader.svg'))),
-                      QVariant(QIcon(I('sd.svg'))),
-                      QVariant(QIcon(I('sd.svg')))]
-        self.text = [_('Library\n%d\nbooks'),
-                     _('Reader\n%s\navailable'),
-                     _('Card A\n%s\navailable'),
-                     _('Card B\n%s\navailable')]
-        self.free = [-1, -1, -1]
-        self.count = 0
-        self.highlight_row = 0
-        self.tooltips = [
-                         _('Click to see the books available on your computer'),
-                         _('Click to see the books in the main memory of your reader'),
-                         _('Click to see the books on storage card A in your reader'),
-                         _('Click to see the books on storage card B in your reader')
-                         ]
-
-    def rowCount(self, *args):
-        return 1 + len([i for i in self.free if i >= 0])
-
-    def get_device_row(self, row):
-        if row == 2 and self.free[1] == -1 and self.free[2] > -1:
-            row = 3
-        return row
-
-    def data(self, index, role):
-        row = index.row()
-        drow = self.get_device_row(row)
-        data = NONE
-        if role == Qt.DisplayRole:
-            text = self.text[drow]%(human_readable(self.free[drow-1])) if row > 0 \
-                            else self.text[drow]%self.count
-            data = QVariant(text)
-        elif role == Qt.DecorationRole:
-            data = self.icons[drow]
-        elif role == Qt.ToolTipRole:
-            data = QVariant(self.tooltips[drow])
-        elif role == Qt.SizeHintRole:
-            data = QVariant(QSize(155, 90))
-        elif role == Qt.FontRole:
-            font = QFont('monospace')
-            font.setBold(row == self.highlight_row)
-            data = QVariant(font)
-        elif role == Qt.ForegroundRole and row == self.highlight_row:
-            return QVariant(QApplication.palette().brush(
-                QPalette.HighlightedText))
-        elif role == Qt.BackgroundRole and row == self.highlight_row:
-            return QVariant(QApplication.palette().brush(
-                QPalette.Highlight))
-
-        return data
-
-    def headerData(self, section, orientation, role):
-        return NONE
-
-    def update_devices(self, cp=(None, None), fs=[-1, -1, -1]):
-        if cp is None:
-            cp = (None, None)
-        if isinstance(cp, (str, unicode)):
-            cp = (cp, None)
-        if len(fs) < 3:
-            fs = list(fs) + [0]
-        self.free[0] = fs[0]
-        self.free[1] = fs[1]
-        self.free[2] = fs[2]
-        cpa, cpb = cp
-        self.free[1] = fs[1] if fs[1] is not None and cpa is not None else -1
-        self.free[2] = fs[2] if fs[2] is not None and cpb is not None else -1
-        self.reset()
-        self.emit(SIGNAL('devicesChanged()'))
-
-    def location_changed(self, row):
-        self.highlight_row = row
-        self.emit(SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                self.index(0), self.index(self.rowCount(QModelIndex())-1))
-
-    def location_for_row(self, row):
-        if row == 0: return 'library'
-        if row == 1: return 'main'
-        if row == 3: return 'cardb'
-        return 'carda' if self.free[1] > -1 else 'cardb'
-
-class LocationView(QListView):
-
-    def __init__(self, parent):
-        QListView.__init__(self, parent)
-        self.setModel(LocationModel(self))
-        self.reset()
-        self.currentChanged = self.current_changed
-
-        self.eject_button = EjectButton(self)
-        self.eject_button.hide()
-
-        self.connect(self, SIGNAL('entered(QModelIndex)'), self.item_entered)
-        self.connect(self, SIGNAL('viewportEntered()'), self.viewport_entered)
-        self.connect(self.eject_button, SIGNAL('clicked()'), lambda: self.emit(SIGNAL('umount_device()')))
-        self.connect(self.model(), SIGNAL('devicesChanged()'), self.eject_button.hide)
-
-    def count_changed(self, new_count):
-        self.model().count = new_count
-        self.model().reset()
-
-    def current_changed(self, current, previous):
-        if current.isValid():
-            i = current.row()
-            location = self.model().location_for_row(i)
-            self.emit(SIGNAL('location_selected(PyQt_PyObject)'), location)
-            self.model().location_changed(i)
-
-    def location_changed(self, row):
-        if 0 <= row and row <= 3:
-            self.model().location_changed(row)
-
-    def leaveEvent(self, event):
-        self.unsetCursor()
-        self.eject_button.hide()
-
-    def item_entered(self, location):
-        self.setCursor(Qt.PointingHandCursor)
-        self.eject_button.hide()
-
-        if location.row() == 1:
-            rect = self.visualRect(location)
-
-            self.eject_button.resize(rect.height()/2, rect.height()/2)
-
-            x, y = rect.left(), rect.top()
-            x = x + (rect.width() - self.eject_button.width() - 2)
-            y += 6
-
-            self.eject_button.move(x, y)
-            self.eject_button.show()
-
-    def viewport_entered(self):
-        self.unsetCursor()
-        self.eject_button.hide()
-
-
-class EjectButton(QAbstractButton):
-
-    def __init__(self, parent):
-        QAbstractButton.__init__(self, parent)
-        self.mouse_over = False
-
-    def enterEvent(self, event):
-        self.mouse_over = True
-
-    def leaveEvent(self, event):
-        self.mouse_over = False
+    def sizeHint(self):
+        if self._pixmap.isNull():
+            return self.minimumSize()
+        return self._pixmap.size()
 
     def paintEvent(self, event):
-        painter = QPainter(self)
-        painter.setClipRect(event.rect())
-        image = QPixmap(I('eject')).scaledToHeight(event.rect().height(),
-            Qt.SmoothTransformation)
-
-        if not self.mouse_over:
-            alpha_mask = QPixmap(image.width(), image.height())
-            color = QColor(128, 128, 128)
-            alpha_mask.fill(color)
-            image.setAlphaChannel(alpha_mask)
-
-        painter.drawPixmap(0, 0, image)
+        QWidget.paintEvent(self, event)
+        pmap = self._pixmap
+        if pmap.isNull():
+            return
+        w, h = pmap.width(), pmap.height()
+        cw, ch = self.rect().width(), self.rect().height()
+        scaled, nw, nh = fit_image(w, h, cw, ch)
+        if scaled:
+            pmap = pmap.scaled(nw, nh, Qt.IgnoreAspectRatio,
+                    Qt.SmoothTransformation)
+        w, h = pmap.width(), pmap.height()
+        x = int(abs(cw - w)/2.)
+        y = int(abs(ch - h)/2.)
+        target = QRect(x, y, w, h)
+        p = QPainter(self)
+        p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
+        p.drawPixmap(target, pmap)
+        pen = QPen()
+        pen.setWidth(self.BORDER_WIDTH)
+        p.setPen(pen)
+        if self.draw_border:
+            p.drawRect(target)
+        p.end()
 
 
-class DetailView(QDialog, Ui_Dialog):
+    # Clipboard copy/paste # {{{
+    def contextMenuEvent(self, ev):
+        cm = QMenu(self)
+        copy = cm.addAction(_('Copy Image'))
+        paste = cm.addAction(_('Paste Image'))
+        if not QApplication.instance().clipboard().mimeData().hasImage():
+            paste.setEnabled(False)
+        copy.triggered.connect(self.copy_to_clipboard)
+        paste.triggered.connect(self.paste_from_clipboard)
+        cm.exec_(ev.globalPos())
 
-    def __init__(self, parent, job):
-        QDialog.__init__(self, parent)
-        self.setupUi(self)
-        self.setWindowTitle(job.description)
-        self.job = job
-        self.next_pos = 0
-        self.update()
-        self.timer = QTimer(self)
-        self.connect(self.timer, SIGNAL('timeout()'), self.update)
-        self.timer.start(1000)
+    def copy_to_clipboard(self):
+        QApplication.instance().clipboard().setPixmap(self.pixmap())
+
+    def paste_from_clipboard(self):
+        cb = QApplication.instance().clipboard()
+        pmap = cb.pixmap()
+        if pmap.isNull() and cb.supportsSelection():
+            pmap = cb.pixmap(cb.Selection)
+        if not pmap.isNull():
+            self.setPixmap(pmap)
+            self.emit(SIGNAL('cover_changed(PyQt_PyObject)'),
+                    pixmap_to_data(pmap))
+    # }}}
 
 
-    def update(self):
-        f = self.job.log_file
-        f.seek(self.next_pos)
-        more = f.read()
-        self.next_pos = f.tell()
-        if more:
-            self.log.appendPlainText(more.decode('utf-8', 'replace'))
-
-
-class JobsView(TableView):
-
-    def __init__(self, parent):
-        TableView.__init__(self, parent)
-        self.connect(self, SIGNAL('doubleClicked(QModelIndex)'), self.show_details)
-
-    def show_details(self, index):
-        row = index.row()
-        job = self.model().row_to_job(row)
-        d = DetailView(self, job)
-        d.exec_()
-        d.timer.stop()
 
 
 class FontFamilyModel(QAbstractListModel):
@@ -456,7 +362,7 @@ class BasicList(QListWidget):
 class LineEditECM(object):
 
     '''
-    Extend the contenxt menu of a QLineEdit to include more actions.
+    Extend the context menu of a QLineEdit to include more actions.
     '''
 
     def contextMenuEvent(self, event):
@@ -478,16 +384,17 @@ class LineEditECM(object):
         menu.exec_(event.globalPos())
 
     def upper_case(self):
-        self.setText(qstring_to_unicode(self.text()).upper())
+        self.setText(unicode(self.text()).upper())
 
     def lower_case(self):
-        self.setText(qstring_to_unicode(self.text()).lower())
+        self.setText(unicode(self.text()).lower())
 
     def swap_case(self):
-        self.setText(qstring_to_unicode(self.text()).swapcase())
+        self.setText(unicode(self.text()).swapcase())
 
     def title_case(self):
-        self.setText(qstring_to_unicode(self.text()).title())
+        from calibre.utils.titlecase import titlecase
+        self.setText(titlecase(unicode(self.text())))
 
 
 class EnLineEdit(LineEditECM, QLineEdit):
@@ -555,13 +462,13 @@ class TagsLineEdit(EnLineEdit):
         self.completer.update_tags_cache(tags)
 
     def text_changed(self, text):
-        all_text = qstring_to_unicode(text)
+        all_text = unicode(text)
         text = all_text[:self.cursorPosition()]
         prefix = text.split(',')[-1].strip()
 
         text_tags = []
         for t in all_text.split(self.separator):
-            t1 = qstring_to_unicode(t).strip()
+            t1 = unicode(t).strip()
             if t1 != '':
                 text_tags.append(t)
         text_tags = list(set(text_tags))
@@ -571,8 +478,8 @@ class TagsLineEdit(EnLineEdit):
 
     def complete_text(self, text):
         cursor_pos = self.cursorPosition()
-        before_text = qstring_to_unicode(self.text())[:cursor_pos]
-        after_text = qstring_to_unicode(self.text())[cursor_pos:]
+        before_text = unicode(self.text())[:cursor_pos]
+        after_text = unicode(self.text())[cursor_pos:]
         prefix_len = len(before_text.split(',')[-1].strip())
         self.setText('%s%s%s %s' % (before_text[:cursor_pos - prefix_len],
             text, self.separator, after_text))
@@ -584,23 +491,109 @@ class EnComboBox(QComboBox):
     '''
     Enhanced QComboBox.
 
-    Includes an extended content menu.
+    Includes an extended context menu.
     '''
 
     def __init__(self, *args):
         QComboBox.__init__(self, *args)
         self.setLineEdit(EnLineEdit(self))
+        self.setAutoCompletionCaseSensitivity(Qt.CaseSensitive)
+        self.setMinimumContentsLength(20)
 
     def text(self):
-        return qstring_to_unicode(self.currentText())
+        return unicode(self.currentText())
 
     def setText(self, text):
-        idx = self.findText(text, Qt.MatchFixedString)
+        idx = self.findText(text, Qt.MatchFixedString|Qt.MatchCaseSensitive)
         if idx == -1:
             self.insertItem(0, text)
             idx = 0
         self.setCurrentIndex(idx)
 
+class HistoryLineEdit(QComboBox):
+
+    def __init__(self, *args):
+        QComboBox.__init__(self, *args)
+        self.setEditable(True)
+        self.setInsertPolicy(self.NoInsert)
+        self.setMaxCount(10)
+
+    @property
+    def store_name(self):
+        return 'lineedit_history_'+self._name
+
+    def initialize(self, name):
+        self._name = name
+        self.addItems(QStringList(history.get(self.store_name, [])))
+        self.setEditText('')
+        self.lineEdit().editingFinished.connect(self.save_history)
+
+    def save_history(self):
+        items = []
+        ct = unicode(self.currentText())
+        if ct:
+            items.append(ct)
+        for i in range(self.count()):
+            item = unicode(self.itemText(i))
+            if item not in items:
+                items.append(item)
+
+        history.set(self.store_name, items)
+
+    def setText(self, t):
+        self.setEditText(t)
+        self.lineEdit().setCursorPosition(0)
+
+    def text(self):
+        return self.currentText()
+
+class ComboBoxWithHelp(QComboBox):
+    '''
+    A combobox where item 0 is help text. CurrentText will return '' for item 0.
+    Be sure to always fetch the text with currentText. Don't use the signals
+    that pass a string, because they will not correct the text.
+    '''
+    def __init__(self, parent=None):
+        QComboBox.__init__(self, parent)
+        self.currentIndexChanged[int].connect(self.index_changed)
+        self.help_text = ''
+        self.state_set = False
+
+    def initialize(self, help_text=_('Search')):
+        self.help_text = help_text
+        self.set_state()
+
+    def set_state(self):
+        if not self.state_set:
+            if self.currentIndex() == 0:
+                self.setItemText(0, self.help_text)
+                self.setStyleSheet('QComboBox { color: gray }')
+            else:
+                self.setItemText(0, '')
+                self.setStyleSheet('QComboBox { color: black }')
+
+    def index_changed(self, index):
+        self.state_set = False
+        self.set_state()
+
+    def currentText(self):
+        if self.currentIndex() == 0:
+            return ''
+        return QComboBox.currentText(self)
+
+    def itemText(self, idx):
+        if idx == 0:
+            return ''
+        return QComboBox.itemText(self, idx)
+
+    def showPopup(self):
+        self.setItemText(0, '')
+        QComboBox.showPopup(self)
+
+    def hidePopup(self):
+        QComboBox.hidePopup(self)
+        self.set_state()
+
 class PythonHighlighter(QSyntaxHighlighter):
 
     Rules = []
@@ -838,3 +831,232 @@ class PythonHighlighter(QSyntaxHighlighter):
         QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
         QSyntaxHighlighter.rehighlight(self)
         QApplication.restoreOverrideCursor()
+
+class SplitterHandle(QSplitterHandle):
+
+    double_clicked = pyqtSignal(object)
+
+    def __init__(self, orientation, splitter):
+        QSplitterHandle.__init__(self, orientation, splitter)
+        splitter.splitterMoved.connect(self.splitter_moved,
+                type=Qt.QueuedConnection)
+        self.double_clicked.connect(splitter.double_clicked,
+                type=Qt.QueuedConnection)
+        self.highlight = False
+        self.setToolTip(_('Drag to resize')+' '+splitter.label)
+
+    def splitter_moved(self, *args):
+        oh = self.highlight
+        self.highlight = 0 in self.splitter().sizes()
+        if oh != self.highlight:
+            self.update()
+
+    def paintEvent(self, ev):
+        QSplitterHandle.paintEvent(self, ev)
+        if self.highlight:
+            painter = QPainter(self)
+            painter.setClipRect(ev.rect())
+            painter.fillRect(self.rect(), Qt.yellow)
+
+    def mouseDoubleClickEvent(self, ev):
+        self.double_clicked.emit(self)
+
+class LayoutButton(QToolButton):
+
+    def __init__(self, icon, text, splitter, parent=None, shortcut=None):
+        QToolButton.__init__(self, parent)
+        self.label = text
+        self.setIcon(QIcon(icon))
+        self.setCheckable(True)
+
+        self.splitter = splitter
+        splitter.state_changed.connect(self.update_state)
+        self.setCursor(Qt.PointingHandCursor)
+        self.shortcut = ''
+        if shortcut:
+            self.shortcut = shortcut
+
+    def set_state_to_show(self, *args):
+        self.setChecked(False)
+        label =_('Show')
+        self.setText(label + ' ' + self.label + u' (%s)'%self.shortcut)
+        self.setToolTip(self.text())
+        self.setStatusTip(self.text())
+
+    def set_state_to_hide(self, *args):
+        self.setChecked(True)
+        label = _('Hide')
+        self.setText(label + ' ' + self.label+ u' (%s)'%self.shortcut)
+        self.setToolTip(self.text())
+        self.setStatusTip(self.text())
+
+    def update_state(self, *args):
+        if self.splitter.is_side_index_hidden:
+            self.set_state_to_show()
+        else:
+            self.set_state_to_hide()
+
+class Splitter(QSplitter):
+
+    state_changed = pyqtSignal(object)
+
+    def __init__(self, name, label, icon, initial_show=True,
+            initial_side_size=120, connect_button=True,
+            orientation=Qt.Horizontal, side_index=0, parent=None, shortcut=None):
+        QSplitter.__init__(self, parent)
+        self.resize_timer = QTimer(self)
+        self.resize_timer.setSingleShot(True)
+        self.desired_side_size = initial_side_size
+        self.desired_show = initial_show
+        self.resize_timer.setInterval(5)
+        self.resize_timer.timeout.connect(self.do_resize)
+        self.setOrientation(orientation)
+        self.side_index = side_index
+        self._name = name
+        self.label = label
+        self.initial_side_size = initial_side_size
+        self.initial_show = initial_show
+        self.splitterMoved.connect(self.splitter_moved, type=Qt.QueuedConnection)
+        self.button = LayoutButton(icon, label, self, shortcut=shortcut)
+        if connect_button:
+            self.button.clicked.connect(self.double_clicked)
+
+        if shortcut is not None:
+            self.action_toggle = QAction(QIcon(icon), _('Toggle') + ' ' + label,
+                    self)
+            self.action_toggle.triggered.connect(self.toggle_triggered)
+            self.action_toggle.setShortcut(shortcut)
+            if parent is not None:
+                parent.addAction(self.action_toggle)
+
+    def toggle_triggered(self, *args):
+        self.toggle_side_pane()
+
+    def createHandle(self):
+        return SplitterHandle(self.orientation(), self)
+
+    def initialize(self):
+        for i in range(self.count()):
+            h = self.handle(i)
+            if h is not None:
+                h.splitter_moved()
+        self.state_changed.emit(not self.is_side_index_hidden)
+
+    def splitter_moved(self, *args):
+        self.desired_side_size = self.side_index_size
+        self.state_changed.emit(not self.is_side_index_hidden)
+
+    @property
+    def is_side_index_hidden(self):
+        sizes = list(self.sizes())
+        try:
+            return sizes[self.side_index] == 0
+        except IndexError:
+            return True
+
+    @property
+    def save_name(self):
+        ori = 'horizontal' if self.orientation() == Qt.Horizontal \
+                else 'vertical'
+        return self._name + '_' + ori
+
+    def print_sizes(self):
+        if self.count() > 1:
+            print self.save_name, 'side:', self.side_index_size, 'other:',
+            print list(self.sizes())[self.other_index]
+
+    @dynamic_property
+    def side_index_size(self):
+        def fget(self):
+            if self.count() < 2: return 0
+            return self.sizes()[self.side_index]
+
+        def fset(self, val):
+            if self.count() < 2: return
+            if val == 0 and not self.is_side_index_hidden:
+                self.save_state()
+            sizes = list(self.sizes())
+            for i in range(len(sizes)):
+                sizes[i] = val if i == self.side_index else 10
+            self.setSizes(sizes)
+            total = sum(self.sizes())
+            sizes = list(self.sizes())
+            for i in range(len(sizes)):
+                sizes[i] = val if i == self.side_index else total-val
+            self.setSizes(sizes)
+            self.initialize()
+
+        return property(fget=fget, fset=fset)
+
+    def do_resize(self, *args):
+        orig = self.desired_side_size
+        QSplitter.resizeEvent(self, self._resize_ev)
+        if orig > 20 and self.desired_show:
+            c = 0
+            while abs(self.side_index_size - orig) > 10 and c < 5:
+                self.apply_state(self.get_state(), save_desired=False)
+                c += 1
+
+    def resizeEvent(self, ev):
+        if self.resize_timer.isActive():
+            self.resize_timer.stop()
+        self._resize_ev = ev
+        self.resize_timer.start()
+
+    def get_state(self):
+        if self.count() < 2: return (False, 200)
+        return (self.desired_show, self.desired_side_size)
+
+    def apply_state(self, state, save_desired=True):
+        if state[0]:
+            self.side_index_size = state[1]
+            if save_desired:
+                self.desired_side_size = self.side_index_size
+        else:
+            self.side_index_size = 0
+        self.desired_show = state[0]
+
+    def default_state(self):
+        return (self.initial_show, self.initial_side_size)
+
+    # Public API {{{
+
+    def save_state(self):
+        if self.count() > 1:
+            gprefs[self.save_name+'_state'] = self.get_state()
+
+    @property
+    def other_index(self):
+        return (self.side_index+1)%2
+
+    def restore_state(self):
+        if self.count() > 1:
+            state = gprefs.get(self.save_name+'_state',
+                    self.default_state())
+            self.apply_state(state, save_desired=False)
+            self.desired_side_size = state[1]
+
+    def toggle_side_pane(self, hide=None):
+        if hide is None:
+            action = 'show' if self.is_side_index_hidden else 'hide'
+        else:
+            action = 'hide' if hide else 'show'
+        getattr(self, action+'_side_pane')()
+
+    def show_side_pane(self):
+        if self.count() < 2 or not self.is_side_index_hidden:
+            return
+        if self.desired_side_size == 0:
+            self.desired_side_size = self.initial_side_size
+        self.apply_state((True, self.desired_side_size))
+
+    def hide_side_pane(self):
+        if self.count() < 2 or self.is_side_index_hidden:
+            return
+        self.apply_state((False, self.desired_side_size))
+
+    def double_clicked(self, *args):
+        self.toggle_side_pane()
+
+    # }}}
+
