@@ -29,12 +29,14 @@ from calibre.utils.config import dynamic, prefs
 from calibre.gui2 import NONE, choose_dir, error_dialog
 from calibre.gui2.dialogs.progress import ProgressDialog
 
+# Devices {{{
+
 class Device(object):
 
     output_profile = 'default'
     output_format = 'EPUB'
-    name = _('Default')
-    manufacturer = _('Default')
+    name = 'Default'
+    manufacturer = 'Default'
     id = 'default'
 
     @classmethod
@@ -59,7 +61,7 @@ class Kindle(Device):
 
     output_profile = 'kindle'
     output_format  = 'MOBI'
-    name = 'Kindle 1 or 2'
+    name = 'Kindle 1, 2 or 3'
     manufacturer = 'Amazon'
     id = 'kindle'
 
@@ -78,20 +80,56 @@ class KindleDX(Kindle):
     name = 'Kindle DX'
     id = 'kindledx'
 
-class Sony500(Device):
+class Sony505(Device):
 
     output_profile = 'sony'
-    name = 'SONY PRS 500'
-    output_format = 'LRF'
-    manufacturer = 'SONY'
-    id = 'prs500'
-
-class Sony505(Sony500):
-
+    name = 'All other SONY devices'
     output_format = 'EPUB'
-    name = 'SONY Reader Pocket/Touch Edition'
+    manufacturer = 'SONY'
     id = 'prs505'
 
+class Kobo(Device):
+    name = 'Kobo Reader'
+    manufacturer = 'Kobo'
+    output_profile = 'kobo'
+    output_format = 'EPUB'
+    id = 'kobo'
+
+class Booq(Device):
+    name = 'Booq Reader'
+    manufacturer = 'Booq'
+    output_profile = 'sony'
+    output_format = 'EPUB'
+    id = 'booq'
+
+class TheBook(Device):
+    name = 'The Book'
+    manufacturer = 'Augen'
+    output_profile = 'sony'
+    output_format = 'EPUB'
+    id = 'thebook'
+
+class Avant(Booq):
+    name = 'Booq Avant'
+
+class Sony300(Sony505):
+
+    name = 'SONY Reader Pocket Edition'
+    id = 'prs300'
+    output_profile = 'sony300'
+
+class Sony900(Sony505):
+
+    name = 'SONY Reader Daily Edition'
+    id = 'prs900'
+    output_profile = 'sony900'
+
+class Nook(Sony505):
+    id = 'nook'
+    name = 'Nook'
+    manufacturer = 'Barnes & Noble'
+    output_profile = 'nook'
+
 class CybookG3(Device):
 
     name = 'Cybook Gen 3'
@@ -107,36 +145,74 @@ class CybookOpus(CybookG3):
     output_profile = 'cybook_opus'
     id = 'cybook_opus'
 
-class BeBook(Device):
+class PocketBook360(CybookOpus):
 
-    name = 'BeBook or BeBook Mini'
-    output_format = 'EPUB'
-    output_profile = 'sony'
-    manufacturer = 'Endless Ideas'
-    id = 'bebook'
+    manufacturer = 'PocketBook'
+    name = 'PocketBook 360'
+    id = 'pocketbook360'
+    output_profile = 'cybook_opus'
+
+class PocketBook(CybookG3):
+
+    manufacturer = 'PocketBook'
+    name = 'PocketBook 301/302'
+    id = 'pocketbook'
+    output_profile = 'cybookg3'
 
 class iPhone(Device):
 
-    name = 'iPhone/iTouch + Stanza'
+    name = 'iPad or iPhone/iTouch + Stanza'
     output_format = 'EPUB'
     manufacturer = 'Apple'
     id = 'iphone'
 
 class Android(Device):
 
-    name = 'Adroid phone + WordPlayer'
+    name = 'Adroid phone + WordPlayer/Aldiko'
     output_format = 'EPUB'
-    manufacturer = 'Google/HTC'
+    manufacturer = 'Android'
     id = 'android'
 
-class Hanlin(Device):
+class HanlinV3(Device):
 
     name = 'Hanlin V3'
-    output_format = 'MOBI'
+    output_format = 'EPUB'
     output_profile = 'hanlinv3'
-    manufacturer = 'Hanlin'
+    manufacturer = 'Jinke'
     id = 'hanlinv3'
 
+class HanlinV5(HanlinV3):
+
+    name = 'Hanlin V5'
+    output_profile = 'hanlinv5'
+    id = 'hanlinv5'
+
+class BeBook(HanlinV3):
+
+    name = 'BeBook'
+    manufacturer = 'Endless Ideas'
+    id = 'bebook'
+
+class BeBookMini(HanlinV5):
+
+    name = 'BeBook Mini'
+    manufacturer = 'Endless Ideas'
+    id = 'bebook_mini'
+
+class EZReader(HanlinV3):
+
+    name = 'EZReader'
+    manufacturer = 'Astak'
+    id = 'ezreader'
+
+class EZReaderPP(HanlinV5):
+
+    name = 'EZReader Pocket Pro'
+    manufacturer = 'Astak'
+    id = 'ezreader_pp'
+
+# }}}
+
 def get_devices():
     for x in globals().values():
         if isinstance(x, type) and issubclass(x, Device):
@@ -146,8 +222,9 @@ def get_manufacturers():
     mans = set([])
     for x in get_devices():
         mans.add(x.manufacturer)
-    mans.remove(_('Default'))
-    return [_('Default')] + sorted(mans)
+    if 'Default' in mans:
+        mans.remove('Default')
+    return ['Default'] + sorted(mans)
 
 def get_devices_of(manufacturer):
     ans = [d for d in get_devices() if d.manufacturer == manufacturer]
@@ -257,9 +334,14 @@ class StanzaPage(QWizardPage, StanzaUI):
         return FinishPage.ID
 
     def commit(self):
+        from calibre.ebooks.conversion.config import load_defaults, save_defaults
+        recs = load_defaults('comic_input')
+        recs['dont_grayscale'] = True
+        save_defaults('comic_input', recs)
+
         p = self.set_port()
         if p is not None:
-            from calibre.library import server_config
+            from calibre.library.server import server_config
             c = server_config()
             c.set('port', p)
 
@@ -436,7 +518,7 @@ def move_library(oldloc, newloc, parent, callback_on_complete):
 
         # Try to load existing library at new location
         try:
-            ndb = LibraryDatabase2(newloc)
+            LibraryDatabase2(newloc)
         except Exception, err:
             det = traceback.format_exc()
             error_dialog(parent, _('Invalid database'),
@@ -464,6 +546,44 @@ class LibraryPage(QWizardPage, LibraryUI):
         self.setupUi(self)
         self.registerField('library_location', self.location)
         self.connect(self.button_change, SIGNAL('clicked()'), self.change)
+        self.init_languages()
+        self.connect(self.language, SIGNAL('currentIndexChanged(int)'),
+                self.change_language)
+        self.connect(self.location, SIGNAL('textChanged(QString)'),
+                self.location_text_changed)
+
+    def location_text_changed(self, newtext):
+        self.emit(SIGNAL('completeChanged()'))
+
+    def init_languages(self):
+        self.language.blockSignals(True)
+        self.language.clear()
+        from calibre.utils.localization import available_translations, \
+            get_language, get_lang
+        lang = get_lang()
+        if lang is None or lang not in available_translations():
+            lang = 'en'
+        self.language.addItem(get_language(lang), QVariant(lang))
+        items = [(l, get_language(l)) for l in available_translations() \
+                 if l != lang]
+        if lang != 'en':
+            items.append(('en', get_language('en')))
+        items.sort(cmp=lambda x, y: cmp(x[1], y[1]))
+        for item in items:
+            self.language.addItem(item[1], QVariant(item[0]))
+        self.language.blockSignals(False)
+        prefs['language'] = str(self.language.itemData(self.language.currentIndex()).toString())
+
+    def change_language(self, idx):
+        prefs['language'] = str(self.language.itemData(self.language.currentIndex()).toString())
+        import __builtin__
+        __builtin__.__dict__['_'] = lambda(x): x
+        from calibre.utils.localization import set_translators
+        from calibre.gui2 import qt_app
+        set_translators()
+        qt_app.load_translations()
+        self.emit(SIGNAL('retranslate()'))
+        self.init_languages()
 
     def change(self):
         dir = choose_dir(self, 'database location dialog',
@@ -473,6 +593,7 @@ class LibraryPage(QWizardPage, LibraryUI):
 
     def initializePage(self):
         lp = prefs['library_path']
+        self.default_library_name = None
         if not lp:
             fname = _('Calibre Library')
             if isinstance(fname, unicode):
@@ -481,6 +602,7 @@ class LibraryPage(QWizardPage, LibraryUI):
                 except:
                     fname = 'Calibre Library'
             lp = os.path.expanduser('~'+os.sep+fname)
+            self.default_library_name = lp
             if not os.path.exists(lp):
                 try:
                     os.makedirs(lp)
@@ -490,13 +612,28 @@ class LibraryPage(QWizardPage, LibraryUI):
         self.location.setText(lp)
 
     def isComplete(self):
-        lp = unicode(self.location.text())
-        return lp and os.path.exists(lp) and os.path.isdir(lp) and os.access(lp,
-                os.W_OK)
+        try:
+            lp = unicode(self.location.text())
+            ans = bool(lp) and os.path.exists(lp) and os.path.isdir(lp) and os.access(lp,
+                    os.W_OK)
+        except:
+            ans = False
+        return ans
 
     def commit(self, completed):
         oldloc = prefs['library_path']
         newloc = unicode(self.location.text())
+        try:
+            newloce = newloc.encode(filesystem_encoding)
+            if self.default_library_name is not None and \
+                os.path.exists(self.default_library_name) and \
+                not os.listdir(self.default_library_name) and \
+                newloce != self.default_library_name:
+                    os.rmdir(self.default_library_name)
+        except:
+            pass
+        if not os.path.exists(newloc):
+            os.mkdir(newloc)
         if not patheq(oldloc, newloc):
             move_library(oldloc, newloc, self.wizard(), completed)
             return True
@@ -535,10 +672,10 @@ class Wizard(QWizard):
         self.setPixmap(self.BackgroundPixmap, QPixmap(I('wizard.svg')))
         self.device_page = DevicePage()
         self.library_page = LibraryPage()
+        self.connect(self.library_page, SIGNAL('retranslate()'),
+                self.retranslate)
         self.finish_page = FinishPage()
-        bt = unicode(self.buttonText(self.FinishButton)).replace('&', '')
-        t = unicode(self.finish_page.finish_text.text())
-        self.finish_page.finish_text.setText(t%bt)
+        self.set_finish_text()
         self.kindle_page = KindlePage()
         self.stanza_page = StanzaPage()
         self.word_player_page = WordPlayerPage()
@@ -559,6 +696,11 @@ class Wizard(QWizard):
         nw = min(580, nw)
         self.resize(nw, nh)
 
+    def retranslate(self):
+        for pid in self.pageIds():
+            page = self.page(pid)
+            page.retranslateUi(page)
+        self.set_finish_text()
 
     def accept(self):
         pages = map(self.page, self.visitedPages())
@@ -572,12 +714,19 @@ class Wizard(QWizard):
     def completed(self, newloc):
         return QWizard.accept(self)
 
+    def set_finish_text(self, *args):
+        bt = unicode(self.buttonText(self.FinishButton)).replace('&', '')
+        t = unicode(self.finish_page.finish_text.text())
+        if '%s' in t:
+            self.finish_page.finish_text.setText(t%bt)
+
+
 def wizard(parent=None):
     w = Wizard(parent)
     return w
 
 if __name__ == '__main__':
-    from PyQt4.Qt import QApplication
-    app = QApplication([])
+    from calibre.gui2 import Application
+    app = Application([])
     wizard().exec_()
 
