@@ -7,95 +7,19 @@ __docformat__ = 'restructuredtext en'
 Command line interface to the calibre database.
 '''
 
-import sys, os, cStringIO
+import sys, os, cStringIO, re
 from textwrap import TextWrapper
-from urllib import quote
 
 from calibre import terminal_controller, preferred_encoding, prints
-from calibre.utils.config import OptionParser, prefs
+from calibre.utils.config import OptionParser, prefs, tweaks
 from calibre.ebooks.metadata.meta import get_metadata
 from calibre.library.database2 import LibraryDatabase2
 from calibre.ebooks.metadata.opf2 import OPFCreator, OPF
-from calibre.utils.genshi.template import MarkupTemplate
+from calibre.utils.date import isoformat
 
-FIELDS = set(['title', 'authors', 'author_sort', 'publisher', 'rating', 'timestamp', 'size', 'tags', 'comments', 'series', 'series_index', 'formats', 'isbn', 'cover'])
-
-XML_TEMPLATE = '''\
-<?xml version="1.0"  encoding="UTF-8"?>
-<calibredb xmlns:py="http://genshi.edgewall.org/">
-<py:for each="record in data">
-    <record>
-        <id>${record['id']}</id>
-        <title>${record['title']}</title>
-        <authors sort="${record['author_sort']}">
-        <py:for each="author in record['authors']">
-            <author>$author</author>
-        </py:for>
-        </authors>
-        <publisher>${record['publisher']}</publisher>
-        <rating>${record['rating']}</rating>
-        <date>${record['timestamp']}</date>
-        <size>${record['size']}</size>
-        <tags py:if="record['tags']">
-        <py:for each="tag in record['tags']">
-            <tag>$tag</tag>
-        </py:for>
-        </tags>
-        <comments>${record['comments']}</comments>
-        <series py:if="record['series']" index="${record['series_index']}">${record['series']}</series>
-        <isbn>${record['isbn']}</isbn>
-        <cover py:if="record['cover']">${record['cover']}</cover>
-        <formats py:if="record['formats']">
-        <py:for each="path in record['formats']">
-            <format>${path}</format>
-        </py:for>
-        </formats>
-    </record>
-</py:for>
-</calibredb>
-'''
-
-STANZA_TEMPLATE='''\
-<?xml version="1.0" encoding="utf-8"?>
-<feed xmlns="http://www.w3.org/2005/Atom" xmlns:py="http://genshi.edgewall.org/">
-  <title>calibre Library</title>
-  <author>
-    <name>calibre</name>
-    <uri>http://calibre.kovidgoyal.net</uri>
-  </author>
-  <id>$id</id>
-  <updated>${updated.strftime('%Y-%m-%dT%H:%M:%SZ')}</updated>
-  <subtitle>
-        ${subtitle}
-  </subtitle>
-  <py:for each="record in data">
-  <entry>
-      <title>${record['title']}</title>
-      <id>urn:calibre:${record['id']}</id>
-      <author><name>${record['author_sort']}</name></author>
-      <updated>${record['timestamp'].strftime('%Y-%m-%dT%H:%M:%SZ')}</updated>
-      <link type="application/epub+zip" href="${quote(record['fmt_epub'].replace(sep, '/')).replace('http%3A', 'http:')}" />
-      <link py:if="record['cover']" rel="x-stanza-cover-image" type="image/png" href="${quote(record['cover'].replace(sep, '/')).replace('http%3A', 'http:')}" />
-      <link py:if="record['cover']" rel="x-stanza-cover-image-thumbnail" type="image/png" href="${quote(record['cover'].replace(sep, '/')).replace('http%3A', 'http:')}" />
-      <content type="xhtml">
-          <div xmlns="http://www.w3.org/1999/xhtml">
-              <py:for each="f in ('authors', 'publisher', 'rating', 'tags', 'series', 'isbn')">
-              <py:if test="record[f]">
-              ${f.capitalize()}:${unicode(', '.join(record[f]) if f=='tags' else record[f])}
-              <py:if test="f =='series'"># ${str(record['series_index'])}</py:if>
-              <br/>
-              </py:if>
-              </py:for>
-              <py:if test="record['comments']">
-              <br/>
-              ${record['comments']}
-              </py:if>
-          </div>
-      </content>
-  </entry>
-  </py:for>
-</feed>
-'''
+FIELDS = set(['title', 'authors', 'author_sort', 'publisher', 'rating',
+    'timestamp', 'size', 'tags', 'comments', 'series', 'series_index',
+    'formats', 'isbn', 'uuid', 'pubdate', 'cover'])
 
 def send_message(msg=''):
     prints('Notifying calibre of the change')
@@ -114,7 +38,7 @@ def send_message(msg=''):
 def get_parser(usage):
     parser = OptionParser(usage)
     go = parser.add_option_group('GLOBAL OPTIONS')
-    go.add_option('--library-path', default=None, help=_('Path to the calibre library. Default is to use the path stored in the settings.'))
+    go.add_option('--library-path', '--with-library', default=None, help=_('Path to the calibre library. Default is to use the path stored in the settings.'))
 
     return parser
 
@@ -124,75 +48,75 @@ def get_db(dbpath, options):
     dbpath = os.path.abspath(dbpath)
     return LibraryDatabase2(dbpath)
 
-def do_list(db, fields, sort_by, ascending, search_text, line_width, separator,
-            prefix, output_format, subtitle='Books in the calibre database'):
+def do_list(db, fields, afields, sort_by, ascending, search_text, line_width, separator,
+            prefix, subtitle='Books in the calibre database'):
     if sort_by:
         db.sort(sort_by, ascending)
     if search_text:
         db.search(search_text)
-    authors_to_string = output_format in ['stanza', 'text']
-    data = db.get_data_as_dict(prefix, authors_as_string=authors_to_string)
+    data = db.get_data_as_dict(prefix, authors_as_string=True)
     fields = ['id'] + fields
-    if output_format == 'text':
-        for f in data:
-            fmts = [x for x in f['formats'] if x is not None]
-            f['formats'] = u'[%s]'%u','.join(fmts)
-        widths = list(map(lambda x : 0, fields))
-        for record in data:
-            for f in record.keys():
+    title_fields = fields
+    fields = [db.custom_column_label_map[x[1:]]['num'] if x[0]=='*'
+            else x for x in fields]
+
+    for f in data:
+        fmts = [x for x in f['formats'] if x is not None]
+        f['formats'] = u'[%s]'%u','.join(fmts)
+    widths = list(map(lambda x : 0, fields))
+    for record in data:
+        for f in record.keys():
+            if hasattr(record[f], 'isoformat'):
+                record[f] = isoformat(record[f], as_utc=False)
+            else:
                 record[f] = unicode(record[f])
-                record[f] = record[f].replace('\n', ' ')
-        for i in data:
-            for j, field in enumerate(fields):
-                widths[j] = max(widths[j], len(unicode(i[str(field)])))
+            record[f] = record[f].replace('\n', ' ')
+    for i in data:
+        for j, field in enumerate(fields):
+            widths[j] = max(widths[j], len(unicode(i[field])))
 
-        screen_width = terminal_controller.COLS if line_width < 0 else line_width
-        if not screen_width:
-            screen_width = 80
-        field_width = screen_width//len(fields)
-        base_widths = map(lambda x: min(x+1, field_width), widths)
+    screen_width = terminal_controller.COLS if line_width < 0 else line_width
+    if not screen_width:
+        screen_width = 80
+    field_width = screen_width//len(fields)
+    base_widths = map(lambda x: min(x+1, field_width), widths)
 
-        while sum(base_widths) < screen_width:
-            adjusted = False
-            for i in range(len(widths)):
-                if base_widths[i] < widths[i]:
-                    base_widths[i] += min(screen_width-sum(base_widths), widths[i]-base_widths[i])
-                    adjusted = True
-                    break
-            if not adjusted:
+    while sum(base_widths) < screen_width:
+        adjusted = False
+        for i in range(len(widths)):
+            if base_widths[i] < widths[i]:
+                base_widths[i] += min(screen_width-sum(base_widths), widths[i]-base_widths[i])
+                adjusted = True
                 break
+        if not adjusted:
+            break
 
-        widths = list(base_widths)
-        titles = map(lambda x, y: '%-*s%s'%(x-len(separator), y, separator), widths, fields)
-        print terminal_controller.GREEN + ''.join(titles)+terminal_controller.NORMAL
+    widths = list(base_widths)
+    titles = map(lambda x, y: '%-*s%s'%(x-len(separator), y, separator),
+            widths, title_fields)
+    print terminal_controller.GREEN + ''.join(titles)+terminal_controller.NORMAL
 
-        wrappers = map(lambda x: TextWrapper(x-1), widths)
-        o = cStringIO.StringIO()
+    wrappers = map(lambda x: TextWrapper(x-1), widths)
+    o = cStringIO.StringIO()
 
-        for record in data:
-            text = [wrappers[i].wrap(unicode(record[field]).encode('utf-8')) for i, field in enumerate(fields)]
-            lines = max(map(len, text))
-            for l in range(lines):
-                for i, field in enumerate(text):
-                    ft = text[i][l] if l < len(text[i]) else ''
-                    filler = '%*s'%(widths[i]-len(ft)-1, '')
-                    o.write(ft)
-                    o.write(filler+separator)
-                print >>o
-        return o.getvalue()
-    elif output_format == 'xml':
-        template = MarkupTemplate(XML_TEMPLATE)
-        return template.generate(data=data).render('xml')
-    elif output_format == 'stanza':
-        data = [i for i in data if i.has_key('fmt_epub')]
-        for x in data:
-            if isinstance(x['fmt_epub'], unicode):
-                x['fmt_epub'] = x['fmt_epub'].encode('utf-8')
-        template = MarkupTemplate(STANZA_TEMPLATE)
-        return template.generate(id="urn:calibre:main", data=data, subtitle=subtitle,
-                sep=os.sep, quote=quote, updated=db.last_modified()).render('xml')
+    for record in data:
+        text = [wrappers[i].wrap(unicode(record[field]).encode('utf-8')) for i, field in enumerate(fields)]
+        lines = max(map(len, text))
+        for l in range(lines):
+            for i, field in enumerate(text):
+                ft = text[i][l] if l < len(text[i]) else ''
+                filler = '%*s'%(widths[i]-len(ft)-1, '')
+                o.write(ft)
+                o.write(filler+separator)
+            print >>o
+    return o.getvalue()
+
+def list_option_parser(db=None):
+    fields = set(FIELDS)
+    if db is not None:
+        for f in db.custom_column_label_map:
+            fields.add('*'+f)
 
-def list_option_parser():
     parser = get_parser(_(
 '''\
 %prog list [options]
@@ -201,7 +125,12 @@ List the books available in the calibre database.
 '''
                             ))
     parser.add_option('-f', '--fields', default='title,authors',
-                      help=_('The fields to display when listing books in the database. Should be a comma separated list of fields.\nAvailable fields: %s\nDefault: %%default. The special field "all" can be used to select all fields. Only has effect in the text output format.')%','.join(FIELDS))
+                      help=_('The fields to display when listing books in the'
+                          ' database. Should be a comma separated list of'
+                          ' fields.\nAvailable fields: %s\nDefault: %%default. The'
+                          ' special field "all" can be used to select all fields.'
+                          ' Only has effect in the text output'
+                          ' format.')%','.join(sorted(fields)))
     parser.add_option('--sort-by', default=None,
                       help=_('The field by which to sort the results.\nAvailable fields: %s\nDefault: %%default')%','.join(FIELDS))
     parser.add_option('--ascending', default=False, action='store_true',
@@ -212,33 +141,40 @@ List the books available in the calibre database.
                       help=_('The maximum width of a single line in the output. Defaults to detecting screen size.'))
     parser.add_option('--separator', default=' ', help=_('The string used to separate fields. Default is a space.'))
     parser.add_option('--prefix', default=None, help=_('The prefix for all file paths. Default is the absolute path to the library folder.'))
-    of = ['text', 'xml', 'stanza']
-    parser.add_option('--output-format', choices=of, default='text',
-                      help=_('The format in which to output the data. Available choices: %s. Defaults is text.')%of)
     return parser
 
 
 def command_list(args, dbpath):
-    parser = list_option_parser()
+    pre = get_parser('')
+    pargs = [x for x in args if x in ('--with-library', '--library-path')
+        or not x.startswith('-')]
+    opts = pre.parse_args(sys.argv[:1] + pargs)[0]
+    db = get_db(dbpath, opts)
+    parser = list_option_parser(db=db)
     opts, args = parser.parse_args(sys.argv[:1] + args)
+    afields = set(FIELDS)
+    if db is not None:
+        for f in db.custom_column_label_map:
+            afields.add('*'+f)
     fields = [str(f.strip().lower()) for f in opts.fields.split(',')]
     if 'all' in fields:
-        fields = sorted(list(FIELDS))
-    if not set(fields).issubset(FIELDS):
+        fields = sorted(list(afields))
+    if not set(fields).issubset(afields):
         parser.print_help()
         print
-        print >>sys.stderr, _('Invalid fields. Available fields:'), ','.join(FIELDS)
+        prints(_('Invalid fields. Available fields:'),
+                ','.join(sorted(afields)), file=sys.stderr)
         return 1
 
-    db = get_db(dbpath, opts)
-    if not opts.sort_by in FIELDS and opts.sort_by is not None:
+    if not opts.sort_by in afields and opts.sort_by is not None:
         parser.print_help()
         print
-        print >>sys.stderr, _('Invalid sort field. Available fields:'), ','.join(FIELDS)
+        prints(_('Invalid sort field. Available fields:'), ','.join(afields),
+                file=sys.stderr)
         return 1
 
-    print do_list(db, fields, opts.sort_by, opts.ascending, opts.search, opts.line_width, opts.separator,
-            opts.prefix, opts.output_format)
+    print do_list(db, fields, afields, opts.sort_by, opts.ascending, opts.search, opts.line_width, opts.separator,
+            opts.prefix)
     return 0
 
 
@@ -342,12 +278,35 @@ the directory related options below.
                       help=_('Process directories recursively'))
     parser.add_option('-d', '--duplicates', action='store_true', default=False,
                       help=_('Add books to database even if they already exist. Comparison is done based on book titles.'))
+    parser.add_option('-e', '--empty', action='store_true', default=False,
+                    help=_('Add an empty book (a book with no formats)'))
+    parser.add_option('-t', '--title', default=None,
+            help=_('Set the title of the added empty book'))
+    parser.add_option('-a', '--authors', default=None,
+            help=_('Set the authors of the added empty book'))
+    parser.add_option('-i', '--isbn', default=None,
+            help=_('Set the ISBN of the added empty book'))
+
     return parser
 
+def do_add_empty(db, title, authors, isbn):
+    from calibre.ebooks.metadata import MetaInformation, string_to_authors
+    mi = MetaInformation(None)
+    if title is not None:
+        mi.title = title
+    if authors:
+        mi.authors = string_to_authors(authors)
+    if isbn:
+        mi.isbn = isbn
+    db.import_book(mi, [])
+    send_message()
 
 def command_add(args, dbpath):
     parser = add_option_parser()
     opts, args = parser.parse_args(sys.argv[:1] + args)
+    if opts.empty:
+        do_add_empty(get_db(dbpath, opts), opts.title, opts.authors, opts.isbn)
+        return 0
     if len(args) < 2:
         parser.print_help()
         print
@@ -579,8 +538,327 @@ def command_export(args, dbpath):
     do_export(get_db(dbpath, opts), ids, dir, opts)
     return 0
 
+def do_add_custom_column(db, label, name, datatype, is_multiple, display):
+    num = db.create_custom_column(label, name, datatype, is_multiple, display=display)
+    prints('Custom column created with id: %d'%num)
+
+def add_custom_column_option_parser():
+    from calibre.library.custom_columns import CustomColumns
+    parser = get_parser(_('''\
+%prog add_custom_column [options] label name datatype
+
+Create a custom column. label is the machine friendly name of the column. Should
+not contain spaces or colons. name is the human friendly name of the column.
+datatype is one of: {0}
+''').format(', '.join(CustomColumns.CUSTOM_DATA_TYPES)))
+
+    parser.add_option('--is-multiple', default=False, action='store_true',
+                      help=_('This column stores tag like data (i.e. '
+                          'multiple comma separated values). Only '
+                          'applies if datatype is text.'))
+    parser.add_option('--display', default='{}',
+            help=_('A dictionary of options to customize how '
+                'the data in this column will be interpreted.'))
+
+    return parser
+
+
+def command_add_custom_column(args, dbpath):
+    import json
+    parser = add_custom_column_option_parser()
+    opts, args = parser.parse_args(args)
+    if len(args) < 3:
+        parser.print_help()
+        print
+        print >>sys.stderr, _('You must specify label, name and datatype')
+        return 1
+    do_add_custom_column(get_db(dbpath, opts), args[0], args[1], args[2],
+            opts.is_multiple, json.loads(opts.display))
+    return 0
+
+def catalog_option_parser(args):
+    from calibre.customize.ui import available_catalog_formats, plugin_for_catalog_format
+    from calibre.utils.logging import Log
+
+    def add_plugin_parser_options(fmt, parser, log):
+
+        # Fetch the extension-specific CLI options from the plugin
+        plugin = plugin_for_catalog_format(fmt)
+        for option in plugin.cli_options:
+            if option.action:
+                parser.add_option(option.option,
+                                  default=option.default,
+                                  dest=option.dest,
+                                  action=option.action,
+                                  help=option.help)
+            else:
+                parser.add_option(option.option,
+                                  default=option.default,
+                                  dest=option.dest,
+                                  help=option.help)
+
+        return plugin
+
+    def print_help(parser, log):
+        help = parser.format_help().encode(preferred_encoding, 'replace')
+        log(help)
+
+    def validate_command_line(parser, args, log):
+        # calibredb catalog path/to/destination.[epub|csv|xml|...] [options]
+
+        # Validate form
+        if not len(args) or args[0].startswith('-'):
+            print_help(parser, log)
+            log.error("\n\nYou must specify a catalog output file of the form 'path/to/destination.extension'\n"
+            "To review options for an output format, type 'calibredb catalog <.extension> --help'\n"
+            "For example, 'calibredb catalog .xml --help'\n")
+            raise SystemExit(1)
+
+        # Validate plugin exists for specified output format
+        output = os.path.abspath(args[0])
+        file_extension = output[output.rfind('.') + 1:].lower()
+
+        if not file_extension in available_catalog_formats():
+            print_help(parser, log)
+            log.error("No catalog plugin available for extension '%s'.\n" % file_extension +
+                      "Catalog plugins available for %s\n" % ', '.join(available_catalog_formats()) )
+            raise SystemExit(1)
+
+        return output, file_extension
+
+    # Entry point
+    log = Log()
+    parser = get_parser(_(
+    '''
+    %prog catalog /path/to/destination.(csv|epub|mobi|xml ...) [options]
+
+    Export a catalog in format specified by path/to/destination extension.
+    Options control how entries are displayed in the generated catalog ouput.
+    '''))
+
+    # Confirm that a plugin handler exists for specified output file extension
+    # Will raise SystemExit(1) if no plugin matching file_extension
+    output, fmt = validate_command_line(parser, args, log)
+
+    # Add options common to all catalog plugins
+    parser.add_option('-i', '--ids', default=None, dest='ids',
+                      help=_("Comma-separated list of database IDs to catalog.\n"
+                      "If declared, --search is ignored.\n"
+                             "Default: all"))
+    parser.add_option('-s', '--search', default=None, dest='search_text',
+                      help=_("Filter the results by the search query. "
+                          "For the format of the search query, please see "
+                          "the search-related documentation in the User Manual.\n"
+                      "Default: no filtering"))
+    parser.add_option('-v','--verbose', default=False, action='store_true',
+                      dest='verbose',
+                      help=_('Show detailed output information. Useful for debugging'))
+
+    # Add options specific to fmt plugin
+    plugin = add_plugin_parser_options(fmt, parser, log)
+
+    return parser, plugin, log
+
+def command_catalog(args, dbpath):
+    parser, plugin, log = catalog_option_parser(args)
+    opts, args = parser.parse_args(sys.argv[1:])
+    if len(args) < 2:
+        parser.print_help()
+        print
+        print >>sys.stderr, _('Error: You must specify a catalog output file')
+        return 1
+    if opts.ids:
+        opts.ids = [int(id) for id in opts.ids.split(',')]
+
+    # No support for connected device in CLI environment
+    # Parallel initialization in calibre.gui2.tools:generate_catalog()
+    opts.connected_device = { 'storage':None,'serial':None,'save_template':None,'name':None}
+
+    with plugin:
+        plugin.run(args[1], opts, get_db(dbpath, opts))
+    return 0
+
+# end of GR additions
+
+def parse_series_string(db, label, value):
+    val = unicode(value).strip()
+    s_index = None
+    pat = re.compile(r'\[([.0-9]+)\]')
+    match = pat.search(val)
+    if match is not None:
+        val = pat.sub('', val).strip()
+        s_index = float(match.group(1))
+    elif val:
+        if tweaks['series_index_auto_increment'] == 'next':
+            s_index = db.get_next_cc_series_num_for(val, label=label)
+        else:
+            s_index = 1.0
+    return val, s_index
+
+def do_set_custom(db, col, id_, val, append):
+    if db.custom_column_label_map[col]['datatype'] == 'series':
+        val, s_index = parse_series_string(db, col, val)
+        db.set_custom(id_, val, extra=s_index, label=col, append=append)
+        prints('Data set to: %r[%4.2f]'%
+               (db.get_custom(id_, label=col, index_is_id=True),
+                db.get_custom_extra(id_, label=col, index_is_id=True)))
+    else:
+        db.set_custom(id_, val, label=col, append=append)
+        prints('Data set to: %r'%db.get_custom(id_, label=col, index_is_id=True))
+
+def set_custom_option_parser():
+    parser = get_parser(_(
+    '''
+    %prog set_custom [options] column id value
+
+    Set the value of a custom column for the book identified by id.
+    You can get a list of ids using the list command.
+    You can get a list of custom column names using the custom_columns
+    command.
+    '''))
+
+    parser.add_option('-a', '--append', default=False, action='store_true',
+            help=_('If the column stores multiple values, append the specified '
+                'values to the existing ones, instead of replacing them.'))
+    return parser
+
+
+def command_set_custom(args, dbpath):
+    parser = set_custom_option_parser()
+    opts, args = parser.parse_args(args)
+    if len(args) < 3:
+        parser.print_help()
+        print
+        print >>sys.stderr, _('Error: You must specify a field name, id and value')
+        return 1
+    do_set_custom(get_db(dbpath, opts), args[0], int(args[1]), args[2],
+            opts.append)
+    return 0
+
+def do_custom_columns(db, details):
+    from pprint import pformat
+    cols = db.custom_column_label_map
+    for col, data in cols.items():
+        if details:
+            prints(col)
+            print
+            prints(pformat(data))
+            print '\n'
+        else:
+            prints(col, '(%d)'%data['num'])
+
+def custom_columns_option_parser():
+    parser = get_parser(_(
+    '''
+    %prog custom_columns [options]
+
+    List available custom columns. Shows column labels and ids.
+    '''))
+    parser.add_option('-d', '--details', default=False, action='store_true',
+            help=_('Show details for each column.'))
+    return parser
+
+
+def command_custom_columns(args, dbpath):
+    parser = custom_columns_option_parser()
+    opts, args = parser.parse_args(args)
+    do_custom_columns(get_db(dbpath, opts), opts.details)
+    return 0
+
+def do_remove_custom_column(db, label, force):
+    if not force:
+        q = raw_input(_('You will lose all data in the column: %r.'
+            ' Are you sure (y/n)? ')%label)
+        if q.lower().strip() != _('y'):
+            return
+    db.delete_custom_column(label=label)
+    prints('Column %r removed.'%label)
+
+def remove_custom_column_option_parser():
+    parser = get_parser(_(
+    '''
+    %prog remove_custom_column [options] label
+
+    Remove the custom column identified by label. You can see available
+    columns with the custom_columns command.
+    '''))
+    parser.add_option('-f', '--force', default=False, action='store_true',
+            help=_('Do not ask for confirmation'))
+    return parser
+
+
+def command_remove_custom_column(args, dbpath):
+    parser = remove_custom_column_option_parser()
+    opts, args = parser.parse_args(args)
+    if len(args) < 1:
+        parser.print_help()
+        print
+        prints(_('Error: You must specify a column label'), file=sys.stderr)
+        return 1
+
+    do_remove_custom_column(get_db(dbpath, opts), args[0], opts.force)
+    return 0
+
+def saved_searches_option_parser():
+    parser = get_parser(_(
+    '''
+    %prog saved_searches [options] list
+    %prog saved_searches add name search
+    %prog saved_searches remove name
+
+    Manage the saved searches stored in this database.
+    If you try to add a query with a name that already exists, it will be
+    replaced.
+    '''))
+    return parser
+
+def command_saved_searches(args, dbpath):
+    parser = saved_searches_option_parser()
+    opts, args = parser.parse_args(args)
+    if len(args) < 1:
+        parser.print_help()
+        print
+        prints(_('Error: You must specify an action (add|remove|list)'), file=sys.stderr)
+        return 1
+    from calibre.utils.search_query_parser import saved_searches
+    db = get_db(dbpath, opts)
+    db
+    ss = saved_searches()
+    if args[0] == 'list':
+        for name in ss.names():
+            prints(_('Name:'), name)
+            prints(_('Search string:'), ss.lookup(name))
+            print
+    elif args[0] == 'add':
+        if len(args) < 3:
+            parser.print_help()
+            print
+            prints(_('Error: You must specify a name and a search string'), file=sys.stderr)
+            return 1
+        ss.add(args[1], args[2])
+        prints(args[1], _('added'))
+    elif args[0] == 'remove':
+        if len(args) < 2:
+            parser.print_help()
+            print
+            prints(_('Error: You must specify a name'), file=sys.stderr)
+            return 1
+        ss.delete(args[1])
+        prints(args[1], _('removed'))
+    else:
+        parser.print_help()
+        print
+        prints(_('Error: Action %s not recognized, must be one '
+            'of: (add|remove|list)') % args[1], file=sys.stderr)
+        return 1
+
+    return 0
+
+
 COMMANDS = ('list', 'add', 'remove', 'add_format', 'remove_format',
-                'show_metadata', 'set_metadata', 'export')
+            'show_metadata', 'set_metadata', 'export', 'catalog',
+            'saved_searches', 'add_custom_column', 'custom_columns',
+            'remove_custom_column', 'set_custom')
 
 
 def option_parser():
