@@ -3,29 +3,17 @@
 
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
-import sys, os, inspect, re, textwrap
+import sys, os, re, textwrap
 
 sys.path.insert(0, os.path.abspath('../../'))
 sys.extensions_location = '../plugins'
 sys.resources_location  = '../../../resources'
 
-from sphinx.builder import StandaloneHTMLBuilder
-from qthelp import QtHelpBuilder
-from epub import EPUBHelpBuilder
-from sphinx.util import rpartition
 from sphinx.util.console import bold
-from sphinx.ext.autodoc import prepare_docstring
-from docutils.statemachine import ViewList
-from docutils import nodes
 
 sys.path.append(os.path.abspath('../../../'))
 from calibre.linux import entry_points
-
-class CustomBuilder(StandaloneHTMLBuilder):
-    name = 'custom'
-
-class CustomQtBuild(QtHelpBuilder):
-    name = 'customqt'
+from epub import EPUBHelpBuilder
 
 def substitute(app, doctree):
     pass
@@ -40,6 +28,9 @@ Command Line Interface
 
 .. image:: ../images/cli.png
 
+On OS X you have to go to Preferences->Advanced and click install command line
+tools to make the command line tools available.  On other platforms, just start
+a terminal and type the command.
 
 Documented Commands
 --------------------
@@ -94,7 +85,12 @@ def generate_calibredb_help(preamble, info):
 
     lines, toc = [], []
     for cmd in COMMANDS:
-        parser = getattr(cli, cmd+'_option_parser')()
+        args = []
+        if cmd == 'catalog':
+            args = [['doc.xml', '-h']]
+        parser = getattr(cli, cmd+'_option_parser')(*args)
+        if cmd == 'catalog':
+            parser = parser[0]
         toc.append('  * :ref:`calibredb-%s`'%cmd)
         lines += ['.. _calibredb-'+cmd+':', '']
         lines += [cmd, '~'*20, '']
@@ -124,7 +120,6 @@ def generate_ebook_convert_help(preamble, info):
     input and the output formats, the various combinations are listed below:
 
     ''')
-    sections = []
     toc = {}
     sec_templ = textwrap.dedent('''\
         .. include:: ../global.rst
@@ -138,7 +133,6 @@ def generate_ebook_convert_help(preamble, info):
 
     ''')
     for i, ip in enumerate(input_format_plugins()):
-        path = os.path.join('cli', 'ebook-convert-%d.rst'%i)
         sraw = sec_templ.format(ip.name)
         toc[ip.name] = 'ebook-convert-%d'%i
         for op in output_format_plugins():
@@ -165,7 +159,15 @@ def generate_ebook_convert_help(preamble, info):
 def update_cli_doc(path, raw, info):
     if isinstance(raw, unicode):
         raw = raw.encode('utf-8')
-    if not os.path.exists(path) or open(path, 'rb').read() != raw:
+    old_raw = open(path, 'rb').read() if os.path.exists(path) else ''
+    if not os.path.exists(path) or old_raw != raw:
+        import difflib
+        print path, 'has changed'
+        if old_raw:
+            lines = difflib.unified_diff(old_raw.splitlines(), raw.splitlines(),
+                    path, path)
+            for line in lines:
+                print line
         info('creating '+os.path.splitext(os.path.basename(path))[0])
         open(path, 'wb').write(raw)
 
@@ -181,7 +183,8 @@ def render_options(cmd, groups, options_header=True, add_program=True):
             lines.append('')
         if desc:
             lines.extend([desc, ''])
-        for opt in options:
+        for opt in sorted(options, cmp=lambda x, y:cmp(x.get_opt_string(),
+                y.get_opt_string())):
             help = opt.help if opt.help else ''
             help = help.replace('\n', ' ').replace('*', '\\*').replace('%default', str(opt.default))
             opt = opt.get_opt_string() + ((', '+', '.join(opt._short_opts)) if opt._short_opts else '')
@@ -237,65 +240,13 @@ def cli_docs(app):
             raw += '\n'+'\n'.join(lines)
             update_cli_doc(os.path.join('cli', cmd+'.rst'), raw, info)
 
-def auto_member(dirname, arguments, options, content, lineno,
-                    content_offset, block_text, state, state_machine):
-    name = arguments[0]
-    env = state.document.settings.env
-
-    mod_cls, obj = rpartition(name, '.')
-    if not mod_cls and hasattr(env, 'autodoc_current_class'):
-        mod_cls = env.autodoc_current_class
-    if not mod_cls:
-        mod_cls = env.currclass
-    mod, cls = rpartition(mod_cls, '.')
-    if not mod and hasattr(env, 'autodoc_current_module'):
-        mod = env.autodoc_current_module
-    if not mod:
-        mod = env.currmodule
-
-    module = __import__(mod, None, None, ['foo'])
-    cls = getattr(module, cls)
-    lines = inspect.getsourcelines(cls)[0]
-
-    comment_lines = []
-    for i, line in enumerate(lines):
-        if re.search(r'%s\s*=\s*\S+'%obj, line) and not line.strip().startswith('#:'):
-            for j in range(i-1, 0, -1):
-                raw = lines[j].strip()
-                if not raw.startswith('#:'):
-                    break
-                comment_lines.append(raw[2:])
-            break
-    comment_lines.reverse()
-    docstring = '\n'.join(comment_lines)
-
-    if module is not None and docstring is not None:
-        docstring = docstring.decode('utf-8')
-
-    result = ViewList()
-    result.append('.. attribute:: %s.%s'%(cls.__name__, obj), '<autodoc>')
-    result.append('', '<autodoc>')
-
-    docstring = prepare_docstring(docstring)
-    for i, line in enumerate(docstring):
-        result.append('    ' + line, '<docstring of %s>' % name, i)
-
-    result.append('', '')
-    result.append('    **Default**: ``%s``'%repr(getattr(cls, obj, None)), '<default memeber value>')
-    result.append('', '')
-    node = nodes.paragraph()
-    state.nested_parse(result, content_offset, node)
-
-    return list(node)
-
 def setup(app):
-    app.add_config_value('epub_titlepage', None, False)
-    app.add_config_value('epub_author', '', False)
-    app.add_config_value('epub_logo', None, False)
-    app.add_builder(CustomBuilder)
-    app.add_builder(CustomQtBuild)
+    app.add_config_value('epub_cover', None, False)
     app.add_builder(EPUBHelpBuilder)
-    app.add_directive('automember', auto_member, 1, (1, 0, 1))
     app.connect('doctree-read', substitute)
     app.connect('builder-inited', cli_docs)
+    app.connect('build-finished', finished)
+
+def finished(app, exception):
+    pass
 
