@@ -11,29 +11,35 @@ import os, time, traceback, re, urlparse, sys
 from collections import defaultdict
 from functools import partial
 from contextlib import nested, closing
-from datetime import datetime
 
 
 from calibre import browser, __appname__, iswindows, \
-                    strftime, __version__, preferred_encoding
+                    strftime, preferred_encoding
 from calibre.ebooks.BeautifulSoup import BeautifulSoup, NavigableString, CData, Tag
 from calibre.ebooks.metadata.opf2 import OPFCreator
 from calibre import entity_to_unicode
 from calibre.web import Recipe
-from calibre.ebooks import render_html
 from calibre.ebooks.metadata.toc import TOC
 from calibre.ebooks.metadata import MetaInformation
 from calibre.web.feeds import feed_from_xml, templates, feeds_from_index, Feed
 from calibre.web.fetch.simple import option_parser as web2disk_option_parser
 from calibre.web.fetch.simple import RecursiveFetcher
 from calibre.utils.threadpool import WorkRequest, ThreadPool, NoResultsPending
-from calibre.ptempfile import PersistentTemporaryFile, \
-                              PersistentTemporaryDirectory
+from calibre.ptempfile import PersistentTemporaryFile
+from calibre.utils.date import now as nowf
 
+class LoginFailed(ValueError):
+    pass
+
+class DownloadDenied(ValueError):
+    pass
 
 class BasicNewsRecipe(Recipe):
     '''
-    Abstract base class that contains logic needed in all feed fetchers.
+    Base class that contains logic needed in all recipes. By overriding
+    progressively more of the functionality in this class, you can make
+    progressively more customized/powerful recipes. For a tutorial introduction
+    to creating recipes, see :doc:`news`.
     '''
 
     #: The title to use for the ebook
@@ -46,6 +52,9 @@ class BasicNewsRecipe(Recipe):
     #: The author of this recipe
     __author__             = __appname__
 
+    #: Minimum calibre version needed to use this recipe
+    requires_version = (0, 6, 0)
+
     #: The language that the news is in. Must be an ISO-639 code either
     #: two or three characters long
     language               = 'und'
@@ -108,7 +117,9 @@ class BasicNewsRecipe(Recipe):
 
     #: Specify an override encoding for sites that have an incorrect
     #: charset specification. The most common being specifying ``latin1`` and
-    #: using ``cp1252``. If None, try to detect the encoding.
+    #: using ``cp1252``. If None, try to detect the encoding. If it is a
+    #: callable, the callable is called with two arguments: The recipe object
+    #: and the source to be decoded. It must return the decoded source.
     encoding               = None
 
     #: Normally we try to guess if a feed has full articles embedded in it
@@ -118,10 +129,13 @@ class BasicNewsRecipe(Recipe):
     #: embedded content.
     use_embedded_content   = None
 
-    #: Set to True and implement :method:`get_obfuscated_article` to handle
+    #: Set to True and implement :meth:`get_obfuscated_article` to handle
     #: websites that try to make it difficult to scrape content.
     articles_are_obfuscated = False
 
+    #: Reverse the order of articles in each feed
+    reverse_article_order = False
+
     #: Specify any extra :term:`CSS` that should be addded to downloaded :term:`HTML` files
     #: It will be inserted into `<style>` tags, just before the closing
     #: `</head>` tag thereby overriding all :term:`CSS` except that which is
@@ -135,11 +149,12 @@ class BasicNewsRecipe(Recipe):
     #: If True empty feeds are removed from the output.
     #: This option has no effect if parse_index is overriden in
     #: the sub class. It is meant only for recipes that return a list
-    #: of feeds using :member:`feeds` or :method:`get_feeds`.
+    #: of feeds using `feeds` or :meth:`get_feeds`.
     remove_empty_feeds = False
 
     #: List of regular expressions that determines which links to follow
-    #: If empty, it is ignored. For example::
+    #: If empty, it is ignored. Used only if is_link_wanted is
+    #: not implemented. For example::
     #:
     #:     match_regexps = [r'page=[0-9]+']
     #:
@@ -150,7 +165,8 @@ class BasicNewsRecipe(Recipe):
     match_regexps         = []
 
     #: List of regular expressions that determines which links to ignore
-    #: If empty it is ignored. For example::
+    #: If empty it is ignored. Used only if is_link_wanted is not
+    #: implemented. For example::
     #:
     #:     filter_regexps = [r'ads\.doubleclick\.net']
     #:
@@ -241,34 +257,65 @@ class BasicNewsRecipe(Recipe):
     #: will remove everythong from `<!--Article ends here-->` to `</body>`.
     preprocess_regexps    = []
 
-    #: The CSS that is used to styles the templates, i.e., the navigation bars and
+    #: The CSS that is used to style the templates, i.e., the navigation bars and
     #: the Tables of Contents. Rather than overriding this variable, you should
-    #: use :member:`extra_css` in your recipe to customize look and feel.
+    #: use `extra_css` in your recipe to customize look and feel.
     template_css = u'''
             .article_date {
                 color: gray; font-family: monospace;
             }
 
             .article_description {
-                font-family: sans; text-indent: 0pt;
+                text-indent: 0pt;
             }
 
             a.article {
-                font-weight: bold;
+                font-weight: bold; text-align:left;
             }
 
             a.feed {
                 font-weight: bold;
             }
 
-            .navbar {
+            .calibre_navbar {
                 font-family:monospace;
             }
-'''
+    '''
+
+    #: By default, calibre will use a default image for the masthead (Kindle only).
+    #: Override this in your recipe to provide a url to use as a masthead.
+    masthead_url = None
+
+    #: By default, the cover image returned by get_cover_url() will be used as
+    #: the cover for the periodical.  Overriding this in your recipe instructs
+    #: calibre to render the downloaded cover into a frame whose width and height
+    #: are expressed as a percentage of the downloaded cover.
+    #: cover_margins = (10,15,'white') pads the cover with a white margin
+    #: 10px on the left and right, 15px on the top and bottom.
+    #: Colors name defined at http://www.imagemagick.org/script/color.php
+    cover_margins = (0,0,'white')
+
+    #: Set to a non empty string to disable this recipe
+    #: The string will be used as the disabled message
+    recipe_disabled = None
 
 
     # See the built-in profiles for examples of these settings.
 
+    def short_title(self):
+        return self.title
+
+    def is_link_wanted(self, url, tag):
+        '''
+        Return True if the link should be followed or False otherwise. By
+        default, raises NotImplementedError which causes the downloader to
+        ignore it.
+
+        :param url: The URL to be followed
+        :param tag: The Tag from which the URL was derived
+        '''
+        raise NotImplementedError
+
     def get_cover_url(self):
         '''
         Return a :term:`URL` to the cover image for this issue or `None`.
@@ -279,6 +326,17 @@ class BasicNewsRecipe(Recipe):
         '''
         return getattr(self, 'cover_url', None)
 
+    def get_masthead_url(self):
+        '''
+        Return a :term:`URL` to the masthead image for this issue or `None`.
+        By default it returns the value of the member `self.masthead_url` which
+        is normally `None`. If you want your recipe to download a masthead for the e-book
+        override this method in your subclass, or set the member variable `self.masthead_url`
+        before this method is called.
+        Masthead images are used in Kindle MOBI files.
+        '''
+        return getattr(self, 'masthead_url', None)
+
     def get_feeds(self):
         '''
         Return a list of :term:`RSS` feeds to fetch for this profile. Each element of the list
@@ -344,15 +402,39 @@ class BasicNewsRecipe(Recipe):
         Override in a subclass to customize extraction of the :term:`URL` that points
         to the content for each article. Return the
         article URL. It is called with `article`, an object representing a parsed article
-        from a feed. See `feedsparser <http://www.feedparser.org/docs/>`_.
-        By default it returns `article.link <http://www.feedparser.org/docs/reference-entry-link.html>`_.
+        from a feed. See `feedparser <http://www.feedparser.org/docs/>`_.
+        By default it looks for the original link (for feeds syndicated via a
+        service like feedburner or pheedo) and if found,
+        returns that or else returns
+        `article.link <http://www.feedparser.org/docs/reference-entry-link.html>`_.
         '''
+        for key in article.keys():
+            if key.endswith('_origlink'):
+                url = article[key]
+                if url and url.startswith('http://'):
+                    return url
         return article.get('link',  None)
 
+    def skip_ad_pages(self, soup):
+        '''
+        This method is called with the source of each downloaded :term:`HTML` file, before
+        any of the cleanup attributes like remove_tags, keep_only_tags are
+        applied. Note that preprocess_regexps will have already been applied.
+        It is meant to allow the recipe to skip ad pages. If the soup represents
+        an ad page, return the HTML of the real page. Otherwise return
+        None.
+
+        `soup`: A `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/documentation.html>`_
+        instance containing the downloaded :term:`HTML`.
+        '''
+        return None
+
+
     def preprocess_html(self, soup):
         '''
         This method is called with the source of each downloaded :term:`HTML` file, before
-        it is parsed for links and images.
+        it is parsed for links and images. It is called after the cleanup as
+        specified by remove_tags etc.
         It can be used to do arbitrarily powerful pre-processing on the :term:`HTML`.
         It should return `soup` after processing it.
 
@@ -390,9 +472,9 @@ class BasicNewsRecipe(Recipe):
         `url_or_raw`: Either a URL or the downloaded index page as a string
         '''
         if re.match(r'\w+://', url_or_raw):
-            f = self.browser.open(url_or_raw)
-            _raw = f.read()
-            f.close()
+            open_func = getattr(self.browser, 'open_novisit', self.browser.open)
+            with closing(open_func(url_or_raw)) as f:
+                _raw = f.read()
             if not _raw:
                 raise RuntimeError('Could not fetch index from %s'%url_or_raw)
         else:
@@ -400,9 +482,14 @@ class BasicNewsRecipe(Recipe):
         if raw:
             return _raw
         if not isinstance(_raw, unicode) and self.encoding:
-            _raw = _raw.decode(self.encoding, 'replace')
+            if callable(self.encoding):
+                _raw = self.encoding(_raw)
+            else:
+                _raw = _raw.decode(self.encoding, 'replace')
         massage = list(BeautifulSoup.MARKUP_MASSAGE)
-        massage.append((re.compile(r'&(\S+?);'), lambda match: entity_to_unicode(match, encoding=self.encoding)))
+        enc = 'cp1252' if callable(self.encoding) or self.encoding is None else self.encoding
+        massage.append((re.compile(r'&(\S+?);'), lambda match:
+            entity_to_unicode(match, encoding=enc)))
         return BeautifulSoup(_raw, markupMassage=massage)
 
 
@@ -447,17 +534,29 @@ class BasicNewsRecipe(Recipe):
 
     def get_obfuscated_article(self, url):
         '''
-        If you set :member:`articles_are_obfuscated` this method is called with
+        If you set `articles_are_obfuscated` this method is called with
         every article URL. It should return the path to a file on the filesystem
         that contains the article HTML. That file is processed by the recursive
         HTML fetching engine, so it can contain links to pages/images on the web.
 
         This method is typically useful for sites that try to make it difficult to
-        access article content automatically. See for example the
-        :module:`calibre.web.recipes.iht` recipe.
+        access article content automatically.
         '''
         raise NotImplementedError
 
+    def populate_article_metadata(self, article, soup, first):
+        '''
+        Called when each HTML page belonging to article is downloaded.
+        Intended to be used to get article metadata like author/summary/etc.
+        from the parsed HTML (soup).
+        :param article: A object of class :class:`calibre.web.feeds.Article`.
+        If you change the summary, remember to also change the
+        text_summary
+        :param soup: Parsed HTML belonging to this article
+        :param first: True iff the parsed HTML is the first page of the article.
+        '''
+        pass
+
     def postprocess_book(self, oeb, opts, log):
         '''
         Run any needed post processing on the parsed downloaded e-book.
@@ -485,7 +584,10 @@ class BasicNewsRecipe(Recipe):
         self.username = options.username
         self.password = options.password
         self.lrf = options.lrf
-        self.include_navbars = not options.no_inline_navbars
+        self.output_profile = options.output_profile
+        self.touchscreen = getattr(self.output_profile, 'touchscreen', False)
+        if self.touchscreen:
+            self.template_css += self.output_profile.touchscreen_news_css
 
         self.output_dir = os.path.abspath(self.output_dir)
         if options.test:
@@ -501,8 +603,10 @@ class BasicNewsRecipe(Recipe):
             if isinstance(self.feeds, basestring):
                 self.feeds = [self.feeds]
 
-        if self.needs_subscription and (self.username is None or self.password is None):
-            raise ValueError('The %s recipe needs a username and password.'%self.title)
+        if self.needs_subscription and (\
+                self.username is None or self.password is None or \
+                (not self.username and not self.password)):
+            raise ValueError(_('The "%s" recipe needs a username and password.')%self.title)
 
         self.browser = self.get_browser()
         self.image_map, self.image_counter = {}, 1
@@ -513,8 +617,6 @@ class BasicNewsRecipe(Recipe):
             '--max-recursions', str(self.recursions),
             '--delay', str(self.delay),
             ]
-        if self.encoding is not None:
-            web2disk_cmdline.extend(['--encoding', self.encoding])
 
         if self.verbose:
             web2disk_cmdline.append('--verbose')
@@ -530,14 +632,17 @@ class BasicNewsRecipe(Recipe):
 
         self.web2disk_options = web2disk_option_parser().parse_args(web2disk_cmdline)[0]
         for extra in ('keep_only_tags', 'remove_tags', 'preprocess_regexps',
-                      'preprocess_html', 'remove_tags_after', 'remove_tags_before'):
+                      'skip_ad_pages', 'preprocess_html', 'remove_tags_after',
+                      'remove_tags_before', 'is_link_wanted'):
             setattr(self.web2disk_options, extra, getattr(self, extra))
         self.web2disk_options.postprocess_html = self._postprocess_html
+        self.web2disk_options.encoding = self.encoding
 
         if self.delay > 0:
             self.simultaneous_downloads = 1
 
-        self.navbar = templates.NavBarTemplate()
+        self.navbar = templates.TouchscreenNavBarTemplate() if self.touchscreen else \
+                      templates.NavBarTemplate()
         self.failed_downloads = []
         self.partial_failures = []
 
@@ -556,7 +661,7 @@ class BasicNewsRecipe(Recipe):
         if first_fetch and job_info:
             url, f, a, feed_len = job_info
             body = soup.find('body')
-            if body is not None and self.include_navbars:
+            if body is not None:
                 templ = self.navbar.generate(False, f, a, feed_len,
                                              not self.has_single_feed,
                                              url, __appname__,
@@ -567,21 +672,36 @@ class BasicNewsRecipe(Recipe):
         if self.remove_javascript:
             for script in list(soup.findAll('script')):
                 script.extract()
+            for o in soup.findAll(onload=True):
+                del o['onload']
+
         for script in list(soup.findAll('noscript')):
-                script.extract()
+            script.extract()
         for attr in self.remove_attributes:
             for x in soup.findAll(attrs={attr:True}):
                 del x[attr]
-        return self.postprocess_html(soup, first_fetch)
+        for base in list(soup.findAll(['base', 'iframe'])):
+            base.extract()
+
+        ans = self.postprocess_html(soup, first_fetch)
+        if job_info:
+            url, f, a, feed_len = job_info
+            try:
+                article = self.feed_objects[f].articles[a]
+            except:
+                self.log.exception('Failed to get article object for postprocessing')
+                pass
+            else:
+                self.populate_article_metadata(article, ans, first_fetch)
+        return ans
 
 
     def download(self):
         '''
         Download and pre-process all articles from the feeds in this recipe.
-        This method should be called only one on a particular Recipe instance.
+        This method should be called only once on a particular Recipe instance.
         Calling it more than once will lead to undefined behavior.
-        @return: Path to index.html
-        @rtype: string
+        :return: Path to index.html
         '''
         try:
             res = self.build_index()
@@ -608,7 +728,10 @@ class BasicNewsRecipe(Recipe):
     def feeds2index(self, feeds):
         templ = templates.IndexTemplate()
         css = self.template_css + '\n\n' +(self.extra_css if self.extra_css else '')
-        return templ.generate(self.title, self.timefmt, feeds,
+        timefmt = self.timefmt
+        if self.touchscreen:
+            templ = templates.TouchscreenIndexTemplate()
+        return templ.generate(self.title, "mastheadImage.jpg", timefmt, feeds,
                               extra_css=css).render(doctype='xhtml')
 
     @classmethod
@@ -633,7 +756,8 @@ class BasicNewsRecipe(Recipe):
 
 
 
-    def feed2index(self, feed):
+    def feed2index(self, f, feeds):
+        feed = feeds[f]
         if feed.image_url is not None: # Download feed image
             imgdir = os.path.join(self.output_dir, 'images')
             if not os.path.isdir(imgdir):
@@ -661,7 +785,11 @@ class BasicNewsRecipe(Recipe):
 
         templ = templates.FeedTemplate()
         css = self.template_css + '\n\n' +(self.extra_css if self.extra_css else '')
-        return templ.generate(feed, self.description_limiter,
+
+        if self.touchscreen:
+            templ = templates.TouchscreenFeedTemplate()
+
+        return templ.generate(f, feeds, self.description_limiter,
                               extra_css=css).render(doctype='xhtml')
 
 
@@ -702,7 +830,8 @@ class BasicNewsRecipe(Recipe):
         self.report_progress(0, _('Fetching feeds...'))
         try:
             feeds = feeds_from_index(self.parse_index(), oldest_article=self.oldest_article,
-                                     max_articles_per_feed=self.max_articles_per_feed)
+                                     max_articles_per_feed=self.max_articles_per_feed,
+                                     log=self.log)
             self.report_progress(0, _('Got feeds from index page'))
         except NotImplementedError:
             feeds = self.parse_feeds()
@@ -711,6 +840,28 @@ class BasicNewsRecipe(Recipe):
 
         self.report_progress(0, _('Trying to download cover...'))
         self.download_cover()
+        self.report_progress(0, _('Generating masthead...'))
+        self.masthead_path = None
+
+        try:
+            murl = self.get_masthead_url()
+        except:
+            self.log.exception('Failed to get masthead url')
+            murl = None
+
+        if murl is not None:
+            # Try downloading the user-supplied masthead_url
+            # Failure sets self.masthead_path to None
+            self.download_masthead(murl)
+        if self.masthead_path is None:
+            self.log.info("Synthesizing mastheadImage")
+            self.masthead_path = os.path.join(self.output_dir, 'mastheadImage.jpg')
+            try:
+                self.default_masthead_image(self.masthead_path)
+            except:
+                self.log.exception('Failed to generate default masthead image')
+                self.masthead_path = None
+
         if self.test:
             feeds = feeds[:2]
         self.has_single_feed = len(feeds) == 1
@@ -725,6 +876,13 @@ class BasicNewsRecipe(Recipe):
             fi.write(html)
 
         self.jobs = []
+
+        if self.reverse_article_order:
+            for feed in feeds:
+                if hasattr(feed, 'reverse'):
+                    feed.reverse()
+
+        self.feed_objects = feeds
         for f, feed in enumerate(feeds):
             feed_dir = os.path.join(self.output_dir, 'feed_%d'%f)
             if not os.path.isdir(feed_dir):
@@ -774,7 +932,7 @@ class BasicNewsRecipe(Recipe):
         #feeds.restore_duplicates()
 
         for f, feed in enumerate(feeds):
-            html = self.feed2index(feed)
+            html = self.feed2index(f,feeds)
             feed_dir = os.path.join(self.output_dir, 'feed_%d'%f)
             with open(os.path.join(feed_dir, 'index.html'), 'wb') as fi:
                 fi.write(html)
@@ -783,7 +941,7 @@ class BasicNewsRecipe(Recipe):
 
         return index
 
-    def download_cover(self):
+    def _download_cover(self):
         self.cover_path = None
         try:
             cu = self.get_cover_url()
@@ -792,100 +950,181 @@ class BasicNewsRecipe(Recipe):
             self.log.error(_('Could not download cover: %s')%str(err))
             self.log.debug(traceback.format_exc())
         if cu is not None:
-            ext = cu.rpartition('.')[-1]
+            ext = cu.split('/')[-1].rpartition('.')[-1]
             if '?' in ext:
                 ext = ''
-            ext = ext.lower() if ext else 'jpg'
-            self.report_progress(1, _('Downloading cover from %s')%cu)
+            ext = ext.lower() if ext and '/' not in ext else 'jpg'
             cpath = os.path.join(self.output_dir, 'cover.'+ext)
-            with nested(open(cpath, 'wb'), closing(self.browser.open(cu))) as (cfile, r):
-                cfile.write(r.read())
+            if os.access(cu, os.R_OK):
+                with open(cpath, 'wb') as cfile:
+                    cfile.write(open(cu, 'rb').read())
+            else:
+                self.report_progress(1, _('Downloading cover from %s')%cu)
+                with nested(open(cpath, 'wb'), closing(self.browser.open(cu))) as (cfile, r):
+                    cfile.write(r.read())
+                if self.cover_margins[0] or self.cover_margins[1]:
+                    from calibre.utils.magick.draw import add_borders_to_image
+                    add_borders_to_image(cpath,
+                                         left=self.cover_margins[0],right=self.cover_margins[0],
+                                         top=self.cover_margins[1],bottom=self.cover_margins[1],
+                                         border_color=self.cover_margins[2])
+            if ext.lower() == 'pdf':
+                from calibre.ebooks.metadata.pdf import get_metadata
+                stream = open(cpath, 'rb')
+                mi = get_metadata(stream)
+                cpath = None
+                if mi.cover_data and mi.cover_data[1]:
+                    cpath = os.path.join(self.output_dir,
+                            'cover.'+mi.cover_data[0])
+                    with open(cpath, 'wb') as f:
+                        f.write(mi.cover_data[1])
             self.cover_path = cpath
 
+    def download_cover(self):
+        try:
+            self._download_cover()
+        except:
+            self.log.exception('Failed to download cover')
+            self.cover_path = None
+
+    def _download_masthead(self, mu):
+        ext = mu.rpartition('.')[-1]
+        if '?' in ext:
+            ext = ''
+        ext = ext.lower() if ext else 'jpg'
+        mpath = os.path.join(self.output_dir, 'masthead_source.'+ext)
+        outfile = os.path.join(self.output_dir, 'mastheadImage.jpg')
+        if os.access(mu, os.R_OK):
+            with open(mpath, 'wb') as mfile:
+                mfile.write(open(mu, 'rb').read())
+        else:
+            with nested(open(mpath, 'wb'), closing(self.browser.open(mu))) as (mfile, r):
+                mfile.write(r.read())
+            self.report_progress(1, _('Masthead image downloaded'))
+        self.prepare_masthead_image(mpath, outfile)
+        self.masthead_path = outfile
+        if os.path.exists(mpath):
+            os.remove(mpath)
+
+
+    def download_masthead(self, url):
+        try:
+            self._download_masthead(url)
+        except:
+            self.log.exception("Failed to download supplied masthead_url")
 
     def default_cover(self, cover_file):
         '''
         Create a generic cover for recipes that dont have a cover
         '''
-        from calibre.gui2 import is_ok_to_use_qt
-        if not is_ok_to_use_qt():
-            return False
-        img_data = open(I('library.png'), 'rb').read()
-        tdir = PersistentTemporaryDirectory('_default_cover')
-        img = os.path.join(tdir, 'logo.png')
-        with open(img, 'wb') as g:
-            g.write(img_data)
-        img = os.path.basename(img)
-        html= u'''\
-        <html>
-            <head>
-                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
-                <style type="text/css">
-                    body {
-                        background: white no-repeat fixed center center;
-                        text-align: center;
-                        vertical-align: center;
-                        overflow: hidden;
-                        font-size: 18px;
-                    }
-                    h1 { font-family: serif; }
-                    h2, h4 { font-family: monospace; }
-                </style>
-            </head>
-            <body>
-                <h1>%(title)s</h1>
-                <br/><br/>
-                <div style="position:relative">
-                    <div style="position: absolute; left: 0; top: 0; width:100%%; height:100%%; vertical-align:center">
-                        <img src="%(img)s" alt="calibre" style="opacity:0.3"/>
-                    </div>
-                    <div style="position: absolute; left: 0; top: 0; width:100%%; height:100%%; vertical-align:center">
-                        <h2>%(date)s</h2>
-                        <br/><br/><br/><br/><br/>
-                        <h3>%(author)s</h3>
-                        <br/><br/></br/><br/><br/><br/><br/><br/><br/>
-                        <h4>Produced by %(app)s</h4>
-                    </div>
-                </div>
-            </body>
-        </html>
-        '''%dict(title=self.title if isinstance(self.title, unicode) else self.title.decode(preferred_encoding, 'replace'),
-                 author=self.__author__ if isinstance(self.__author__, unicode) else self.__author__.decode(preferred_encoding, 'replace'),
-                 date=strftime(self.timefmt),
-                 app=__appname__ +' '+__version__,
-                 img=img)
-        hf = os.path.join(tdir, 'cover.htm')
-        with open(hf, 'wb') as f:
-            f.write(html.encode('utf-8'))
-        renderer = render_html(hf)
-        if renderer.tb is not None:
-            self.log.warning('Failed to render default cover')
-            self.log.debug(renderer.tb)
-        else:
-            cover_file.write(renderer.data)
+        try:
+            from calibre.utils.magick.draw import create_cover_page, TextLine
+            title = self.title if isinstance(self.title, unicode) else \
+                    self.title.decode(preferred_encoding, 'replace')
+            date = strftime(self.timefmt)
+            lines = [TextLine(title, 44), TextLine(date, 32)]
+            img_data = create_cover_page(lines, I('library.png'), output_format='jpg')
+            cover_file.write(img_data)
             cover_file.flush()
+        except:
+            self.log.exception('Failed to generate default cover')
+            return False
         return True
 
+    def get_masthead_title(self):
+        'Override in subclass to use something other than the recipe title'
+        return self.title
+
+    MI_WIDTH = 600
+    MI_HEIGHT = 60
+
+    def default_masthead_image(self, out_path):
+        from calibre.ebooks.conversion.config import load_defaults
+        from calibre.utils.fonts import fontconfig
+        font_path = default_font = P('fonts/liberation/LiberationSerif-Bold.ttf')
+        recs = load_defaults('mobi_output')
+        masthead_font_family = recs.get('masthead_font', 'Default')
+
+        if masthead_font_family != 'Default':
+            masthead_font = fontconfig.files_for_family(masthead_font_family)
+            # Assume 'normal' always in dict, else use default
+            # {'normal': (path_to_font, friendly name)}
+            if 'normal' in masthead_font:
+                font_path = masthead_font['normal'][0]
+
+        if not font_path or not os.access(font_path, os.R_OK):
+            font_path = default_font
+
+        try:
+            from PIL import Image, ImageDraw, ImageFont
+            Image, ImageDraw, ImageFont
+        except ImportError:
+            import Image, ImageDraw, ImageFont
+
+        img = Image.new('RGB', (self.MI_WIDTH, self.MI_HEIGHT), 'white')
+        draw = ImageDraw.Draw(img)
+        try:
+            font = ImageFont.truetype(font_path, 48)
+        except:
+            font = ImageFont.truetype(default_font, 48)
+        text = self.get_masthead_title().encode('utf-8')
+        width, height = draw.textsize(text, font=font)
+        left = max(int((self.MI_WIDTH - width)/2.), 0)
+        top = max(int((self.MI_HEIGHT - height)/2.), 0)
+        draw.text((left, top), text, fill=(0,0,0), font=font)
+        img.save(open(out_path, 'wb'), 'JPEG')
+
+    def prepare_masthead_image(self, path_to_image, out_path):
+        from calibre import fit_image
+        from calibre.utils.magick import Image, create_canvas
+
+        img = Image()
+        img.open(path_to_image)
+        width, height = img.size
+        scaled, nwidth, nheight = fit_image(width, height, self.MI_WIDTH, self.MI_HEIGHT)
+        img2 = create_canvas(width, height)
+        frame = create_canvas(self.MI_WIDTH, self.MI_HEIGHT)
+        img2.compose(img)
+        if scaled:
+            img2.size = (nwidth, nheight, 'LanczosFilter', 0.5)
+        left = int((self.MI_WIDTH - nwidth)/2.0)
+        top = int((self.MI_HEIGHT - nheight)/2.0)
+        frame.compose(img2, left, top)
+        frame.save(out_path)
 
     def create_opf(self, feeds, dir=None):
         if dir is None:
             dir = self.output_dir
-        mi = MetaInformation(self.title + strftime(self.timefmt), [__appname__])
+        title = self.short_title()
+        if self.output_profile.periodical_date_in_title:
+            title += strftime(self.timefmt)
+        mi = MetaInformation(title, [__appname__])
         mi.publisher = __appname__
         mi.author_sort = __appname__
         mi.publication_type = 'periodical:'+self.publication_type
-        mi.timestamp = datetime.now()
+        mi.timestamp = nowf()
         mi.comments = self.description
         if not isinstance(mi.comments, unicode):
             mi.comments = mi.comments.decode('utf-8', 'replace')
-        mi.pubdate = datetime.now()
+        mi.pubdate = nowf()
         opf_path = os.path.join(dir, 'index.opf')
         ncx_path = os.path.join(dir, 'index.ncx')
+
         opf = OPFCreator(dir, mi)
+        # Add mastheadImage entry to <guide> section
+        mp = getattr(self, 'masthead_path', None)
+        if mp is not None and os.access(mp, os.R_OK):
+            from calibre.ebooks.metadata.opf2 import Guide
+            ref = Guide.Reference(os.path.basename(self.masthead_path), os.getcwdu())
+            ref.type = 'masthead'
+            ref.title = 'Masthead Image'
+            opf.guide.append(ref)
 
         manifest = [os.path.join(dir, 'feed_%d'%i) for i in range(len(feeds))]
         manifest.append(os.path.join(dir, 'index.html'))
         manifest.append(os.path.join(dir, 'index.ncx'))
+
+        # Get cover
         cpath = getattr(self, 'cover_path', None)
         if cpath is None:
             pf = open(os.path.join(dir, 'cover.jpg'), 'wb')
@@ -894,10 +1133,18 @@ class BasicNewsRecipe(Recipe):
         if cpath is not None and os.access(cpath, os.R_OK):
             opf.cover = cpath
             manifest.append(cpath)
+
+        # Get masthead
+        mpath = getattr(self, 'masthead_path', None)
+        if mpath is not None and os.access(mpath, os.R_OK):
+            manifest.append(mpath)
+
         opf.create_manifest_from_files_in(manifest)
         for mani in opf.manifest:
             if mani.path.endswith('.ncx'):
                 mani.id = 'ncx'
+            if mani.path.endswith('mastheadImage.jpg'):
+                mani.id = 'masthead-image'
 
         entries = ['index.html']
         toc = TOC(base_path=dir)
@@ -938,15 +1185,16 @@ class BasicNewsRecipe(Recipe):
                         body = soup.find('body')
                         if body is not None:
                             prefix = '/'.join('..'for i in range(2*len(re.findall(r'link\d+', last))))
-                            if self.include_navbars:
-                                templ = self.navbar.generate(True, num, j, len(f),
-                                                not self.has_single_feed,
-                                                a.orig_url, __appname__, prefix=prefix,
-                                                center=self.center_navbar)
-                                elem = BeautifulSoup(templ.render(doctype='xhtml').decode('utf-8')).find('div')
-                                body.insert(len(body.contents), elem)
+                            templ = self.navbar.generate(True, num, j, len(f),
+                                            not self.has_single_feed,
+                                            a.orig_url, __appname__, prefix=prefix,
+                                            center=self.center_navbar)
+                            elem = BeautifulSoup(templ.render(doctype='xhtml').decode('utf-8')).find('div')
+                            body.insert(len(body.contents), elem)
                             with open(last, 'wb') as fi:
                                 fi.write(unicode(soup).encode('utf-8'))
+        if len(feeds) == 0:
+            raise Exception('All feeds are empty, aborting.')
 
         if len(feeds) > 1:
             for i, f in enumerate(feeds):
@@ -976,7 +1224,6 @@ class BasicNewsRecipe(Recipe):
         with nested(open(opf_path, 'wb'), open(ncx_path, 'wb')) as (opf_file, ncx_file):
             opf.render(opf_file, ncx_file)
 
-
     def article_downloaded(self, request, result):
         index = os.path.join(os.path.dirname(result[0]), 'index.html')
         if index != result[0]:
@@ -1026,6 +1273,7 @@ class BasicNewsRecipe(Recipe):
                 with closing(self.browser.open(url)) as f:
                     parsed_feeds.append(feed_from_xml(f.read(),
                                           title=title,
+                                          log=self.log,
                                           oldest_article=self.oldest_article,
                                           max_articles_per_feed=self.max_articles_per_feed,
                                           get_article_url=self.get_article_url))
@@ -1033,7 +1281,7 @@ class BasicNewsRecipe(Recipe):
                 feed = Feed()
                 msg = 'Failed feed: %s'%(title if title else url)
                 feed.populate_from_preparsed_feed(msg, [])
-                feed.description = unicode(err)
+                feed.description = repr(err)
                 parsed_feeds.append(feed)
                 self.log.exception(msg)
 
@@ -1046,7 +1294,7 @@ class BasicNewsRecipe(Recipe):
         return parsed_feeds
 
     @classmethod
-    def tag_to_string(self, tag, use_alt=True):
+    def tag_to_string(self, tag, use_alt=True, normalize_whitespace=True):
         '''
         Convenience method to take a
         `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/documentation.html>`_
@@ -1073,7 +1321,10 @@ class BasicNewsRecipe(Recipe):
                     strings.append(res)
                 elif use_alt and item.has_key('alt'):
                     strings.append(item['alt'])
-        return u''.join(strings)
+        ans = u''.join(strings)
+        if normalize_whitespace:
+            ans = re.sub(r'\s+', ' ', ans)
+        return ans
 
     @classmethod
     def soup(cls, raw):
@@ -1088,7 +1339,7 @@ class BasicNewsRecipe(Recipe):
          '''
          If your recipe when converted to EPUB has problems with images when
          viewed in Adobe Digital Editions, call this method from within
-         :method:`postprocess_html`.
+         :meth:`postprocess_html`.
          '''
          for item in soup.findAll('img'):
              for attrib in ['height','width','border','align','style']:
@@ -1145,3 +1396,57 @@ class AutomaticNewsRecipe(BasicNewsRecipe):
         if self.use_embedded_content:
             self.web2disk_options.keep_only_tags = []
         return BasicNewsRecipe.fetch_embedded_article(self, article, dir, f, a, num_of_feeds)
+
+class CalibrePeriodical(BasicNewsRecipe):
+
+    #: Set this to the slug for the calibre periodical
+    calibre_periodicals_slug = None
+
+    LOG_IN = 'http://news.calibre-ebook.com/accounts/login'
+    needs_subscription = True
+    __author__ = 'calibre Periodicals'
+
+    def get_browser(self):
+        br = BasicNewsRecipe.get_browser(self)
+        br.open(self.LOG_IN)
+        br.select_form(name='login')
+        br['username'] = self.username
+        br['password'] = self.password
+        raw = br.submit().read()
+        if 'href="/my-account"' not in raw:
+            raise LoginFailed(
+                    _('Failed to log in, check your username and password for'
+                    ' the calibre Periodicals service.'))
+
+        return br
+
+    def download(self):
+        import cStringIO
+        self.log('Fetching downloaded recipe')
+        try:
+            raw = self.browser.open_novisit(
+                'http://news.calibre-ebook.com/subscribed_files/%s/0/temp.downloaded_recipe'
+                % self.calibre_periodicals_slug
+                    ).read()
+        except Exception, e:
+            if hasattr(e, 'getcode') and e.getcode() == 403:
+                raise DownloadDenied(
+                        _('You do not have permission to download this issue.'
+                        ' Either your subscription has expired or you have'
+                        ' exceeded the maximum allowed downloads for today.'))
+            raise
+        f = cStringIO.StringIO(raw)
+        from calibre.utils.zipfile import ZipFile
+        zf = ZipFile(f)
+        zf.extractall()
+        zf.close()
+        from calibre.web.feeds.recipes import compile_recipe
+        from glob import glob
+        try:
+            recipe = compile_recipe(open(glob('*.recipe')[0],
+                'rb').read())
+            self.conversion_options = recipe.conversion_options
+        except:
+            self.log.exception('Failed to compile downloaded recipe')
+        return os.path.abspath('index.html')
+
